@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -42,10 +42,11 @@
 #include "weechat.h"
 #include "backtrace.h"
 #include "log.h"
+#include "util.h"
 
 
 /*
- * weechat_backtrace_printf: display a backtrage line (on stderr and in WeeChat log)
+ * weechat_backtrace_printf: display a backtrace line (on stderr and in WeeChat log)
  */
 
 void
@@ -58,7 +59,7 @@ weechat_backtrace_printf (char *message, ...)
     vsnprintf (buffer, sizeof (buffer) - 1, message, argptr);
     va_end (argptr);
 
-    fprintf (stderr, "%s", buffer);
+    weechat_iconv_fprintf (stderr, "%s", buffer);
     weechat_log_printf ("%s", buffer);
 }
 
@@ -88,7 +89,7 @@ weechat_backtrace_addr2line (int number, void *address, char *symbol)
     
     addr = address;
     if (info.dli_fbase >= (const void *) 0x40000000)
-        addr = (const char *) addr - (unsigned int) info.dli_fbase;
+        addr = (void *)((unsigned long)((const char *) addr) - (unsigned long) info.dli_fbase);
     
     snprintf (cmd_line, sizeof (cmd_line),
               "addr2line --functions --demangle -e $(which %s) %p",
