@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,10 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <dirent.h>
+#include <unistd.h>
 
 #include "weechat.h"
 #include "completion.h"
@@ -61,6 +65,7 @@ completion_init (t_completion *completion, void *server, void *channel)
     completion->position = -1; 
     completion->args = NULL;
     completion->direction = 0;
+    completion->add_space = 1;
     
     completion->completion_list = NULL;
     completion->last_completion = NULL;
@@ -154,7 +159,7 @@ completion_get_command_infos (t_completion *completion,
         for (ptr_handler = ptr_plugin->handlers;
              ptr_handler; ptr_handler = ptr_handler->next_handler)
         {
-            if ((ptr_handler->type == HANDLER_COMMAND)
+            if ((ptr_handler->type == PLUGIN_HANDLER_COMMAND)
                 && (ascii_strcasecmp (ptr_handler->command,
                                       ptr_command2) == 0))
             {
@@ -283,8 +288,90 @@ completion_list_add_server_channels (t_completion *completion)
 void
 completion_list_add_filename (t_completion *completion)
 {
-    /* TODO: add filename completion */
-    completion_stop (completion);
+    char *path_d, *path_b, *p, *d_name;
+    char *real_prefix, *prefix;
+    char *buffer;
+    int buffer_len;
+    DIR *dp;
+    struct dirent *entry;
+    struct stat statbuf;
+    char home[3] = { '~', DIR_SEPARATOR_CHAR, '\0' };
+    
+    buffer_len = PATH_MAX;
+    buffer = (char *) malloc (buffer_len * sizeof (char));
+    if (!buffer)
+	return;
+    
+    completion->add_space = 0;
+    
+    if ((strncmp (completion->base_word, home, 2) == 0) && getenv("HOME"))
+    {
+	real_prefix = strdup (getenv("HOME"));
+	prefix = strdup (home);
+    }
+    else if ((strncmp (completion->base_word, DIR_SEPARATOR, 1) != 0)
+	     || (strcmp (completion->base_word, "") == 0))
+    {
+	real_prefix = strdup (weechat_home);
+	prefix = strdup ("");
+    }
+    else
+    {
+	real_prefix = strdup (DIR_SEPARATOR);
+	prefix = strdup (DIR_SEPARATOR);
+    }
+	
+    snprintf (buffer, buffer_len, "%s", completion->base_word + strlen (prefix));
+    p = strrchr (buffer, DIR_SEPARATOR_CHAR);
+    if (p)
+    {
+	*p = '\0';
+	path_d = strdup (buffer);
+	p++;
+	path_b = strdup (p);
+    }
+    else {
+	path_d = strdup ("");
+	path_b = strdup (buffer);
+    }
+    
+    sprintf (buffer, "%s%s%s", real_prefix, DIR_SEPARATOR, path_d);
+    d_name = strdup (buffer);
+    dp = opendir(d_name);
+    if (dp != NULL)
+    {
+	while((entry = readdir(dp)) != NULL) 
+	{	
+	    if (strncmp (entry->d_name, path_b, strlen(path_b)) == 0) {
+		
+		if (strcmp (entry->d_name, ".") == 0 || strcmp (entry->d_name, "..") == 0)
+		    continue;
+		
+		snprintf(buffer, buffer_len, "%s%s%s", 
+			 d_name, DIR_SEPARATOR, entry->d_name);
+		if (stat(buffer, &statbuf) == -1)
+		    continue;
+		
+		snprintf(buffer, buffer_len, "%s%s%s%s%s%s",
+			 prefix, 
+			 ((strcmp(prefix, "") == 0)
+			  || strchr(prefix, DIR_SEPARATOR_CHAR)) ? "" : DIR_SEPARATOR, 
+			 path_d, 
+			 strcmp(path_d, "") == 0 ? "" : DIR_SEPARATOR, 
+			 entry->d_name, 
+			 S_ISDIR(statbuf.st_mode) ? DIR_SEPARATOR : "");
+		
+		completion_list_add (completion, buffer);
+	    }
+	}
+    }
+    
+    free (d_name);
+    free (prefix);
+    free (real_prefix);
+    free (path_d);
+    free (path_b);
+    free (buffer);
 }
 
 /*
@@ -304,7 +391,7 @@ completion_list_add_plugin_cmd (t_completion *completion)
         for (ptr_handler = ptr_plugin->handlers;
              ptr_handler; ptr_handler = ptr_handler->next_handler)
         {
-            if (ptr_handler->type == HANDLER_COMMAND)
+            if (ptr_handler->type == PLUGIN_HANDLER_COMMAND)
                 completion_list_add (completion, ptr_handler->command);
         }
     }
@@ -582,7 +669,7 @@ completion_list_add_servers (t_completion *completion)
 void
 completion_list_add_topic (t_completion *completion)
 {
-    char *string, *string2;
+    char *string;
     
     if (!completion->server || !completion->channel
         || !((t_irc_channel *)(completion->channel))->topic
@@ -594,16 +681,11 @@ completion_list_add_topic (t_completion *completion)
             string = (char *)gui_color_decode_for_user_entry ((unsigned char *)((t_irc_channel *)(completion->channel))->topic);
         else
             string = (char *)gui_color_decode ((unsigned char *)((t_irc_channel *)(completion->channel))->topic, 0);
-        string2 = channel_iconv_decode ((t_irc_server *)(completion->server),
-                                        (t_irc_channel *)(completion->channel),
-                                        (string) ? string : ((t_irc_channel *)(completion->channel))->topic);
         completion_list_add (completion,
-                             (string2) ? string2 : ((string) ?
-                                                    string : ((t_irc_channel *)(completion->channel))->topic));
+                             (string) ?
+                             string : ((t_irc_channel *)(completion->channel))->topic);
         if (string)
             free (string);
-        if (string2)
-            free (string2);
     }
 }
 
@@ -786,6 +868,7 @@ completion_build_list_template (t_completion *completion, char *template)
                             break;
                         case 'n': /* channel nicks */
                             completion_list_add_channel_nicks (completion);
+                            completion->context = COMPLETION_NICK;
                             break;
                         case 'N': /* channel nicks and hosts */
                             completion_list_add_channel_nicks_hosts (completion);
@@ -887,6 +970,7 @@ completion_find_context (t_completion *completion, char *buffer, int size, int p
     
     /* look for context */
     completion_free (completion);
+    completion_init (completion, completion->server, completion->channel);
     command = ((buffer[0] == '/') && (buffer[1] != '/')) ? 1 : 0;
     command_arg = 0;
     i = 0;
@@ -917,12 +1001,7 @@ completion_find_context (t_completion *completion, char *buffer, int size, int p
         }
     }
     else
-    {
-        if (completion->channel)
-            completion->context = COMPLETION_NICK;
-        else
-            completion->context = COMPLETION_NULL;
-    }
+        completion->context = COMPLETION_AUTO;
     
     /* look for word to complete (base word) */
     completion->base_word_pos = 0;
@@ -995,29 +1074,14 @@ completion_find_context (t_completion *completion, char *buffer, int size, int p
         }
     }
     
-    /* nick completion with nothing as base word is disabled,
+    /* auto completion with nothing as base word is disabled,
        in order to prevent completion when pasting messages with [tab] inside */
-    if ((completion->context == COMPLETION_NICK)
+    if ((completion->context == COMPLETION_AUTO)
         && ((!completion->base_word) || (!completion->base_word[0])))
     {
         completion->context = COMPLETION_NULL;
         return;
     }
-    
-    if (!completion->completion_list && completion->channel &&
-        ((((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_PRIVATE)
-         || (((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_DCC_CHAT))
-        && (completion->context == COMPLETION_NICK))
-    {
-        /* nick completion in private (only other nick and self) */
-        completion->context = COMPLETION_NICK;
-        weelist_add (&completion->completion_list,
-                     &completion->last_completion,
-                     ((t_irc_channel *)(completion->channel))->name);
-        weelist_add (&completion->completion_list,
-                     &completion->last_completion,
-                     ((t_irc_server *)(completion->server))->nick);
-    }
 }
 
 /*
@@ -1240,6 +1304,8 @@ completion_nick (t_completion *completion)
     if (!completion->channel)
         return;
     
+    completion->context = COMPLETION_NICK;
+    
     if ((((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_PRIVATE)
         || (((t_irc_channel *)(completion->channel))->type == CHANNEL_TYPE_DCC_CHAT))
     {
@@ -1323,6 +1389,39 @@ completion_nick (t_completion *completion)
     }
 }
 
+/*
+ * completion_auto: auto complete: nick, filename or channel
+ */
+
+void
+completion_auto (t_completion *completion)
+{
+    /* filename completion */
+    if ((completion->base_word[0] == '/')
+        || (completion->base_word[0] == '~'))
+    {
+        if (!completion->completion_list)
+            completion_list_add_filename (completion);
+        completion_command_arg (completion, 0);
+        return;
+    }
+    
+    /* channel completion */
+    if (string_is_channel (completion->base_word))
+    {
+        if (!completion->completion_list)
+            completion_list_add_server_channels (completion);
+        completion_command_arg (completion, 0);
+        return;
+    }
+    
+    /* default: nick completion (if channel) */
+    if (completion->channel)
+        completion_nick (completion);
+    else
+        completion->context = COMPLETION_NULL;
+}
+
 /*
  * completion_search: complete word according to context
  */
@@ -1350,10 +1449,7 @@ completion_search (t_completion *completion, int direction,
             /* should never be executed */
             return;
         case COMPLETION_NICK:
-            if (completion->channel)
-                completion_nick (completion);
-            else
-                return;
+            completion_nick (completion);
             break;
         case COMPLETION_COMMAND:
             completion_command (completion);
@@ -1362,7 +1458,13 @@ completion_search (t_completion *completion, int direction,
             if (completion->completion_list)
                 completion_command_arg (completion, completion->arg_is_nick);
             else
-                completion_nick (completion);
+            {
+                completion->context = COMPLETION_AUTO;
+                completion_auto (completion);
+            }
+            break;
+        case COMPLETION_AUTO:
+            completion_auto (completion);
             break;
     }
     if (completion->word_found)
@@ -1408,6 +1510,7 @@ completion_print_log (t_completion *completion)
     weechat_log_printf ("  position . . . . . . . : %d\n",   completion->position);
     weechat_log_printf ("  args . . . . . . . . . : '%s'\n", completion->args);
     weechat_log_printf ("  direction. . . . . . . : %d\n",   completion->direction);
+    weechat_log_printf ("  add_space. . . . . . . : %d\n",   completion->add_space);
     weechat_log_printf ("  completion_list. . . . : 0x%X\n", completion->completion_list);
     weechat_log_printf ("  last_completion. . . . : 0x%X\n", completion->last_completion);
     weechat_log_printf ("  word_found . . . . . . : '%s'\n", completion->word_found);
