@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #define COMPLETION_NICK         1
 #define COMPLETION_COMMAND      2
 #define COMPLETION_COMMAND_ARG  3
+#define COMPLETION_AUTO         4
 
 typedef struct t_completion t_completion;
 
@@ -44,6 +45,7 @@ struct t_completion
     int position;               /* position where Tab was pressed             */
     char *args;                 /* command line args (including base word)    */
     int direction;              /* +1 = search next word, -1 = previous word  */
+    int add_space;              /* add space after completion?                */
     
     /* for command argument completion */
     t_weelist *completion_list; /* data list for completion                   */
