@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,12 +28,17 @@
 #include <unistd.h>
 #include <string.h>
 #include <stdarg.h>
+
+#ifdef HAVE_FLOCK
 #include <sys/file.h>
+#endif
+
 #include <sys/types.h>
 #include <time.h>
 
 #include "weechat.h"
 #include "log.h"
+#include "util.h"
 
 
 char *weechat_log_filename = NULL; /* log name (~/.weechat/weechat.log)     */
@@ -95,11 +100,11 @@ weechat_log_init ()
 {
     if (!weechat_log_open (NULL, "w"))
     {
-        fprintf (stderr,
-                 _("%s unable to create/append to log file\n"
-                   "If another WeeChat process is using this file, try to run WeeChat\n"
-                   "with another home using \"--dir\" command line option.\n"),
-                 WEECHAT_ERROR);
+        weechat_iconv_fprintf (stderr,
+                               _("%s unable to create/append to log file\n"
+                                 "If another WeeChat process is using this file, try to run WeeChat\n"
+                                 "with another home using \"--dir\" command line option.\n"),
+                               WEECHAT_ERROR);
         exit (1);
     }
 }
@@ -138,12 +143,13 @@ weechat_log_printf (char *message, ...)
     seconds = time (NULL);
     date_tmp = localtime (&seconds);
     if (date_tmp)
-        fprintf (weechat_log_file, "[%04d-%02d-%02d %02d:%02d:%02d] %s",
-                 date_tmp->tm_year + 1900, date_tmp->tm_mon + 1, date_tmp->tm_mday,
-                 date_tmp->tm_hour, date_tmp->tm_min, date_tmp->tm_sec,
-                 buffer);
+        weechat_iconv_fprintf (weechat_log_file, "[%04d-%02d-%02d %02d:%02d:%02d] %s",
+                               date_tmp->tm_year + 1900, date_tmp->tm_mon + 1, date_tmp->tm_mday,
+                               date_tmp->tm_hour, date_tmp->tm_min, date_tmp->tm_sec,
+                               buffer);
     else
-        fprintf (weechat_log_file, "%s", buffer);
+        weechat_iconv_fprintf (weechat_log_file, "%s", buffer);
+    
     fflush (weechat_log_file);
 }
 
@@ -208,8 +214,8 @@ weechat_log_crash_rename ()
                   getpid());
         if (rename (old_name, new_name) == 0)
         {
-            fprintf (stderr, "*** Full crash dump was saved to %s file.\n",
-                     new_name);
+            weechat_iconv_fprintf (stderr, "*** Full crash dump was saved to %s file.\n",
+                                   new_name);
             weechat_log_open (new_name, "a");
             free (old_name);
             free (new_name);
