@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,8 @@
 #include "session.h"
 #include "hotlist.h"
 #include "log.h"
+#include "utf8.h"
+#include "util.h"
 #include "../irc/irc.h"
 #include "../gui/gui.h"
 
@@ -265,9 +267,6 @@ session_save_servers (FILE *file)
         rc = rc && (session_write_int (file, SESSION_SERV_LAG, ptr_server->lag));
         rc = rc && (session_write_buf (file, SESSION_SERV_LAG_CHECK_TIME, &(ptr_server->lag_check_time), sizeof (struct timeval)));
         rc = rc && (session_write_buf (file, SESSION_SERV_LAG_NEXT_CHECK, &(ptr_server->lag_next_check), sizeof (time_t)));
-        rc = rc && (session_write_str (file, SESSION_SERV_CHARSET_DECODE_ISO, ptr_server->charset_decode_iso));
-        rc = rc && (session_write_str (file, SESSION_SERV_CHARSET_DECODE_UTF, ptr_server->charset_decode_utf));
-        rc = rc && (session_write_str (file, SESSION_SERV_CHARSET_ENCODE, ptr_server->charset_encode));
         rc = rc && (session_write_id  (file, SESSION_SERV_END));
         
         if (!rc)
@@ -377,6 +376,7 @@ session_save_line (FILE *file, t_gui_line *line)
     rc = rc && (session_write_int (file, SESSION_LINE_OFS_AFTER_DATE, line->ofs_start_message));
     rc = rc && (session_write_int (file, SESSION_LINE_OFS_START_MESSAGE, line->ofs_start_message));
     rc = rc && (session_write_str (file, SESSION_LINE_NICK, line->nick));
+    rc = rc && (session_write_buf (file, SESSION_LINE_DATE, &(line->date), sizeof (time_t)));
     rc = rc && (session_write_id  (file, SESSION_LINE_END));
     return rc;
 }
@@ -509,21 +509,21 @@ session_crash (FILE *file, char *message, ...)
     
     fclose (file);
     gui_main_end ();
-    fprintf (stderr, "%s %s\n",
-             WEECHAT_ERROR, buffer);
-    fprintf (stderr,
-             _("Last operation with session file was at position %ld, "
-               "read of %d bytes\n"),
-             session_last_read_pos,
-             session_last_read_length);
-    fprintf (stderr,
-             _("Please send %s/%s, %s/%s and "
-               "above messages to WeeChat developers for support.\n"
-               "Be careful, private info may be in these files.\n"),
-             weechat_home,
-             WEECHAT_LOG_NAME,
-             weechat_home,
-             WEECHAT_SESSION_NAME);
+    weechat_iconv_fprintf (stderr, "%s %s\n",
+                           WEECHAT_ERROR, buffer);
+    weechat_iconv_fprintf (stderr,
+                           _("Last operation with session file was at position %ld, "
+                             "read of %d bytes\n"),
+                           session_last_read_pos,
+                           session_last_read_length);
+    weechat_iconv_fprintf (stderr,
+                           _("Please send %s/%s, %s/%s and "
+                             "above messages to WeeChat developers for support.\n"
+                             "Be careful, private info may be in these files.\n"),
+                           weechat_home,
+                           WEECHAT_LOG_NAME,
+                           weechat_home,
+                           WEECHAT_SESSION_NAME);
     exit (EXIT_FAILURE);
 }
 
@@ -617,6 +617,22 @@ session_read_str (FILE *file, char **string)
     return 1;
 }
 
+/*
+ * session_read_str_utf8: read string from file, then normalize UTF-8
+ */
+
+int
+session_read_str_utf8 (FILE *file, char **string)
+{
+    int rc;
+
+    rc = session_read_str (file, string);
+    if (rc && *string)
+        utf8_normalize (*string, '?');
+    
+    return rc;
+}
+
 /*
  * session_read_buf: read buffer from file
  */
@@ -952,7 +968,7 @@ session_load_server (FILE *file)
                     gnutls_session_set_data (session_current_server->gnutls_sess, session_data, session_size);
                     free (session_data);
                     gnutls_transport_set_ptr (session_current_server->gnutls_sess,
-                                              (gnutls_transport_ptr) session_current_server->sock);
+                                              (gnutls_transport_ptr) ((unsigned long) session_current_server->sock));
                     if (gnutls_handshake (session_current_server->gnutls_sess) < 0)
                     {
                         session_crash (file, _("gnutls handshake failed"));
@@ -980,7 +996,7 @@ session_load_server (FILE *file)
                 rc = rc && (session_read_int (file, &(session_current_server->is_away)));
                 break;
             case SESSION_SERV_AWAY_MESSAGE:
-                rc = rc && (session_read_str (file, &(session_current_server->away_message)));
+                 rc = rc && (session_read_str (file, &(session_current_server->away_message)));
                 break;
             case SESSION_SERV_AWAY_TIME:
                 rc = rc && (session_read_buf (file, &(session_current_server->away_time), sizeof (time_t)));
@@ -994,14 +1010,14 @@ session_load_server (FILE *file)
             case SESSION_SERV_LAG_NEXT_CHECK:
                 rc = rc && (session_read_buf (file, &(session_current_server->lag_next_check), sizeof (time_t)));
                 break;
-            case SESSION_SERV_CHARSET_DECODE_ISO:
-                rc = rc && (session_read_str (file, &(session_current_server->charset_decode_iso)));
+            case SESSION_SERV_CHARSET_DECODE_ISO__UNUSED:
+                rc = rc && (session_read_ignore_value (file));
                 break;
-            case SESSION_SERV_CHARSET_DECODE_UTF:
-                rc = rc && (session_read_str (file, &(session_current_server->charset_decode_utf)));
+            case SESSION_SERV_CHARSET_DECODE_UTF__UNUSED:
+                rc = rc && (session_read_ignore_value (file));
                 break;
-            case SESSION_SERV_CHARSET_ENCODE:
-                rc = rc && (session_read_str (file, &(session_current_server->charset_encode)));
+            case SESSION_SERV_CHARSET_ENCODE__UNUSED:
+                rc = rc && (session_read_ignore_value (file));
                 break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
@@ -1075,7 +1091,7 @@ session_load_channel (FILE *file)
             case SESSION_CHAN_END:
                 return 1;
             case SESSION_CHAN_TOPIC:
-                rc = rc && (session_read_str (file, &(session_current_channel->topic)));
+                rc = rc && (session_read_str_utf8 (file, &(session_current_channel->topic)));
                 break;
             case SESSION_CHAN_MODES:
                 rc = rc && (session_read_str (file, (char **)(&(session_current_channel->modes))));
@@ -1384,7 +1400,7 @@ session_load_history (FILE *file)
                 return 1;
             case SESSION_HIST_TEXT:
                 text = NULL;
-                if (!session_read_str (file, &text))
+                if (!session_read_str_utf8 (file, &text))
                     return 0;
                 if (session_current_buffer)
                     history_buffer_add (session_current_buffer, text);
@@ -1524,7 +1540,7 @@ session_load_line (FILE *file)
     }
     
     /* allocate line */
-    line = gui_buffer_line_new (session_current_buffer);
+    line = gui_buffer_line_new (session_current_buffer, time (NULL));
     if (!line)
     {
         session_crash (file, _("can't create new line"));
@@ -1562,7 +1578,7 @@ session_load_line (FILE *file)
                 rc = rc && (session_read_int (file, &(line->line_with_highlight)));
                 break;
             case SESSION_LINE_DATA:
-                rc = rc && (session_read_str (file, &(line->data)));
+                rc = rc && (session_read_str_utf8 (file, &(line->data)));
                 break;
             case SESSION_LINE_OFS_AFTER_DATE:
                 rc = rc && (session_read_int (file, &(line->ofs_after_date)));
@@ -1573,6 +1589,9 @@ session_load_line (FILE *file)
             case SESSION_LINE_NICK:
                 rc = rc && (session_read_str (file, &(line->nick)));
                 break;
+            case SESSION_LINE_DATE:
+                rc = rc && (session_read_buf (file, &(line->date), sizeof (time_t)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
                                       "line (object id: %d)\n"),
