@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,13 +21,18 @@
 #ifndef __WEECHAT_UTIL_H
 #define __WEECHAT_UTIL_H 1
 
+#ifndef HAVE_STRNDUP
+extern char *strndup (char *, int);
+#endif
 extern void ascii_tolower (char *);
 extern void ascii_toupper (char *);
 extern int ascii_strcasecmp (char *, char *);
 extern int ascii_strncasecmp (char *, char *, int);
 extern char *ascii_strcasestr (char *, char *);
 extern char *weechat_iconv (char *, char *, char *);
-extern int weechat_iconv_check (char *, char *);
+extern char *weechat_iconv_to_internal (char *, char *);
+extern char *weechat_iconv_from_internal (char *, char *);
+extern void weechat_iconv_fprintf (FILE *, char *, ...);
 extern char *weechat_strreplace (char *, char *, char *);
 extern long get_timeval_diff (struct timeval *, struct timeval *);
 extern char **explode_string (char *, char *, int, int *);
