@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -92,10 +92,6 @@ extern int cfg_look_set_title;
 extern int cfg_look_startup_logo;
 extern int cfg_look_startup_version;
 extern char *cfg_look_weechat_slogan;
-extern char *cfg_look_charset_decode_iso;
-extern char *cfg_look_charset_decode_utf;
-extern char *cfg_look_charset_encode;
-extern char *cfg_look_charset_internal;
 extern int cfg_look_one_server_buffer;
 extern int cfg_look_open_near_server;
 extern int cfg_look_scroll_amount;
@@ -106,6 +102,7 @@ extern int cfg_look_nicklist;
 extern int cfg_look_nicklist_position;
 extern int cfg_look_nicklist_min_size;
 extern int cfg_look_nicklist_max_size;
+extern int cfg_look_nicklist_separator;
 extern int cfg_look_nickmode;
 extern int cfg_look_nickmode_empty;
 extern char *cfg_look_no_nickname;
@@ -133,6 +130,7 @@ extern char *cfg_look_input_format;
 extern int cfg_col_real_white;
 extern int cfg_col_separator;
 extern int cfg_col_title;
+extern int cfg_col_title_more;
 extern int cfg_col_title_bg;
 extern int cfg_col_chat;
 extern int cfg_col_chat_time;
@@ -212,10 +210,12 @@ extern int cfg_irc_away_check_max_nicks;
 extern int cfg_irc_lag_check;
 extern int cfg_irc_lag_min_show;
 extern int cfg_irc_lag_disconnect;
+extern int cfg_irc_anti_flood;
 extern int cfg_irc_fifo_pipe;
 extern char *cfg_irc_highlight;
 extern int cfg_irc_colors_receive;
 extern int cfg_irc_colors_send;
+extern int cfg_irc_send_unknown_commands;
 
 extern int cfg_dcc_auto_accept_files;
 extern int cfg_dcc_auto_accept_chats;
