@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include "../../common/weechat.h"
 #include "../gui.h"
 #include "../../common/utf8.h"
+#include "../../common/util.h"
 #include "../../common/weeconfig.h"
 #include "../../irc/irc.h"
 #include "gui-curses.h"
@@ -167,7 +168,7 @@ void
 gui_chat_draw_title (t_gui_buffer *buffer, int erase)
 {
     t_gui_window *ptr_win;
-    char format[32], *buf, *buf2;
+    char format[32], *buf, *buf2, *ptr_topic;
     
     if (!gui_ok)
         return;
@@ -179,37 +180,68 @@ gui_chat_draw_title (t_gui_buffer *buffer, int erase)
             if (erase)
                 gui_window_curses_clear (GUI_CURSES(ptr_win)->win_title, COLOR_WIN_TITLE);
             
-            gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_title, COLOR_WIN_TITLE);
             snprintf (format, 32, "%%-%ds", ptr_win->win_title_width);
+            wmove (GUI_CURSES(ptr_win)->win_title, 0, 0);
+            
             if (CHANNEL(buffer))
             {
                 if (CHANNEL(buffer)->topic)
                 {
                     buf = (char *)gui_color_decode ((unsigned char *)(CHANNEL(buffer)->topic), 0);
-                    buf2 = channel_iconv_decode (SERVER(buffer),
-                                                 CHANNEL(buffer),
-                                                 (buf) ? buf : CHANNEL(buffer)->topic);
-                    mvwprintw (GUI_CURSES(ptr_win)->win_title, 0, 0,
-                               format, (buf2) ? buf2 : CHANNEL(buffer)->topic);
+                    ptr_topic = utf8_add_offset ((buf) ? buf : CHANNEL(buffer)->topic,
+                                                 ptr_win->win_title_start);
+                    if (!ptr_topic || !ptr_topic[0])
+                    {
+                        ptr_win->win_title_start = 0;
+                        ptr_topic = (buf) ? buf : CHANNEL(buffer)->topic;
+                    }
+                    buf2 = weechat_iconv_from_internal (NULL,
+                                                        ptr_topic);
+                    
+                    if (ptr_win->win_title_start > 0)
+                    {
+                        gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_title,
+                                                      COLOR_WIN_TITLE_MORE);
+                        wprintw (GUI_CURSES(ptr_win)->win_title, "%s", "++");
+                    }
+                    
+                    if (utf8_width_screen (ptr_topic) > ptr_win->win_width)
+                    {
+                        gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_title, COLOR_WIN_TITLE);
+                        wprintw (GUI_CURSES(ptr_win)->win_title, "%s", (buf2) ? buf2 : ptr_topic);
+                        gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_title,
+                                                      COLOR_WIN_TITLE_MORE);
+                        mvwprintw (GUI_CURSES(ptr_win)->win_title, 0, ptr_win->win_width - 2,
+                                   "%s", "++");
+                    }
+                    else
+                    {
+                        gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_title, COLOR_WIN_TITLE);
+                        wprintw (GUI_CURSES(ptr_win)->win_title, "%s", (buf2) ? buf2 : ptr_topic);
+                    }
                     if (buf)
                         free (buf);
                     if (buf2)
                         free (buf2);
                 }
                 else
-                    mvwprintw (GUI_CURSES(ptr_win)->win_title, 0, 0, format, " ");
+                {
+                    gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_title, COLOR_WIN_TITLE);
+                    wprintw (GUI_CURSES(ptr_win)->win_title, format, " ");
+                }
             }
             else
             {
+                gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_title, COLOR_WIN_TITLE);
                 if (buffer->type == BUFFER_TYPE_STANDARD)
                 {
-                    mvwprintw (GUI_CURSES(ptr_win)->win_title, 0, 0,
-                               format,
-                               PACKAGE_STRING " " WEECHAT_COPYRIGHT_DATE " - "
-                               WEECHAT_WEBSITE);
+                    wprintw (GUI_CURSES(ptr_win)->win_title,
+                             format,
+                             PACKAGE_STRING " " WEECHAT_COPYRIGHT_DATE " - "
+                             WEECHAT_WEBSITE);
                 }
                 else
-                    mvwprintw (GUI_CURSES(ptr_win)->win_title, 0, 0, format, " ");
+                    wprintw (GUI_CURSES(ptr_win)->win_title, format, " ");
             }
             wnoutrefresh (GUI_CURSES(ptr_win)->win_title);
             refresh ();
@@ -217,6 +249,20 @@ gui_chat_draw_title (t_gui_buffer *buffer, int erase)
     }
 }
 
+/*
+ * gui_chat_get_real_width: return real width: width - 1 if nicklist is at right,
+ *                          for good copy/paste (without nicklist separator)
+ */
+
+int
+gui_chat_get_real_width (t_gui_window *window)
+{
+    if (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_RIGHT)
+        return window->win_chat_width - 1;
+    else
+        return window->win_chat_width;
+}
+
 /*
  * gui_chat_display_new_line: display a new line
  */
@@ -227,7 +273,8 @@ gui_chat_display_new_line (t_gui_window *window, int num_lines, int count,
 {
     if ((count == 0) || (*lines_displayed >= num_lines - count))
     {
-        if ((!simulate) && (window->win_chat_cursor_x <= window->win_chat_width - 1))
+        if ((!simulate)
+            && (window->win_chat_cursor_x <= gui_chat_get_real_width (window) - 1))
         {
             wmove (GUI_CURSES(window)->win_chat,
                    window->win_chat_cursor_y,
@@ -427,13 +474,14 @@ gui_chat_word_get_next_char (t_gui_window *window, unsigned char *string,
  */
 
 void
-gui_chat_display_word_raw (t_gui_window *window, char *string)
+gui_chat_display_word_raw (t_gui_window *window, char *string, int display)
 {
-    char *prev_char, *next_char, saved_char;
-    
-    wmove (GUI_CURSES(window)->win_chat,
-           window->win_chat_cursor_y,
-           window->win_chat_cursor_x);
+    char *prev_char, *next_char, saved_char, *output;
+
+    if (display)
+        wmove (GUI_CURSES(window)->win_chat,
+               window->win_chat_cursor_y,
+               window->win_chat_cursor_x);
     
     while (string && string[0])
     {
@@ -442,14 +490,19 @@ gui_chat_display_word_raw (t_gui_window *window, char *string)
             return;
         
         prev_char = utf8_prev_char (string, next_char);
-        if (prev_char)
+        if (display && prev_char)
         {
             saved_char = next_char[0];
             next_char[0] = '\0';
-            if ((prev_char[0] == -110) && (!prev_char[1]))
+            if (((unsigned char)(prev_char[0]) == 146) && (!prev_char[1]))
                 wprintw (GUI_CURSES(window)->win_chat, ".");
             else
-                wprintw (GUI_CURSES(window)->win_chat, "%s", prev_char);
+            {
+                output = weechat_iconv_from_internal (NULL, prev_char);
+                wprintw (GUI_CURSES(window)->win_chat, "%s", (output) ? output : prev_char);
+                if (output)
+                    free (output);
+            }
             next_char[0] = saved_char;
         }
         
@@ -510,24 +563,32 @@ gui_chat_display_word (t_gui_window *window,
         chars_to_display = gui_word_strlen (window, data);
 
         /* too long for current line */
-        if (window->win_chat_cursor_x + chars_to_display > window->win_chat_width)
+        if (window->win_chat_cursor_x + chars_to_display > gui_chat_get_real_width (window))
         {
-            num_displayed = window->win_chat_width - window->win_chat_cursor_x;
+            num_displayed = gui_chat_get_real_width (window) - window->win_chat_cursor_x;
             pos_saved_char = gui_word_real_pos (window, data, num_displayed);
             saved_char = data[pos_saved_char];
             data[pos_saved_char] = '\0';
-            if ((!simulate) &&
-                ((count == 0) || (*lines_displayed >= num_lines - count)))
-                gui_chat_display_word_raw (window, data);
+            if (!simulate)
+            {
+                if ((count == 0) || (*lines_displayed >= num_lines - count))
+                    gui_chat_display_word_raw (window, data, 1);
+                else
+                    gui_chat_display_word_raw (window, data, 0);
+            }
             data[pos_saved_char] = saved_char;
             data += pos_saved_char;
         }
         else
         {
             num_displayed = chars_to_display;
-            if ((!simulate) &&
-                ((count == 0) || (*lines_displayed >= num_lines - count)))
-                gui_chat_display_word_raw (window, data);
+            if (!simulate)
+            {
+                if ((count == 0) || (*lines_displayed >= num_lines - count))
+                    gui_chat_display_word_raw (window, data, 1);
+                else
+                    gui_chat_display_word_raw (window, data, 0);
+            }
             data += strlen (data);
         }
         
@@ -537,7 +598,7 @@ gui_chat_display_word (t_gui_window *window,
         if ((data >= end_line) ||
             (((simulate) ||
              (window->win_chat_cursor_y <= window->win_chat_height - 1)) &&
-            (window->win_chat_cursor_x > (window->win_chat_width - 1))))
+            (window->win_chat_cursor_x > (gui_chat_get_real_width (window) - 1))))
             gui_chat_display_new_line (window, num_lines, count,
                                        lines_displayed, simulate);
         
@@ -681,8 +742,8 @@ gui_chat_display_line (t_gui_window *window, t_gui_line *line, int count,
         if (word_length > 0)
         {
             /* spaces + word too long for current line but ok for next line */
-            if ((window->win_chat_cursor_x + word_length_with_spaces > window->win_chat_width)
-                && (word_length <= window->win_chat_width - line->length_align))
+            if ((window->win_chat_cursor_x + word_length_with_spaces > gui_chat_get_real_width (window))
+                && (word_length <= gui_chat_get_real_width (window) - line->length_align))
             {
                 gui_chat_display_new_line (window, num_lines, count,
                                            &lines_displayed, simulate);
@@ -921,12 +982,14 @@ gui_chat_draw (t_gui_buffer *buffer, int erase)
                     mvwprintw (GUI_CURSES(ptr_win)->win_chat, i, 0, "%s %-16s ",
                                (ptr_dcc == dcc_selected) ? "***" : "   ",
                                ptr_dcc->nick);
-                    buf = channel_iconv_decode (SERVER(buffer),
-                                                CHANNEL(buffer),
-                                                (DCC_IS_CHAT(ptr_dcc->type)) ?
-                                                _(ptr_dcc->filename) : ptr_dcc->filename);
-                    wprintw (GUI_CURSES(ptr_win)->win_chat, "%s", buf);
-                    free (buf);
+                    buf = weechat_iconv_from_internal (NULL,
+                                                       (DCC_IS_CHAT(ptr_dcc->type)) ?
+                                                       _(ptr_dcc->filename) : ptr_dcc->filename);
+                    wprintw (GUI_CURSES(ptr_win)->win_chat, "%s",
+                             (buf) ? buf : ((DCC_IS_CHAT(ptr_dcc->type)) ?
+                             _(ptr_dcc->filename) : ptr_dcc->filename));
+                    if (buf)
+                        free (buf);
                     if (DCC_IS_FILE(ptr_dcc->type))
                     {
                         if (ptr_dcc->filename_suffix > 0)
@@ -943,11 +1006,11 @@ gui_chat_draw (t_gui_buffer *buffer, int erase)
                                (DCC_IS_RECV(ptr_dcc->type)) ? "-->>" : "<<--");
                     gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_chat,
                                                   COLOR_DCC_WAITING + ptr_dcc->status);
-                    buf = channel_iconv_decode (SERVER(buffer),
-                                                CHANNEL(buffer),
-                                                _(dcc_status_string[ptr_dcc->status]));
-                    wprintw (GUI_CURSES(ptr_win)->win_chat, "%-10s", buf);
-                    free (buf);
+                    buf = weechat_iconv_from_internal (NULL, _(dcc_status_string[ptr_dcc->status]));
+                    wprintw (GUI_CURSES(ptr_win)->win_chat, "%-10s",
+                             (buf) ? buf : _(dcc_status_string[ptr_dcc->status]));
+                    if (buf)
+                        free (buf);
                     
                     /* other infos */
                     gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_chat,
@@ -1016,13 +1079,12 @@ gui_chat_draw (t_gui_buffer *buffer, int erase)
                                      ptr_dcc->eta % 60);
                         }
                         sprintf (format, "%s %%s/s)", unit_format[num_unit]);
-                        buf = channel_iconv_decode (SERVER(buffer),
-                                                    CHANNEL(buffer),
-                                                    unit_name[num_unit]);
+                        buf = weechat_iconv_from_internal (NULL, unit_name[num_unit]);
                         wprintw (GUI_CURSES(ptr_win)->win_chat, format,
                                  ((float)ptr_dcc->bytes_per_sec) / ((float)(unit_divide[num_unit])),
-                                 buf);
-                        free (buf);
+                                 (buf) ? buf : unit_name[num_unit]);
+                        if (buf)
+                            free (buf);
                     }
                     else
                     {
