@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@
 #include "../../common/weechat.h"
 #include "../gui.h"
 #include "../../common/utf8.h"
+#include "../../common/util.h"
 #include "../../common/weeconfig.h"
 #include "gui-curses.h"
 
@@ -89,11 +90,11 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
                 {
                     case 'c':
                         if (CHANNEL(window->buffer))
-                            length += utf8_strlen (CHANNEL(window->buffer)->name);
+                            length += utf8_width_screen (CHANNEL(window->buffer)->name);
                         else
                         {
                             if (SERVER(window->buffer))
-                                length += utf8_strlen (SERVER(window->buffer)->name);
+                                length += utf8_width_screen (SERVER(window->buffer)->name);
                         }
                         pos++;
                         break;
@@ -107,7 +108,7 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
                         pos++;
                         break;
                     case 'n':
-                        length += utf8_strlen (nick);
+                        length += utf8_width_screen (nick);
                         pos++;
                         break;
                     default:
@@ -147,7 +148,7 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
 void
 gui_input_draw_prompt (t_gui_window *window, char *nick)
 {
-    char *pos, saved_char;
+    char *pos, saved_char, *buf;
     int char_size;
     
     wmove (GUI_CURSES(window)->win_input, 0, 0);
@@ -165,8 +166,12 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                         {
                             gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
                                                           COLOR_WIN_INPUT_CHANNEL);
+                            buf = weechat_iconv_from_internal (NULL,
+                                                               CHANNEL(window->buffer)->name);
                             wprintw (GUI_CURSES(window)->win_input, "%s",
-                                     CHANNEL(window->buffer)->name);
+                                     (buf) ? buf : CHANNEL(window->buffer)->name);
+                            if (buf)
+                                free (buf);
                         }
                         else
                         {
@@ -174,8 +179,12 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                             {
                                 gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
                                                               COLOR_WIN_INPUT_SERVER);
+                                buf = weechat_iconv_from_internal (NULL,
+                                                                   SERVER(window->buffer)->name);
                                 wprintw (GUI_CURSES(window)->win_input, "%s",
-                                         SERVER(window->buffer)->name);
+                                         (buf) ? buf : SERVER(window->buffer)->name);
+                                if (buf)
+                                    free (buf);
                             }
                         }
                         pos++;
@@ -197,7 +206,10 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                     case 'n':
                         gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
                                                       COLOR_WIN_INPUT_NICK);
-                        wprintw (GUI_CURSES(window)->win_input, "%s", nick);
+                        buf = weechat_iconv_from_internal (NULL, nick);
+                        wprintw (GUI_CURSES(window)->win_input, "%s", (buf) ? buf : nick);
+                        if (buf)
+                            free (buf);
                         pos++;
                         break;
                     default:
@@ -243,7 +255,7 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
 int
 gui_input_draw_text (t_gui_window *window, int input_width)
 {
-    char *ptr_start, *ptr_next, saved_char;
+    char *ptr_start, *ptr_next, saved_char, *output;
     int pos_mask, size, last_color, color, count_cursor, offset_cursor;
     
     ptr_start = utf8_add_offset (window->buffer->input_buffer,
@@ -273,7 +285,10 @@ gui_input_draw_text (t_gui_window *window, int input_width)
                     gui_input_set_color (window, color);
             }
             last_color = color;
-            wprintw (GUI_CURSES(window)->win_input, "%s", ptr_start);
+            output = weechat_iconv_from_internal (NULL, ptr_start);
+            wprintw (GUI_CURSES(window)->win_input, "%s", (output) ? output : ptr_start);
+            if (output)
+                free (output);
             if (count_cursor > 0)
             {
                 offset_cursor += utf8_width_screen (ptr_start);
@@ -387,22 +402,28 @@ gui_input_draw (t_gui_buffer *buffer, int erase)
                         {
                             case DCC_WAITING:
                                 if (DCC_IS_RECV(dcc_selected->type))
-                                    wprintw (GUI_CURSES(ptr_win)->win_input, _("  [A] Accept"));
-                                wprintw (GUI_CURSES(ptr_win)->win_input, _("  [C] Cancel"));
+                                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                        _("  [A] Accept"));
+                                gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                    _("  [C] Cancel"));
                                 break;
                             case DCC_CONNECTING:
                             case DCC_ACTIVE:
-                                wprintw (GUI_CURSES(ptr_win)->win_input, _("  [C] Cancel"));
+                                gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                    _("  [C] Cancel"));
                                 break;
                             case DCC_DONE:
                             case DCC_FAILED:
                             case DCC_ABORTED:
-                                wprintw (GUI_CURSES(ptr_win)->win_input, _("  [R] Remove"));
+                                gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                    _("  [R] Remove"));
                                 break;
                         }
                     }
-                    wprintw (GUI_CURSES(ptr_win)->win_input, _("  [P] Purge old DCC"));
-                    wprintw (GUI_CURSES(ptr_win)->win_input, _("  [Q] Close DCC view"));
+                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                        _("  [P] Purge old DCC"));
+                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                        _("  [Q] Close DCC view"));
                     wclrtoeol (GUI_CURSES(ptr_win)->win_input);
                     ptr_win->win_input_cursor_x = 0;
                     if (ptr_win == gui_current_window)
@@ -410,8 +431,10 @@ gui_input_draw (t_gui_buffer *buffer, int erase)
                     break;
                 case BUFFER_TYPE_RAW_DATA:
                     wmove (GUI_CURSES(ptr_win)->win_input, 0, 0);
-                    wprintw (GUI_CURSES(ptr_win)->win_input, _("  [C] Clear buffer"));
-                    wprintw (GUI_CURSES(ptr_win)->win_input, _("  [Q] Close raw data view"));
+                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                        _("  [C] Clear buffer"));
+                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                        _("  [Q] Close raw data view"));
                     wclrtoeol (GUI_CURSES(ptr_win)->win_input);
                     ptr_win->win_input_cursor_x = 0;
                     if (ptr_win == gui_current_window)
