@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@
 #include "../../common/weechat.h"
 #include "../gui.h"
 #include "../../common/utf8.h"
+#include "../../common/util.h"
 #include "gui-curses.h"
 
 #ifdef PLUGINS
@@ -82,7 +83,8 @@ gui_keyboard_default_bindings ()
     gui_keyboard_bind ( /* m-pgdn      */ "meta-meta2-6~",      "scroll_down");
     gui_keyboard_bind ( /* m-home      */ "meta-meta2-1~",      "scroll_top");
     gui_keyboard_bind ( /* m-end       */ "meta-meta2-4~",      "scroll_bottom");
-    gui_keyboard_bind ( /* F10         */ "meta2-21~",          "infobar_clear");
+    gui_keyboard_bind ( /* F9          */ "meta2-20~",          "scroll_topic_left");
+    gui_keyboard_bind ( /* F10         */ "meta2-21~",          "scroll_topic_right");
     gui_keyboard_bind ( /* F11         */ "meta2-23~",          "nick_page_up");
     gui_keyboard_bind ( /* F12         */ "meta2-24~",          "nick_page_down");
     gui_keyboard_bind ( /* m-F11       */ "meta-meta2-23~",     "nick_beginning");
@@ -95,6 +97,7 @@ gui_keyboard_default_bindings ()
     gui_keyboard_bind ( /* m-f         */ "meta-f",             "next_word");
     gui_keyboard_bind ( /* ^right      */ "meta-Oc",            "next_word");
     gui_keyboard_bind ( /* m-h         */ "meta-h",             "hotlist_clear");
+    gui_keyboard_bind ( /* m-i         */ "meta-i"   ,          "infobar_clear");
     gui_keyboard_bind ( /* m-j,m-d     */ "meta-jmeta-d",       "jump_dcc");
     gui_keyboard_bind ( /* m-j,m-l     */ "meta-jmeta-l",       "jump_last_buffer");
     gui_keyboard_bind ( /* m-j,m-s     */ "meta-jmeta-s",       "jump_server");
@@ -198,7 +201,7 @@ void
 gui_keyboard_read ()
 {
     int key, i, insert_ok;
-    char key_str[32];
+    char key_str[32], *key_utf;
     char *buffer_before_key;
 #ifdef PLUGINS
     char key_str2[33];
@@ -281,6 +284,14 @@ gui_keyboard_read ()
             {
                 key_str[0] = (char) key;
                 key_str[1] = '\0';
+                
+                /* convert input to UTF-8 is user is not using UTF-8 as locale */
+                if (!local_utf8)
+                {
+                    key_utf = weechat_iconv_to_internal (NULL, key_str);
+                    strncpy (key_str, key_utf, sizeof (key_str));
+                    key_str[sizeof (key_str) - 1] = '\0';
+                }
             }
         }
         
@@ -313,6 +324,8 @@ gui_keyboard_read ()
                                                          buffer_before_key,
                                                          gui_current_window->buffer->input_buffer);
 #endif
+                    if (buffer_before_key)
+                        free (buffer_before_key);
                     break;
                 case BUFFER_TYPE_DCC:
                     gui_exec_action_dcc (gui_current_window, key_str);
