@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -67,11 +67,10 @@ gui_main_loop ()
 {
     fd_set read_fd;
     static struct timeval timeout, tv;
-    static struct timezone tz;
     t_irc_server *ptr_server;
     t_gui_buffer *ptr_buffer;
     int old_day, old_min, old_sec, diff;
-    char text_time[1024];
+    char text_time[1024], *text_time2;
     time_t new_time;
     struct tm *local_time;
     
@@ -111,6 +110,7 @@ gui_main_loop ()
             {
                 strftime (text_time, sizeof (text_time),
                           cfg_look_day_change_timestamp, local_time);
+                text_time2 = weechat_iconv_to_internal (NULL, text_time);
                 gui_add_hotlist = 0;
                 for (ptr_buffer = gui_buffers; ptr_buffer;
                      ptr_buffer = ptr_buffer->next_buffer)
@@ -118,8 +118,11 @@ gui_main_loop ()
                     if (ptr_buffer->type == BUFFER_TYPE_STANDARD)
                         gui_printf_nolog_notime (ptr_buffer,
                                                  _("Day changed to %s\n"),
-                                                 text_time);
+                                                 (text_time2) ?
+                                                 text_time2 : text_time);
                 }
+                if (text_time2)
+                    free (text_time2);
                 gui_add_hotlist = 1;
             }
             old_day = local_time->tm_mday;
@@ -129,7 +132,18 @@ gui_main_loop ()
         if (local_time->tm_sec != old_sec)
         {
             old_sec = local_time->tm_sec;
+
+            /* send queued messages */
+            for (ptr_server = irc_servers; ptr_server;
+                 ptr_server = ptr_server->next_server)
+            {
+                if (ptr_server->is_connected)
+                {
+                    server_outqueue_send (ptr_server);
+                }
+            }
             
+            /* display time in infobar (if seconds displayed) */
             if (cfg_look_infobar_seconds)
             {
                 gui_infobar_draw_time (gui_current_window->buffer);
@@ -218,15 +232,15 @@ gui_main_loop ()
                     if ((ptr_server->lag_check_time.tv_sec == 0)
                         && (new_time >= ptr_server->lag_next_check))
                     {
-                        server_sendf (ptr_server, "PING %s\r\n", ptr_server->address);
-                        gettimeofday (&(ptr_server->lag_check_time), &tz);
+                        server_sendf (ptr_server, "PING %s", ptr_server->address);
+                        gettimeofday (&(ptr_server->lag_check_time), NULL);
                     }
                     
                     /* lag timeout => disconnect */
                     if ((ptr_server->lag_check_time.tv_sec != 0)
                         && (cfg_irc_lag_disconnect > 0))
                     {
-                        gettimeofday (&tv, &tz);
+                        gettimeofday (&tv, NULL);
                         diff = (int) get_timeval_diff (&(ptr_server->lag_check_time), &tv);
                         if (diff / 1000 > cfg_irc_lag_disconnect * 60)
                         {
