@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,8 +24,12 @@
 #include "config.h"
 #endif
 
+#include <stdlib.h>
+
 #include "../../common/weechat.h"
 #include "../gui.h"
+#include "../../common/utf8.h"
+#include "../../common/util.h"
 #include "../../common/weeconfig.h"
 #include "../../irc/irc.h"
 #include "gui-curses.h"
@@ -39,8 +43,8 @@ void
 gui_nicklist_draw (t_gui_buffer *buffer, int erase, int calculate_size)
 {
     t_gui_window *ptr_win;
-    int i, j, x, y, x2, column, max_length, nicks_displayed;
-    char format[32], format_empty[32];
+    int i, j, k, x, y, x2, max_y, column, max_length, max_chars, nicks_displayed;
+    char format_empty[32], *buf, *ptr_buf, *ptr_next, saved_char;
     t_irc_nick *ptr_nick;
     
     if (!gui_ok || !BUFFER_HAS_NICKLIST(buffer))
@@ -81,17 +85,16 @@ gui_nicklist_draw (t_gui_buffer *buffer, int erase, int calculate_size)
             
             if ((cfg_look_nicklist_position == CFG_LOOK_NICKLIST_TOP) ||
                 (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_BOTTOM))
-                snprintf (format, 32, "%%.%ds", max_length);
+                max_chars = max_length;
             else
-                snprintf (format, 32, "%%.%ds",
-                          ((cfg_look_nicklist_min_size > 0)
-                           && (max_length < cfg_look_nicklist_min_size)) ?
-                          cfg_look_nicklist_min_size :
-                          (((cfg_look_nicklist_max_size > 0)
-                            && (max_length > cfg_look_nicklist_max_size)) ?
-                           cfg_look_nicklist_max_size : max_length));
+                max_chars = ((cfg_look_nicklist_min_size > 0)
+                             && (max_length < cfg_look_nicklist_min_size)) ?
+                    cfg_look_nicklist_min_size :
+                    (((cfg_look_nicklist_max_size > 0)
+                      && (max_length > cfg_look_nicklist_max_size)) ?
+                     cfg_look_nicklist_max_size : max_length);
             
-            if (has_colors ())
+            if (cfg_look_nicklist_separator && has_colors ())
             {
                 gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_nick, COLOR_WIN_NICK_SEP);
                 switch (cfg_look_nicklist_position)
@@ -121,12 +124,26 @@ gui_nicklist_draw (t_gui_buffer *buffer, int erase, int calculate_size)
             
             gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_nick, COLOR_WIN_NICK);
             x = 0;
-            y = (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_BOTTOM) ? 1 : 0;
+            y = (cfg_look_nicklist_separator && (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_BOTTOM)) ? 1 : 0;
+            max_y = 0;
+            switch (cfg_look_nicklist_position)
+            {
+                case CFG_LOOK_NICKLIST_LEFT:
+                case CFG_LOOK_NICKLIST_RIGHT:
+                    max_y = 0;
+                    break;
+                case CFG_LOOK_NICKLIST_TOP:
+                    max_y = ptr_win->win_nick_height - cfg_look_nicklist_separator;
+                    break;
+                case CFG_LOOK_NICKLIST_BOTTOM:
+                    max_y = ptr_win->win_nick_height;
+                    break;
+            }
             column = 0;
             
             if ((cfg_look_nicklist_position == CFG_LOOK_NICKLIST_TOP) ||
                 (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_BOTTOM))
-                nicks_displayed = (ptr_win->win_width / (max_length + 2)) * (ptr_win->win_nick_height - 1);
+                nicks_displayed = (ptr_win->win_width / (max_length + 2)) * (ptr_win->win_nick_height - cfg_look_nicklist_separator);
             else
                 nicks_displayed = ptr_win->win_nick_height;
             
@@ -147,7 +164,7 @@ gui_nicklist_draw (t_gui_buffer *buffer, int erase, int calculate_size)
                             x = 0;
                             break;
                         case CFG_LOOK_NICKLIST_RIGHT:
-                            x = 1;
+                            x = cfg_look_nicklist_separator;
                             break;
                         case CFG_LOOK_NICKLIST_TOP:
                         case CFG_LOOK_NICKLIST_BOTTOM:
@@ -203,7 +220,30 @@ gui_nicklist_draw (t_gui_buffer *buffer, int erase, int calculate_size)
                         gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_nick,
                                                       ((cfg_irc_away_check > 0) && (ptr_nick->flags & NICK_AWAY)) ?
                                                       COLOR_WIN_NICK_AWAY : COLOR_WIN_NICK);
-                        mvwprintw (GUI_CURSES(ptr_win)->win_nick, y, x, format, ptr_nick->nick);
+                        wmove (GUI_CURSES(ptr_win)->win_nick, y, x);
+                        ptr_buf = ptr_nick->nick;
+                        saved_char = '\0';
+                        for (k = 0; k < max_chars; k++)
+                        {
+                            if (ptr_buf && ptr_buf[0])
+                            {
+                                ptr_next = utf8_next_char (ptr_buf);
+                                if (ptr_next)
+                                {
+                                    saved_char = ptr_next[0];
+                                    ptr_next[0] = '\0';
+                                }
+                                buf = weechat_iconv_from_internal (NULL, ptr_buf);
+                                wprintw (GUI_CURSES(ptr_win)->win_nick, "%s", (buf) ? buf : "?");
+                                if (buf)
+                                    free (buf);
+                                if (ptr_next)
+                                    ptr_next[0] = saved_char;
+                                ptr_buf = ptr_next;
+                            }
+                            else
+                                wprintw (GUI_CURSES(ptr_win)->win_nick, " ");
+                        }
                         
                         ptr_nick = ptr_nick->next_nick;
                         
@@ -214,11 +254,11 @@ gui_nicklist_draw (t_gui_buffer *buffer, int erase, int calculate_size)
                     if ((cfg_look_nicklist_position == CFG_LOOK_NICKLIST_TOP) ||
                         (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_BOTTOM))
                     {
-                        if (y - ((cfg_look_nicklist_position == CFG_LOOK_NICKLIST_BOTTOM) ? 1 : 0) >= ptr_win->win_nick_height - 1)
+                        if (y >= max_y)
                         {
                             column += max_length + 2;
-                            y = (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_TOP) ?
-                                0 : 1;
+                            y = (cfg_look_nicklist_separator && (cfg_look_nicklist_position == CFG_LOOK_NICKLIST_BOTTOM)) ?
+                                1 : 0;
                         }
                     }
                 }
