@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #endif
 
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <signal.h>
 #include <libgen.h>
@@ -33,6 +34,7 @@
 #include "../gui.h"
 #include "../../common/hotlist.h"
 #include "../../common/log.h"
+#include "../../common/util.h"
 #include "../../common/weeconfig.h"
 #include "gui-curses.h"
 
@@ -130,6 +132,27 @@ gui_window_objects_free (t_gui_window *window, int free_separator)
     }
 }
 
+/*
+ * gui_window_wprintw: decode then display string with wprintw
+ */
+
+void
+gui_window_wprintw (WINDOW *window, char *data, ...)
+{
+    va_list argptr;
+    static char buf[4096];
+    char *buf2;
+    
+    va_start (argptr, data);
+    vsnprintf (buf, sizeof (buf) - 1, data, argptr);
+    va_end (argptr);
+    
+    buf2 = weechat_iconv_from_internal (NULL, buf);
+    wprintw (window, "%s", (buf2) ? buf2 : buf);
+    if (buf2)
+        free (buf2);
+}
+
 /*
  * gui_window_curses_clear: clear a Curses window
  */
@@ -196,7 +219,7 @@ gui_window_calculate_pos_size (t_gui_window *window, int force_calculate)
             else if ((cfg_look_nicklist_max_size > 0)
                      && (max_length > cfg_look_nicklist_max_size))
                 max_length = cfg_look_nicklist_max_size;
-            if (!force_calculate && (window->win_nick_width == max_length + 2))
+            if (!force_calculate && (window->win_nick_width == max_length + 1 + cfg_look_nicklist_separator))
                 return 0;
         }
         else
@@ -218,19 +241,19 @@ gui_window_calculate_pos_size (t_gui_window *window, int force_calculate)
                 window->win_height - add_top - add_bottom - 2 - 4;
             if (lines > max_height)
                 lines = max_height;
-            if (!force_calculate && (window->win_nick_height == lines + 1))
+            if (!force_calculate && (window->win_nick_height == lines + cfg_look_nicklist_separator))
                 return 0;
         }
         
         switch (cfg_look_nicklist_position)
         {
             case CFG_LOOK_NICKLIST_LEFT:
-                window->win_chat_x = window->win_x + add_left + max_length + 2;
+                window->win_chat_x = window->win_x + add_left + max_length + 1 + cfg_look_nicklist_separator;
                 window->win_chat_y = window->win_y + add_top + 1;
-                window->win_chat_width = window->win_width - add_left - add_right - max_length - 2;
+                window->win_chat_width = window->win_width - add_left - add_right - max_length - 1 - cfg_look_nicklist_separator;
                 window->win_nick_x = window->win_x + add_left + 0;
                 window->win_nick_y = window->win_y + add_top + 1;
-                window->win_nick_width = max_length + 2;
+                window->win_nick_width = max_length + 1 + cfg_look_nicklist_separator;
                 if (cfg_look_infobar)
                 {
                     window->win_chat_height = window->win_height - add_top - add_bottom - 4;
@@ -246,10 +269,10 @@ gui_window_calculate_pos_size (t_gui_window *window, int force_calculate)
             case CFG_LOOK_NICKLIST_RIGHT:
                 window->win_chat_x = window->win_x + add_left;
                 window->win_chat_y = window->win_y + add_top + 1;
-                window->win_chat_width = window->win_width - add_left - add_right - max_length - 2;
-                window->win_nick_x = window->win_x + window->win_width - add_right - max_length - 2;
+                window->win_chat_width = window->win_width - add_left - add_right - max_length - 1 - cfg_look_nicklist_separator;
+                window->win_nick_x = window->win_x + window->win_width - add_right - max_length - 1 - cfg_look_nicklist_separator;
                 window->win_nick_y = window->win_y + add_top + 1;
-                window->win_nick_width = max_length + 2;
+                window->win_nick_width = max_length + 1 + cfg_look_nicklist_separator;
                 if (cfg_look_infobar)
                 {
                     window->win_chat_height = window->win_height - add_top - add_bottom - 4;
@@ -264,16 +287,16 @@ gui_window_calculate_pos_size (t_gui_window *window, int force_calculate)
                 break;
             case CFG_LOOK_NICKLIST_TOP:
                 window->win_chat_x = window->win_x + add_left;
-                window->win_chat_y = window->win_y + add_top + 1 + (lines + 1);
+                window->win_chat_y = window->win_y + add_top + 1 + lines + cfg_look_nicklist_separator;
                 window->win_chat_width = window->win_width - add_left - add_right;
                 if (cfg_look_infobar)
-                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - (lines + 1) - 1;
+                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - lines - cfg_look_nicklist_separator - 1;
                 else
-                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - (lines + 1);
+                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - lines - cfg_look_nicklist_separator;
                 window->win_nick_x = window->win_x + add_left;
                 window->win_nick_y = window->win_y + add_top + 1;
                 window->win_nick_width = window->win_width - add_left - add_right;
-                window->win_nick_height = lines + 1;
+                window->win_nick_height = lines + cfg_look_nicklist_separator;
                 window->win_nick_num_max = lines * (window->win_nick_width / (max_length + 2));
                 break;
             case CFG_LOOK_NICKLIST_BOTTOM:
@@ -281,16 +304,16 @@ gui_window_calculate_pos_size (t_gui_window *window, int force_calculate)
                 window->win_chat_y = window->win_y + add_top + 1;
                 window->win_chat_width = window->win_width - add_left - add_right;
                 if (cfg_look_infobar)
-                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - (lines + 1) - 1;
+                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - lines - cfg_look_nicklist_separator - 1;
                 else
-                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - (lines + 1);
+                    window->win_chat_height = window->win_height - add_top - add_bottom - 3 - lines - cfg_look_nicklist_separator;
                 window->win_nick_x = window->win_x + add_left;
                 if (cfg_look_infobar)
-                    window->win_nick_y = window->win_y + window->win_height - add_bottom - 2 - (lines + 1) - 1;
+                    window->win_nick_y = window->win_y + window->win_height - add_bottom - 2 - lines - cfg_look_nicklist_separator - 1;
                 else
-                    window->win_nick_y = window->win_y + window->win_height - add_bottom - 2 - (lines + 1);
+                    window->win_nick_y = window->win_y + window->win_height - add_bottom - 2 - lines - cfg_look_nicklist_separator;
                 window->win_nick_width = window->win_width - add_left - add_right;
-                window->win_nick_height = lines + 1;
+                window->win_nick_height = lines + cfg_look_nicklist_separator;
                 window->win_nick_num_max = lines * (window->win_nick_width / (max_length + 2));
                 break;
         }
@@ -429,6 +452,7 @@ gui_window_switch_to_buffer (t_gui_window *window, t_gui_buffer *buffer)
     }
     
     window->buffer = buffer;
+    window->win_title_start = 0;
     window->win_nick_start = 0;
     
     gui_window_calculate_pos_size (window, 1);
@@ -646,6 +670,37 @@ gui_window_scroll_bottom (t_gui_window *window)
     }
 }
 
+/*
+ * gui_window_scroll_topic_left: scroll left topic
+ */
+
+void
+gui_window_scroll_topic_left (t_gui_window *window)
+{
+    if (!gui_ok)
+        return;
+    
+    if (window->win_title_start > 0)
+        window->win_title_start -= (window->win_width * 3) / 4;
+    if (window->win_title_start < 0)
+        window->win_title_start = 0;
+    gui_chat_draw_title (window->buffer, 1);
+}
+
+/*
+ * gui_window_scroll_topic_right: scroll right topic
+ */
+
+void
+gui_window_scroll_topic_right (t_gui_window *window)
+{
+    if (!gui_ok)
+        return;
+    
+    window->win_title_start += (window->win_width * 3) / 4;
+    gui_chat_draw_title (window->buffer, 1);
+}
+
 /*
  * gui_window_nick_beginning: go to beginning of nicklist
  */
@@ -1227,8 +1282,8 @@ gui_window_set_title ()
 	if (strcmp( envterm, "sun-cmd") == 0)
 	    printf ("\033]l%s %s\033\\", PACKAGE_NAME, PACKAGE_VERSION);
 	else if (strcmp(envterm, "hpterm") == 0)
-	    printf ("\033&f0k%dD%s %s", strlen(PACKAGE_NAME) + 
-		    strlen(PACKAGE_VERSION) + 1,
+	    printf ("\033&f0k%dD%s %s",
+                    (int)(strlen(PACKAGE_NAME) + strlen(PACKAGE_VERSION) + 1),
 		    PACKAGE_NAME, PACKAGE_VERSION);
 	/* the following term supports the xterm excapes */
 	else if (strncmp (envterm, "xterm", 5) == 0
@@ -1262,7 +1317,7 @@ gui_window_reset_title ()
 	if (strcmp( envterm, "sun-cmd") == 0)
 	    printf ("\033]l%s\033\\", "Terminal");
 	else if (strcmp( envterm, "hpterm") == 0)
-	    printf ("\033&f0k%dD%s", strlen("Terminal"), "Terminal");
+	    printf ("\033&f0k%dD%s", (int)strlen("Terminal"), "Terminal");
 	/* the following term supports the xterm excapes */
 	else if (strncmp (envterm, "xterm", 5) == 0
 		 || strncmp (envterm, "rxvt", 4) == 0
