@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -408,13 +408,14 @@ gui_action_delete_line (t_gui_window *window)
 void
 gui_action_transpose_chars (t_gui_window *window)
 {
-    char *start, *prev_char, saved_char[4];
-    int size_current_char, size_start_char;
+    char *start, *prev_char, saved_char[5];
+    int size_prev_char, size_start_char;
     int pos_prev_char, pos_start;
     
     if (window->buffer->has_input)
     {
-        if (window->buffer->input_buffer_pos > 0)
+        if ((window->buffer->input_buffer_pos > 0)
+            && (window->buffer->input_buffer_length > 1))
         {
             if (window->buffer->input_buffer_pos == window->buffer->input_buffer_length)
                 window->buffer->input_buffer_pos--;
@@ -425,18 +426,18 @@ gui_action_transpose_chars (t_gui_window *window)
             prev_char = utf8_prev_char (window->buffer->input_buffer,
                                         start);
             pos_prev_char = prev_char - window->buffer->input_buffer;
-            size_current_char = start - prev_char;
+            size_prev_char = start - prev_char;
             size_start_char = utf8_char_size (start);
             
-            memcpy (saved_char, prev_char, size_current_char);
+            memcpy (saved_char, prev_char, size_prev_char);
             memcpy (prev_char, start, size_start_char);
-            memcpy (start, saved_char, size_current_char);
+            memcpy (prev_char + size_start_char, saved_char, size_prev_char);
             
-            memcpy (saved_char, window->buffer->input_buffer_color_mask + pos_prev_char, size_current_char);
+            memcpy (saved_char, window->buffer->input_buffer_color_mask + pos_prev_char, size_prev_char);
             memcpy (window->buffer->input_buffer_color_mask + pos_prev_char,
                     window->buffer->input_buffer_color_mask + pos_start, size_start_char);
-            memcpy (window->buffer->input_buffer_color_mask + pos_start,
-                    saved_char, size_current_char);
+            memcpy (window->buffer->input_buffer_color_mask + pos_prev_char + size_start_char,
+                    saved_char, size_prev_char);
             
             window->buffer->input_buffer_pos++;
             
@@ -882,6 +883,26 @@ gui_action_scroll_bottom (t_gui_window *window)
     gui_window_scroll_bottom (window);
 }
 
+/*
+ * gui_action_scroll_topic_left: scroll left topic
+ */
+
+void
+gui_action_scroll_topic_left (t_gui_window *window)
+{
+    gui_window_scroll_topic_left (window);
+}
+
+/*
+ * gui_action_scroll_topic_right: scroll right topic
+ */
+
+void
+gui_action_scroll_topic_right (t_gui_window *window)
+{
+    gui_window_scroll_topic_right (window);
+}
+
 /*
  * gui_action_nick_beginning: go to beginning of nicklist
  */
