@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -88,7 +88,8 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
 #endif
     
     /* use first buffer if no server was assigned to this buffer */
-    if ((type == BUFFER_TYPE_STANDARD) && gui_buffers && (!SERVER(gui_buffers)))
+    if ((type == BUFFER_TYPE_STANDARD) && gui_buffers
+        && (!SERVER(gui_buffers)) && (!channel))
     {
         if (server)
             ((t_irc_server *)(server))->buffer = gui_buffers;
@@ -544,6 +545,8 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     if (!gui_buffers && create_new && switch_to_another)
         (void) gui_buffer_new (gui_windows, NULL, NULL,
                                BUFFER_TYPE_STANDARD, 1);
+    
+    gui_status_draw (gui_current_window->buffer, 1);
 }
 
 /*
@@ -551,7 +554,7 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
  */
 
 t_gui_line *
-gui_buffer_line_new (t_gui_buffer *buffer)
+gui_buffer_line_new (t_gui_buffer *buffer, time_t date)
 {
     t_gui_line *new_line, *ptr_line;
     
@@ -562,6 +565,7 @@ gui_buffer_line_new (t_gui_buffer *buffer)
         new_line->log_write = 1;
         new_line->line_with_message = 0;
         new_line->line_with_highlight = 0;
+        new_line->date = date;
         new_line->nick = NULL;
         new_line->data = NULL;
         new_line->ofs_after_date = -1;
