@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,6 +41,7 @@
 #include "../common/hotlist.h"
 #include "../common/log.h"
 #include "../common/utf8.h"
+#include "../common/util.h"
 #include "../irc/irc.h"
 
 
@@ -108,7 +109,7 @@ gui_word_real_pos (t_gui_window *window, char *string, int pos)
  */
 
 void
-gui_add_to_line (t_gui_buffer *buffer, int type, char *nick, char *message)
+gui_add_to_line (t_gui_buffer *buffer, int type, time_t date, char *nick, char *message)
 {
     char *pos;
     int length;
@@ -117,7 +118,7 @@ gui_add_to_line (t_gui_buffer *buffer, int type, char *nick, char *message)
     if (buffer->line_complete)
     {
         buffer->line_complete = 0;
-        if (!gui_buffer_line_new (buffer))
+        if (!gui_buffer_line_new (buffer, date))
             return;
     }
     
@@ -220,14 +221,12 @@ gui_printf_internal (t_gui_buffer *buffer, int display_time, int type, char *nic
     static char buf[8192];
     char text_time[1024];
     char text_time_char[2];
-    time_t time_seconds;
+    time_t date;
     struct tm *local_time;
     int time_first_digit, time_last_digit;
-    char *buf2, *pos;
+    char *pos;
     int i;
     va_list argptr;
-    static time_t seconds;
-    struct tm *date_tmp;
     
     if (gui_init_ok)
     {
@@ -245,8 +244,9 @@ gui_printf_internal (t_gui_buffer *buffer, int display_time, int type, char *nic
     
         if (buffer == NULL)
         {
-            weechat_log_printf ("WARNING: gui_printf_internal without buffer! This is a bug, "
-                                "please send to developers - thanks\n");
+            weechat_log_printf ("%s gui_printf_internal without buffer! This is a bug, "
+                                "please send to developers - thanks\n",
+                                WEECHAT_WARNING);
             return;
         }
         
@@ -263,27 +263,21 @@ gui_printf_internal (t_gui_buffer *buffer, int display_time, int type, char *nic
     
     if (!buf[0])
         return;
-    
-    if (gui_init_ok)
-        buf2 = channel_iconv_decode (SERVER(buffer), CHANNEL(buffer), buf);
-    else
-        buf2 = strdup (buf);
+
+    utf8_normalize (buf, '?');
     
     if (gui_init_ok)
     {
-        seconds = time (NULL);
-        date_tmp = localtime (&seconds);
-        
-        pos = buf2;
+        pos = buf;
         while (pos)
         {
+            date = time (NULL);
             if ((!buffer->last_line) || (buffer->line_complete))
             {
                 if (display_time && cfg_look_buffer_timestamp &&
                     cfg_look_buffer_timestamp[0])
                 {
-                    time_seconds = time (NULL);
-                    local_time = localtime (&time_seconds);
+                    local_time = localtime (&date);
                     strftime (text_time, sizeof (text_time), cfg_look_buffer_timestamp, local_time);
                     
                     time_first_digit = -1;
@@ -307,41 +301,46 @@ gui_printf_internal (t_gui_buffer *buffer, int display_time, int type, char *nic
                         text_time_char[0] = text_time[i];
                         if (time_first_digit < 0)
                         {
-                            gui_add_to_line (buffer, MSG_TYPE_TIME, NULL,
-                                             GUI_COLOR(COLOR_WIN_CHAT_TIME));
-                            gui_add_to_line (buffer, MSG_TYPE_TIME, NULL, text_time_char);
+                            gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                             NULL, GUI_COLOR(COLOR_WIN_CHAT_TIME));
+                            gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                             NULL, text_time_char);
                         }
                         else
                         {
                             if ((i < time_first_digit) || (i > time_last_digit))
                             {
-                                gui_add_to_line (buffer, MSG_TYPE_TIME, NULL,
-                                                 GUI_COLOR(COLOR_WIN_CHAT_DARK));
-                                gui_add_to_line (buffer, MSG_TYPE_TIME, NULL, text_time_char);
+                                gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                                 NULL, GUI_COLOR(COLOR_WIN_CHAT_DARK));
+                                gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                                 NULL, text_time_char);
                             }
                             else
                             {
                                 if (isdigit (text_time[i]))
                                 {
-                                    gui_add_to_line (buffer, MSG_TYPE_TIME, NULL,
-                                                     GUI_COLOR(COLOR_WIN_CHAT_TIME));
-                                    gui_add_to_line (buffer, MSG_TYPE_TIME, NULL, text_time_char);
+                                    gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                                     NULL, GUI_COLOR(COLOR_WIN_CHAT_TIME));
+                                    gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                                     NULL, text_time_char);
                                 }
                                 else
                                 {
-                                    gui_add_to_line (buffer, MSG_TYPE_TIME, NULL,
-                                                     GUI_COLOR(COLOR_WIN_CHAT_TIME_SEP));
-                                    gui_add_to_line (buffer, MSG_TYPE_TIME, NULL, text_time_char);
+                                    gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                                     NULL, GUI_COLOR(COLOR_WIN_CHAT_TIME_SEP));
+                                    gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                                     NULL, text_time_char);
                                 }
                             }
                         }
                         i++;
                     }
-                    gui_add_to_line (buffer, MSG_TYPE_TIME, NULL, GUI_COLOR(COLOR_WIN_CHAT));
+                    gui_add_to_line (buffer, MSG_TYPE_TIME, date,
+                                     NULL, GUI_COLOR(COLOR_WIN_CHAT));
                 }
-                gui_add_to_line (buffer, MSG_TYPE_TIME, NULL, " ");
+                gui_add_to_line (buffer, MSG_TYPE_TIME, date, NULL, " ");
             }
-            gui_add_to_line (buffer, type, nick, pos);
+            gui_add_to_line (buffer, type, date, nick, pos);
             pos = strchr (pos, '\n');
             if (pos)
             {
@@ -353,17 +352,16 @@ gui_printf_internal (t_gui_buffer *buffer, int display_time, int type, char *nic
         }
     }
     else
-        printf ("%s", buf2);
-    
-    free (buf2);
+        weechat_iconv_fprintf (stdout, buf);
 }
 
 /*
  * gui_printf_raw_data: display raw IRC data (only if raw IRC data buffer exists)
+ *                      type: 0 = recv, 1 = send, -1 = recv, modified by a modifier (plugin)
  */
 
 void
-gui_printf_raw_data (void *server, int send, char *message)
+gui_printf_raw_data (void *server, int send, int modified, char *message)
 {
     char *pos;
     
@@ -381,7 +379,8 @@ gui_printf_raw_data (void *server, int send, char *message)
                               ((t_irc_server *)server)->name,
                               GUI_COLOR(COLOR_WIN_CHAT_DARK),
                               GUI_COLOR((send) ? COLOR_WIN_CHAT_PART : COLOR_WIN_CHAT_JOIN),
-                              (send) ? PREFIX_PART : PREFIX_JOIN,
+                              (send) ? ((modified) ? PREFIX_SEND_MOD : PREFIX_PART) :
+                              ((modified) ? PREFIX_RECV_MOD : PREFIX_JOIN),
                               GUI_COLOR(COLOR_WIN_CHAT),
                               message);
             if (pos)
@@ -405,7 +404,7 @@ gui_infobar_printf (int time_displayed, int color, char *message, ...)
     static char buf[1024];
     va_list argptr;
     t_gui_infobar *ptr_infobar;
-    char *pos;
+    char *buf2, *ptr_buf, *pos;
     
     va_start (argptr, message);
     vsnprintf (buf, sizeof (buf) - 1, message, argptr);
@@ -414,8 +413,11 @@ gui_infobar_printf (int time_displayed, int color, char *message, ...)
     ptr_infobar = (t_gui_infobar *)malloc (sizeof (t_gui_infobar));
     if (ptr_infobar)
     {
+        buf2 = (char *)gui_color_decode ((unsigned char *)buf, 0);
+        ptr_buf = (buf2) ? buf2 : buf;
+        
         ptr_infobar->color = color;
-        ptr_infobar->text = strdup (buf);
+        ptr_infobar->text = strdup (ptr_buf);
         pos = strchr (ptr_infobar->text, '\n');
         if (pos)
             pos[0] = '\0';
@@ -423,43 +425,12 @@ gui_infobar_printf (int time_displayed, int color, char *message, ...)
         ptr_infobar->next_infobar = gui_infobar;
         gui_infobar = ptr_infobar;
         gui_infobar_draw (gui_current_window->buffer, 1);
+        if (buf2)
+            free (buf2);
     }
     else
-        weechat_log_printf (_("Not enough memory for infobar message\n"));
-}
-
-/*
- * gui_infobar_printf_from_buffer: remove color, convert charset, then
- *                                 display message in infobar
- */
-
-void
-gui_infobar_printf_from_buffer (t_gui_buffer *buffer, int time_displayed,
-                                int color, char *message1, char *message, ...)
-{
-    static char buf[1024];
-    va_list argptr;
-    char *buf2, *buf3;
-    
-    va_start (argptr, message);
-    vsnprintf (buf, sizeof (buf) - 1, message, argptr);
-    va_end (argptr);
-    
-    buf2 = (char *)gui_color_decode ((unsigned char *)buf, 0);
-    
-    if (buf2)
-        buf3 = channel_iconv_decode (SERVER(buffer), CHANNEL(buffer), buf2);
-    else
-        buf3 = NULL;
-    
-    gui_infobar_printf (time_displayed, color,
-                        "%s%s", message1,
-                        (buf3) ? buf3 : ((buf2) ? buf2 : buf));
-    
-    if (buf2)
-        free (buf2);
-    if (buf3)
-        free (buf3);
+        weechat_log_printf (_("%s not enough memory for infobar message\n"),
+                            WEECHAT_ERROR);
 }
 
 /*
@@ -626,13 +597,16 @@ gui_input_complete (t_gui_window *window)
         if ((window->buffer->completion.context == COMPLETION_COMMAND)
             || (window->buffer->completion.context == COMPLETION_COMMAND_ARG))
         {
-            if (window->buffer->input_buffer[utf8_real_pos (window->buffer->input_buffer,
-                                                            window->buffer->input_buffer_pos)] != ' ')
-                gui_insert_string_input (window, " ",
-                                         window->buffer->input_buffer_pos);
-            if (window->buffer->completion.position >= 0)
-                window->buffer->completion.position++;
-            window->buffer->input_buffer_pos++;
+            if (window->buffer->completion.add_space)
+            {
+                if (window->buffer->input_buffer[utf8_real_pos (window->buffer->input_buffer,
+                                                                window->buffer->input_buffer_pos)] != ' ')
+                    gui_insert_string_input (window, " ",
+                                             window->buffer->input_buffer_pos);
+                if (window->buffer->completion.position >= 0)
+                    window->buffer->completion.position++;
+                window->buffer->input_buffer_pos++;
+            }
         }
         else
         {
