@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -103,6 +103,10 @@ t_gui_key_function gui_key_functions[] =
     N_("scroll to top of buffer") },
   { "scroll_bottom",             gui_action_scroll_bottom,
     N_("scroll to bottom of buffer") },
+  { "scroll_topic_left",         gui_action_scroll_topic_left,
+    N_("scroll left topic") },
+  { "scroll_topic_right",        gui_action_scroll_topic_right,
+    N_("scroll right topic") },
   { "nick_beginning",            gui_action_nick_beginning,
     N_("display beginning of nicklist") },
   { "nick_end",                  gui_action_nick_end,
