@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -71,8 +71,8 @@ gui_log_write_line (t_gui_buffer *buffer, char *message)
     if (buffer->log_file)
     {
         msg_no_color = (char *)gui_color_decode ((unsigned char *)message, 0);
-        fprintf (buffer->log_file, "%s\n",
-                 (msg_no_color) ? msg_no_color : message);
+        weechat_iconv_fprintf (buffer->log_file,
+                               "%s\n", (msg_no_color) ? msg_no_color : message);
         fflush (buffer->log_file);
         if (msg_no_color)
             free (msg_no_color);
@@ -91,8 +91,8 @@ gui_log_write (t_gui_buffer *buffer, char *message)
     if (buffer->log_file)
     {
         msg_no_color = (char *)gui_color_decode ((unsigned char *)message, 0);
-        fprintf (buffer->log_file, "%s",
-                 (msg_no_color) ? msg_no_color : message);
+        weechat_iconv_fprintf (buffer->log_file,
+                               "%s", (msg_no_color) ? msg_no_color : message);
         fflush (buffer->log_file);
         if (msg_no_color)
             free (msg_no_color);
