@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -212,9 +212,29 @@ gui_window_new (t_gui_window *parent, int x, int y, int width, int height,
         new_window->win_nick_height = 0;
         new_window->win_nick_num_max = 0;
         new_window->win_nick_start = 0;
-            
+        
+        new_window->win_title_x = 0;
+        new_window->win_title_y = 0;
+        new_window->win_title_width = 0;
+        new_window->win_title_height = 0;
+        new_window->win_title_start = 0;
+        
+        new_window->win_status_x = 0;
+        new_window->win_status_y = 0;
+        new_window->win_status_width = 0;
+        new_window->win_status_height = 0;
+        
+        new_window->win_infobar_x = 0;
+        new_window->win_infobar_y = 0;
+        new_window->win_infobar_width = 0;
+        new_window->win_infobar_height = 0;
+        
+        new_window->win_input_x = 0;
+        new_window->win_input_y = 0;
+        new_window->win_input_width = 0;
+        new_window->win_input_height = 0;
         new_window->win_input_cursor_x = 0;
-            
+        
         new_window->dcc_first = NULL;
         new_window->dcc_selected = NULL;
         new_window->dcc_last_displayed = NULL;
@@ -413,6 +433,7 @@ gui_window_print_log (t_gui_window *window)
     weechat_log_printf ("  win_title_y . . . . : %d\n",   window->win_title_y);
     weechat_log_printf ("  win_title_width . . : %d\n",   window->win_title_width);
     weechat_log_printf ("  win_title_height. . : %d\n",   window->win_title_height);
+    weechat_log_printf ("  win_title_start . . : %d\n",   window->win_title_start);
     weechat_log_printf ("  win_status_x. . . . : %d\n",   window->win_status_x);
     weechat_log_printf ("  win_status_y. . . . : %d\n",   window->win_status_y);
     weechat_log_printf ("  win_status_width. . : %d\n",   window->win_status_width);
