@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -89,7 +89,7 @@ extern t_gui_buffer *gui_buffer_get_dcc (t_gui_window *);
 extern void gui_buffer_clear (t_gui_buffer *);
 extern void gui_buffer_clear_all ();
 extern void gui_buffer_free (t_gui_buffer *, int);
-extern t_gui_line *gui_buffer_line_new (t_gui_buffer *);
+extern t_gui_line *gui_buffer_line_new (t_gui_buffer *, time_t);
 extern void gui_buffer_merge_servers (t_gui_window *);
 extern void gui_buffer_split_server (t_gui_window *);
 extern void gui_buffer_switch_previous (t_gui_window *);
@@ -136,6 +136,8 @@ extern void gui_action_scroll_up (t_gui_window *);
 extern void gui_action_scroll_down (t_gui_window *);
 extern void gui_action_scroll_top (t_gui_window *);
 extern void gui_action_scroll_bottom (t_gui_window *);
+extern void gui_action_scroll_topic_left (t_gui_window *);
+extern void gui_action_scroll_topic_right (t_gui_window *);
 extern void gui_action_nick_beginning (t_gui_window *);
 extern void gui_action_nick_end (t_gui_window *);
 extern void gui_action_nick_page_up (t_gui_window *);
@@ -177,14 +179,14 @@ extern void gui_log_start (t_gui_buffer *);
 extern void gui_log_end (t_gui_buffer *);
 
 /* other */
+extern int gui_word_strlen (t_gui_window *, char *);
+extern int gui_word_real_pos (t_gui_window *, char *, int);
+extern void gui_printf_internal (t_gui_buffer *, int, int, char *, char *, ...);
+extern void gui_printf_raw_data (void *, int, int, char *);
 extern void gui_infobar_printf (int, int, char *, ...);
 extern void gui_infobar_printf_from_buffer (t_gui_buffer *, int, int, char *, char *, ...);
 extern void gui_infobar_remove ();
 extern void gui_infobar_remove_all ();
-extern int gui_word_strlen (t_gui_window *, char *);
-extern int gui_word_real_pos (t_gui_window *, char *, int);
-extern void gui_printf_internal (t_gui_buffer *, int, int, char *, char *, ...);
-extern void gui_printf_raw_data (void *, int, char *);
 extern void gui_input_optimize_size (t_gui_buffer *);
 extern void gui_input_init_color_mask (t_gui_buffer *);
 extern void gui_input_move (t_gui_buffer *, char *, char *, int );
@@ -240,6 +242,8 @@ extern void gui_window_scroll_up (t_gui_window *);
 extern void gui_window_scroll_down (t_gui_window *);
 extern void gui_window_scroll_top (t_gui_window *);
 extern void gui_window_scroll_bottom (t_gui_window *);
+extern void gui_window_scroll_topic_left (t_gui_window *);
+extern void gui_window_scroll_topic_right (t_gui_window *);
 extern void gui_window_nick_beginning (t_gui_window *);
 extern void gui_window_nick_end (t_gui_window *);
 extern void gui_window_nick_page_up (t_gui_window *);
