@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -258,143 +258,6 @@ string_is_channel (char *string)
     return (strpbrk (first_char, CHANNEL_PREFIX)) ? 1 : 0;    
 }
 
-/*
- * channel_get_charset_decode_iso: get decode iso value for channel
- *                                 if not found for channel, look for server
- *                                 if not found for server, look for global
- */
-
-char *
-channel_get_charset_decode_iso (t_irc_server *server, t_irc_channel *channel)
-{
-    char *pos, *result;
-    int length;
-    
-    if (!server)
-        return (cfg_look_charset_decode_iso) ?
-            strdup (cfg_look_charset_decode_iso) : strdup ("");
-    
-    if (!channel)
-        return server_get_charset_decode_iso (server);
-    
-    config_option_list_get_value (&(server->charset_decode_iso),
-                                  channel->name, &pos, &length);
-    if (pos && (length > 0))
-    {
-        result = strdup (pos);
-        result[length] = '\0';
-        return result;
-    }
-    
-    return server_get_charset_decode_iso (server);
-}
-
-/*
- * channel_get_charset_decode_utf: get decode utf value for channel
- *                                 if not found for channel, look for server
- *                                 if not found for server, look for global
- */
-
-char *
-channel_get_charset_decode_utf (t_irc_server *server, t_irc_channel *channel)
-{
-    char *pos, *result;
-    int length;
-    
-    if (!server)
-        return (cfg_look_charset_decode_utf) ?
-            strdup (cfg_look_charset_decode_utf) : strdup ("");
-    
-    if (!channel)
-        return server_get_charset_decode_utf (server);
-    
-    config_option_list_get_value (&(server->charset_decode_utf),
-                                  channel->name, &pos, &length);
-    if (pos && (length > 0))
-    {
-        result = strdup (pos);
-        result[length] = '\0';
-        return result;
-    }
-    
-    return server_get_charset_decode_utf (server);
-}
-
-/*
- * channel_get_charset_encode: get encode value for channel
- *                             if not found for channel, look for server
- *                             if not found for server, look for global
- */
-
-char *
-channel_get_charset_encode (t_irc_server *server, t_irc_channel *channel)
-{
-    char *pos, *result;
-    int length;
-    
-    if (!server)
-        return (cfg_look_charset_encode) ?
-            strdup (cfg_look_charset_encode) : strdup ("");
-    
-    if (!channel)
-        return server_get_charset_encode (server);
-    
-    config_option_list_get_value (&(server->charset_encode),
-                                  channel->name, &pos, &length);
-    if (pos && (length > 0))
-    {
-        result = strdup (pos);
-        result[length] = '\0';
-        return result;
-    }
-    
-    return server_get_charset_encode (server);
-}
-
-/*
- * channel_iconv_decode: convert string to local charset
- */
-
-char *
-channel_iconv_decode (t_irc_server *server, t_irc_channel *channel, char *string)
-{
-    char *from_charset, *string2;
-    
-    if (!local_utf8 || !utf8_is_valid (string))
-    {
-        if (local_utf8)
-            from_charset = channel_get_charset_decode_iso (server, channel);
-        else
-            from_charset = channel_get_charset_decode_utf (server, channel);
-        string2 = weechat_iconv (from_charset,
-                                 (cfg_look_charset_internal && cfg_look_charset_internal[0]) ?
-                                 cfg_look_charset_internal : local_charset,
-                                 string);
-        free (from_charset);
-        return string2;
-    }
-    else
-        return strdup (string);
-}
-
-/*
- *
- */
-
-char *
-channel_iconv_encode (t_irc_server *server, t_irc_channel *channel, char *string)
-{
-    char *to_charset, *string2;
-    
-    to_charset = channel_get_charset_encode (server, channel);
-    string2 = weechat_iconv ((cfg_look_charset_internal && cfg_look_charset_internal[0]) ?
-                             cfg_look_charset_internal : local_charset,
-                             to_charset,
-                             string);
-    free (to_charset);
-    return string2;
-}
-
 /*
  * channel_remove_away: remove away for all nicks on a channel
  */
@@ -427,7 +290,7 @@ channel_check_away (t_irc_server *server, t_irc_channel *channel, int force)
             (channel->nicks_count <= cfg_irc_away_check_max_nicks))
         {
             channel->checking_away++;
-            server_sendf (server, "WHO %s\r\n", channel->name);
+            server_sendf (server, "WHO %s", channel->name);
         }
         else
             channel_remove_away (channel);
