@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -632,8 +632,8 @@ dcc_accept (t_irc_dcc *ptr_dcc)
         ptr_dcc->status = DCC_CONNECTING;
         server_sendf (ptr_dcc->server,
                       (strchr (ptr_dcc->filename, ' ')) ?
-                          "PRIVMSG %s :\01DCC RESUME \"%s\" %d %u\01\r\n" :
-                          "PRIVMSG %s :\01DCC RESUME %s %d %u\01\r\n",
+                          "PRIVMSG %s :\01DCC RESUME \"%s\" %d %u\01\n" :
+                          "PRIVMSG %s :\01DCC RESUME %s %d %u\01",
                       ptr_dcc->nick, ptr_dcc->filename,
                       ptr_dcc->port, ptr_dcc->start_resume);
         dcc_redraw (HOTLIST_MSG);
@@ -661,8 +661,8 @@ dcc_accept_resume (t_irc_server *server, char *filename, int port,
         ptr_dcc->last_check_pos = pos_start;
         server_sendf (ptr_dcc->server,
                       (strchr (ptr_dcc->filename, ' ')) ?
-                          "PRIVMSG %s :\01DCC ACCEPT \"%s\" %d %u\01\r\n" :
-                          "PRIVMSG %s :\01DCC ACCEPT %s %d %u\01\r\n",
+                          "PRIVMSG %s :\01DCC ACCEPT \"%s\" %d %u\01\n" :
+                          "PRIVMSG %s :\01DCC ACCEPT %s %d %u\01",
                       ptr_dcc->nick, ptr_dcc->filename,
                       ptr_dcc->port, ptr_dcc->start_resume);
         
@@ -958,12 +958,14 @@ dcc_send_request (t_irc_server *server, int type, char *nick, char *filename)
     
     if (type == DCC_FILE_SEND)
     {
-        /* add home if filename not beginning with '/' (not for Win32) */
+        /* add home if filename not beginning with '/' or '~' (not for Win32) */
 #ifdef _WIN32
         filename2 = strdup (filename);
 #else
         if (filename[0] == '/')
             filename2 = strdup (filename);
+        else if (filename[0] == '~')
+            filename2 = weechat_strreplace (filename, "~", getenv ("HOME"));
         else
         {
             dir1 = weechat_strreplace (cfg_dcc_upload_path, "~", getenv ("HOME"));
@@ -1158,13 +1160,13 @@ dcc_send_request (t_irc_server *server, int type, char *nick, char *filename)
     /* send DCC request to nick */
     if (type == DCC_CHAT_SEND)
         server_sendf (server, 
-                      "PRIVMSG %s :\01DCC CHAT chat %lu %d\01\r\n",
+                      "PRIVMSG %s :\01DCC CHAT chat %lu %d\01",
                       nick, local_addr, port);
     else
         server_sendf (server, 
                       (spaces) ?
-                          "PRIVMSG %s :\01DCC SEND \"%s\" %lu %d %u\01\r\n" :
-                          "PRIVMSG %s :\01DCC SEND %s %lu %d %u\01\r\n",
+                          "PRIVMSG %s :\01DCC SEND \"%s\" %lu %d %u\01\n" :
+                          "PRIVMSG %s :\01DCC SEND %s %lu %d %u\01",
                       nick, short_filename, local_addr, port,
                       (unsigned long) st.st_size);
     
@@ -1196,9 +1198,8 @@ dcc_chat_sendf (t_irc_dcc *ptr_dcc, char *fmt, ...)
 {
     va_list args;
     static char buffer[4096];
-    char *buf2;
     int size_buf;
-
+    
     if (!ptr_dcc || (ptr_dcc->sock < 0))
         return;
     
@@ -1217,10 +1218,7 @@ dcc_chat_sendf (t_irc_dcc *ptr_dcc, char *fmt, ...)
     gui_printf (ptr_dcc->server->buffer, "[DEBUG] Sending to remote host (DCC CHAT) >>> %s\n", buffer);
     buffer[size_buf - 2] = '\r';
 #endif
-    buf2 = channel_iconv_encode (ptr_dcc->server,
-                                 ptr_dcc->channel,
-                                 buffer);
-    if (dcc_chat_send (ptr_dcc, buf2, strlen (buf2)) <= 0)
+    if (dcc_chat_send (ptr_dcc, buffer, strlen (buffer)) <= 0)
     {
         irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
                             PREFIX_ERROR);
@@ -1229,7 +1227,6 @@ dcc_chat_sendf (t_irc_dcc *ptr_dcc, char *fmt, ...)
                     WEECHAT_ERROR, ptr_dcc->nick);
         dcc_close (ptr_dcc, DCC_FAILED);
     }
-    free (buf2);
 }
 
 /*
@@ -1242,7 +1239,7 @@ dcc_chat_recv (t_irc_dcc *ptr_dcc)
     fd_set read_fd;
     static struct timeval timeout;
     static char buffer[4096 + 2];
-    char *buf2, *pos, *ptr_buf, *ptr_buf2, *next_ptr_buf;
+    char *buf2, *pos, *ptr_buf, *next_ptr_buf;
     char *ptr_buf_color;
     int num_read;
     
@@ -1307,11 +1304,7 @@ dcc_chat_recv (t_irc_dcc *ptr_dcc)
             
             if (ptr_buf)
             {
-                ptr_buf2 = channel_iconv_decode (ptr_dcc->server,
-                                                 ptr_dcc->channel,
-                                                 ptr_buf);
-                ptr_buf_color = (char *)gui_color_decode ((ptr_buf2) ?
-                                                          (unsigned char *)ptr_buf2 : (unsigned char *)ptr_buf,
+                ptr_buf_color = (char *)gui_color_decode ((unsigned char *)ptr_buf,
                                                           cfg_irc_colors_receive);
                 
                 if (irc_is_highlight (ptr_buf, ptr_dcc->server->nick))
@@ -1326,7 +1319,7 @@ dcc_chat_recv (t_irc_dcc *ptr_dcc)
                                             COLOR_WIN_INFOBAR_HIGHLIGHT,
                                             _("Private %s> %s"),
                                             ptr_dcc->nick,
-                                            (ptr_buf_color) ? ptr_buf_color : ((ptr_buf2) ? ptr_buf2 : ptr_buf));
+                                            (ptr_buf_color) ? ptr_buf_color : ptr_buf);
                     }
                 }
                 else
@@ -1338,8 +1331,6 @@ dcc_chat_recv (t_irc_dcc *ptr_dcc)
                                  (ptr_buf_color) ? ptr_buf_color : ptr_buf);
                 if (ptr_buf_color)
                     free (ptr_buf_color);
-                if (ptr_buf2)
-                    free (ptr_buf2);
             }
             
             ptr_buf = next_ptr_buf;
