@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 
 #include "../common/weechat.h"
 #include "irc.h"
+#include "../common/utf8.h"
 #include "../common/weeconfig.h"
 #include "../gui/gui.h"
 
@@ -159,7 +160,7 @@ irc_display_nick (t_gui_buffer *buffer, t_irc_nick *nick, char *nickname,
     ptr_nickname = strdup ((nick) ? nick->nick : nickname);
     if (!ptr_nickname)
         return;
-    nickname_length = strlen (ptr_nickname); 
+    nickname_length = utf8_width_screen (ptr_nickname); 
     external_nick = (!nick && !BUFFER_IS_PRIVATE(buffer));
     disable_prefix_suffix = ((cfg_look_align_nick != CFG_LOOK_ALIGN_NICK_NONE)
                              && ((int)strlen (cfg_look_nick_prefix) +
@@ -428,13 +429,4 @@ irc_display_server (t_irc_server *server)
     gui_printf (NULL, "  server_notify_levels . . . : %s\n",
                 (server->notify_levels && server->notify_levels[0]) ?
                 server->notify_levels : "");
-    gui_printf (NULL, "  server_charset_decode_iso. : %s\n",
-                (server->charset_decode_iso && server->charset_decode_iso[0]) ?
-                server->charset_decode_iso : "");
-    gui_printf (NULL, "  server_charset_decode_utf. : %s\n",
-                (server->charset_decode_utf && server->charset_decode_utf[0]) ?
-                server->charset_decode_utf : "");
-    gui_printf (NULL, "  server_charset_encode. . . : %s\n",
-                (server->charset_encode && server->charset_encode[0]) ?
-                server->charset_encode : "");
 }
