@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -46,9 +46,14 @@
 #include "../common/weechat.h"
 #include "irc.h"
 #include "../common/log.h"
+#include "../common/util.h"
 #include "../common/weeconfig.h"
 #include "../gui/gui.h"
 
+#ifdef PLUGINS
+#include "../plugins/plugins.h"
+#endif
+
 
 t_irc_server *irc_servers = NULL;
 t_irc_server *last_irc_server = NULL;
@@ -89,9 +94,6 @@ server_init (t_irc_server *server)
     server->autojoin = NULL;
     server->autorejoin = 0;
     server->notify_levels = NULL;
-    server->charset_decode_iso = NULL;
-    server->charset_decode_utf = NULL;
-    server->charset_encode = NULL;
     
     /* internal vars */
     server->child_pid = 0;
@@ -113,6 +115,10 @@ server_init (t_irc_server *server)
     server->lag_check_time.tv_usec = 0;
     server->lag_next_check = time (NULL) + cfg_irc_lag_check;
     server->cmd_list_regexp = NULL;
+    server->queue_msg = 0;
+    server->last_user_message = 0;
+    server->outqueue = NULL;
+    server->last_outqueue = NULL;
     server->buffer = NULL;
     server->saved_buffer = NULL;
     server->channels = NULL;
@@ -185,10 +191,10 @@ server_init_with_url (char *irc_url, t_irc_server *server)
             server->nick1 = strdup (my_passwd->pw_name);
         else
         {
-            fprintf (stderr, "%s: %s (%s).",
-                     WEECHAT_WARNING,
-                     _("Unable to get user's name"),
-                     strerror (errno));
+            weechat_iconv_fprintf (stderr, "%s: %s (%s).",
+                                   WEECHAT_WARNING,
+                                   _("Unable to get user's name"),
+                                   strerror (errno));
             free (url);
             return -1;
         }
@@ -255,7 +261,9 @@ server_alloc ()
     /* alloc memory for new server */
     if ((new_server = (t_irc_server *) malloc (sizeof (t_irc_server))) == NULL)
     {
-        fprintf (stderr, _("%s cannot allocate new server\n"), WEECHAT_ERROR);
+        weechat_iconv_fprintf (stderr,
+                               _("%s cannot allocate new server\n"),
+                               WEECHAT_ERROR);
         return NULL;
     }
 
@@ -275,6 +283,74 @@ server_alloc ()
     return new_server;
 }
 
+/*
+ * server_outqueue_add: add a message in out queue
+ */
+
+void
+server_outqueue_add (t_irc_server *server, char *msg1, char *msg2, int modified)
+{
+    t_irc_outqueue *new_outqueue;
+
+    new_outqueue = (t_irc_outqueue *)malloc (sizeof (t_irc_outqueue));
+    if (new_outqueue)
+    {
+        new_outqueue->message_before_mod = (msg1) ? strdup (msg1) : NULL;
+        new_outqueue->message_after_mod = (msg2) ? strdup (msg2) : NULL;
+        new_outqueue->modified = modified;
+        
+        new_outqueue->prev_outqueue = server->last_outqueue;
+        new_outqueue->next_outqueue = NULL;
+        if (server->outqueue)
+            server->last_outqueue->next_outqueue = new_outqueue;
+        else
+            server->outqueue = new_outqueue;
+        server->last_outqueue = new_outqueue;
+    }
+}
+
+/*
+ * server_outqueue_free: free a message in out queue
+ */
+
+void
+server_outqueue_free (t_irc_server *server, t_irc_outqueue *outqueue)
+{
+    t_irc_outqueue *new_outqueue;
+    
+    /* remove outqueue message */
+    if (server->last_outqueue == outqueue)
+        server->last_outqueue = outqueue->prev_outqueue;
+    if (outqueue->prev_outqueue)
+    {
+        (outqueue->prev_outqueue)->next_outqueue = outqueue->next_outqueue;
+        new_outqueue = server->outqueue;
+    }
+    else
+        new_outqueue = outqueue->next_outqueue;
+    
+    if (outqueue->next_outqueue)
+        (outqueue->next_outqueue)->prev_outqueue = outqueue->prev_outqueue;
+    
+    if (outqueue->message_before_mod)
+        free (outqueue->message_before_mod);
+    if (outqueue->message_after_mod)
+        free (outqueue->message_after_mod);
+    free (outqueue);
+    server->outqueue = new_outqueue;
+}
+
+/*
+ * server_outqueue_free_all: free all outqueued messages
+ */
+
+void
+server_outqueue_free_all (t_irc_server *server)
+{
+    while (server->outqueue)
+        server_outqueue_free (server, server->outqueue);
+}
+
 /*
  * server_destroy: free server data (not struct himself)
  */
@@ -310,12 +386,6 @@ server_destroy (t_irc_server *server)
         free (server->autojoin);
     if (server->notify_levels)
         free (server->notify_levels);
-    if (server->charset_decode_iso)
-        free (server->charset_decode_iso);
-    if (server->charset_decode_utf)
-        free (server->charset_decode_utf);
-    if (server->charset_encode)
-        free (server->charset_encode);
     if (server->unterminated_message)
         free (server->unterminated_message);
     if (server->nick)
@@ -324,6 +394,8 @@ server_destroy (t_irc_server *server)
         free (server->nick_modes);
     if (server->away_message)
         free (server->away_message);
+    if (server->outqueue)
+        server_outqueue_free_all (server);
     if (server->channels)
         channel_free_all (server);
 }
@@ -385,9 +457,7 @@ server_new (char *name, int autoconnect, int autoreconnect,
             int port, int ipv6, int ssl, char *password,
             char *nick1, char *nick2, char *nick3, char *username,
             char *realname, char *hostname, char *command, int command_delay,
-            char *autojoin, int autorejoin, char *notify_levels,
-            char *charset_decode_iso, char *charset_decode_utf,
-            char *charset_encode)
+            char *autojoin, int autorejoin, char *notify_levels)
 {
     t_irc_server *new_server;
     
@@ -398,16 +468,13 @@ server_new (char *name, int autoconnect, int autoreconnect,
     weechat_log_printf ("Creating new server (name:%s, address:%s, port:%d, pwd:%s, "
                         "nick1:%s, nick2:%s, nick3:%s, username:%s, realname:%s, "
                         "hostname: %s, command:%s, autojoin:%s, autorejoin:%s, "
-                        "notify_levels:%s, decode_iso:%s, decode_utf:%s, encode:%s)\n",
+                        "notify_levels:%s)\n",
                         name, address, port, (password) ? password : "",
                         (nick1) ? nick1 : "", (nick2) ? nick2 : "", (nick3) ? nick3 : "",
                         (username) ? username : "", (realname) ? realname : "",
                         (hostname) ? hostname : "", (command) ? command : "",
                         (autojoin) ? autojoin : "", (autorejoin) ? "on" : "off",
-                        (notify_levels) ? notify_levels : "",
-                        (charset_decode_iso) ? charset_decode_iso : "",
-                        (charset_decode_utf) ? charset_decode_utf : "",
-                        (charset_encode) ? charset_encode : "");
+                        (notify_levels) ? notify_levels : "");
 #endif
     
     if ((new_server = server_alloc ()))
@@ -439,102 +506,12 @@ server_new (char *name, int autoconnect, int autoreconnect,
         new_server->autorejoin = autorejoin;
         new_server->notify_levels =
             (notify_levels) ? strdup (notify_levels) : NULL;
-        new_server->charset_decode_iso =
-            (charset_decode_iso) ? strdup (charset_decode_iso) : NULL;
-        new_server->charset_decode_utf =
-            (charset_decode_utf) ? strdup (charset_decode_utf) : NULL;
-        new_server->charset_encode =
-            (charset_encode) ? strdup (charset_encode) : NULL;
     }
     else
         return NULL;
     return new_server;
 }
 
-/*
- * server_get_charset_decode_iso: get decode iso value for server
- *                                if not found for server, look for global
- */
-
-char *
-server_get_charset_decode_iso (t_irc_server *server)
-{
-    char *pos, *result;
-    int length;
-    
-    if (!server)
-        return (cfg_look_charset_decode_iso) ?
-            strdup (cfg_look_charset_decode_iso) : strdup ("");
-    
-    config_option_list_get_value (&(server->charset_decode_iso),
-                                  "server", &pos, &length);
-    if (pos && (length > 0))
-    {
-        result = strdup (pos);
-        result[length] = '\0';
-        return result;
-    }
-    
-    return (cfg_look_charset_decode_iso) ?
-        strdup (cfg_look_charset_decode_iso) : strdup ("");
-}
-
-/*
- * server_get_charset_decode_utf: get decode utf value for server
- *                                if not found for server, look for global
- */
-
-char *
-server_get_charset_decode_utf (t_irc_server *server)
-{
-    char *pos, *result;
-    int length;
-    
-    if (!server)
-        return (cfg_look_charset_decode_utf) ?
-            strdup (cfg_look_charset_decode_utf) : strdup ("");
-    
-    config_option_list_get_value (&(server->charset_decode_utf),
-                                  "server", &pos, &length);
-    if (pos && (length > 0))
-    {
-        result = strdup (pos);
-        result[length] = '\0';
-        return result;
-    }
-    
-    return (cfg_look_charset_decode_utf) ?
-        strdup (cfg_look_charset_decode_utf) : strdup ("");
-}
-
-/*
- * server_get_charset_encode: get encode value for server
- *                            if not found for server, look for global
- */
-
-char *
-server_get_charset_encode (t_irc_server *server)
-{
-    char *pos, *result;
-    int length;
-    
-    if (!server)
-        return (cfg_look_charset_encode) ?
-            strdup (cfg_look_charset_encode) : strdup ("");
-    
-    config_option_list_get_value (&(server->charset_encode),
-                                  "server", &pos, &length);
-    if (pos && (length > 0))
-    {
-        result = strdup (pos);
-        result[length] = '\0';
-        return result;
-    }
-    
-    return (cfg_look_charset_encode) ?
-        strdup (cfg_look_charset_encode) : strdup ("");
-}
-
 /*
  * server_send: send data to IRC server
  */
@@ -553,8 +530,158 @@ server_send (t_irc_server *server, char *buffer, int size_buf)
         return send (server->sock, buffer, size_buf, 0);
 }
 
+/*
+ * server_outqueue_send: send a message from outqueue
+ */
+
+void
+server_outqueue_send (t_irc_server *server)
+{
+    time_t time_now;
+    char *pos;
+    
+    if (server->outqueue)
+    {
+        time_now = time (NULL);
+        if (time_now >= server->last_user_message + cfg_irc_anti_flood)
+        {
+            if (server->outqueue->message_before_mod)
+            {
+                pos = strchr (server->outqueue->message_before_mod, '\r');
+                if (pos)
+                    pos[0] = '\0';
+                gui_printf_raw_data (server, 1, 0,
+                                     server->outqueue->message_before_mod);
+                if (pos)
+                    pos[0] = '\r';
+            }
+            if (server->outqueue->message_after_mod)
+            {
+                pos = strchr (server->outqueue->message_after_mod, '\r');
+                if (pos)
+                    pos[0] = '\0';
+                gui_printf_raw_data (server, 1, server->outqueue->modified,
+                                     server->outqueue->message_after_mod);
+                if (pos)
+                    pos[0] = '\r';
+            }
+            if (server_send (server, server->outqueue->message_after_mod,
+                             strlen (server->outqueue->message_after_mod)) <= 0)
+            {
+                irc_display_prefix (server, server->buffer, PREFIX_ERROR);
+                gui_printf (server->buffer, _("%s error sending data to IRC server\n"),
+                            WEECHAT_ERROR);
+            }
+            server->last_user_message = time_now;
+            server_outqueue_free (server, server->outqueue);
+        }
+    }
+}
+
+/*
+ * server_send_one_msg: send one message to IRC server
+ */
+
+int
+server_send_one_msg (t_irc_server *server, char *message)
+{
+    static char buffer[4096];
+    char *new_msg, *ptr_msg, *pos;
+    int rc, queue, first_message;
+    time_t time_now;
+    
+    rc = 1;
+    
+#ifdef DEBUG
+    gui_printf (server->buffer, "[DEBUG] Sending to server >>> %s\n", message);
+#endif
+#ifdef PLUGINS
+    new_msg = plugin_modifier_exec (PLUGIN_MODIFIER_IRC_OUT,
+                                    server->name,
+                                    message);
+#else
+    new_msg = NULL;
+#endif
+    
+    /* no changes in new message */
+    if (new_msg && (strcmp (buffer, new_msg) == 0))
+    {
+        free (new_msg);
+        new_msg = NULL;
+    }
+    
+    /* message not dropped? */
+    if (!new_msg || new_msg[0])
+    {
+        first_message = 1;
+        ptr_msg = (new_msg) ? new_msg : message;
+        
+        while (rc && ptr_msg && ptr_msg[0])
+        {
+            pos = strchr (ptr_msg, '\n');
+            if (pos)
+                pos[0] = '\0';
+            
+            snprintf (buffer, sizeof (buffer) - 1, "%s\r\n", ptr_msg);
+
+            /* anti-flood: look whether we should queue outgoing message or not */
+            time_now = time (NULL);
+            queue = 0;
+            if ((server->queue_msg)
+                && ((server->outqueue)
+                    || ((cfg_irc_anti_flood > 0)
+                        && (time_now - server->last_user_message < cfg_irc_anti_flood))))
+                queue = 1;
+            
+            /* if queue, then only queue message and send nothing now */
+            if (queue)
+            {
+                server_outqueue_add (server,
+                                     (new_msg && first_message) ? message : NULL,
+                                     buffer,
+                                     (new_msg) ? 1 : 0);
+            }
+            else
+            {
+                if (first_message)
+                    gui_printf_raw_data (server, 1, 0, message);
+                if (new_msg)
+                    gui_printf_raw_data (server, 1, 1, ptr_msg);
+                if (server_send (server, buffer, strlen (buffer)) <= 0)
+                {
+                    irc_display_prefix (server, server->buffer, PREFIX_ERROR);
+                    gui_printf (server->buffer, _("%s error sending data to IRC server\n"),
+                                WEECHAT_ERROR);
+                    rc = 0;
+                }
+                else
+                {
+                    if (server->queue_msg)
+                        server->last_user_message = time_now;
+                }
+            }
+            if (pos)
+            {
+                pos[0] = '\n';
+                ptr_msg = pos + 1;
+            }
+            else
+                ptr_msg = NULL;
+            
+            first_message = 0;
+        }
+    }
+    else
+        gui_printf_raw_data (server, 1, 1, _("(message dropped)"));
+    if (new_msg)
+        free (new_msg);
+    
+    return rc;
+}
+
 /*
  * server_sendf: send formatted data to IRC server
+ *               many messages may be sent, separated by '\n'
  */
 
 void
@@ -562,34 +689,81 @@ server_sendf (t_irc_server *server, char *fmt, ...)
 {
     va_list args;
     static char buffer[4096];
-    int size_buf;
+    char *ptr_buf, *pos;
+    int rc;
     
     if (!server)
         return;
     
     va_start (args, fmt);
-    size_buf = vsnprintf (buffer, sizeof (buffer) - 1, fmt, args);
-    va_end (args);
-
-    if ((size_buf == 0) || (strcmp (buffer, "\r\n") == 0))
-        return;
+    vsnprintf (buffer, sizeof (buffer) - 1, fmt, args);
+    va_end (args);    
     
-    buffer[sizeof (buffer) - 1] = '\0';
-    if ((size_buf < 0) || (size_buf > (int) (sizeof (buffer) - 1)))
-        size_buf = strlen (buffer);
-    
-    buffer[size_buf - 2] = '\0';
-    gui_printf_raw_data (server, 1, buffer);
-#ifdef DEBUG
-    gui_printf (server->buffer, "[DEBUG] Sending to server >>> %s\n", buffer);
-#endif
-    buffer[size_buf - 2] = '\r';
-    
-    if (server_send (server, buffer, strlen (buffer)) <= 0)
+    ptr_buf = buffer;
+    while (ptr_buf && ptr_buf[0])
     {
-        irc_display_prefix (server, server->buffer, PREFIX_ERROR);
-        gui_printf (server->buffer, _("%s error sending data to IRC server\n"),
-                    WEECHAT_ERROR);
+        pos = strchr (ptr_buf, '\n');
+        if (pos)
+            pos[0] = '\0';
+        
+        rc = server_send_one_msg (server, ptr_buf);
+        
+        if (pos)
+        {
+            pos[0] = '\n';
+            ptr_buf = pos + 1;
+        }
+        else
+            ptr_buf = NULL;
+        
+        if (!rc)
+            ptr_buf = NULL;
+    }
+}
+
+/*
+ * server_parse_message: parse IRC message and return pointer to
+ *                       host, command and arguments (if any)
+ */
+
+void
+server_parse_message (char *message, char **host, char **command, char **args)
+{
+    char *pos, *pos2;
+    
+    *host = NULL;
+    *command = NULL;
+    *args = NULL;
+    
+    if (message[0] == ':')
+    {
+        pos = strchr (message, ' ');
+        if (pos)
+        {
+            *host = strndup (message + 1, pos - (message + 1));
+            pos++;
+        }
+        else
+            pos = message;
+    }
+    else
+        pos = message;
+    
+    if (pos && pos[0])
+    {
+        while (pos[0] == ' ')
+            pos++;
+        pos2 = strchr (pos, ' ');
+        if (pos2)
+        {
+            *command = strndup (pos, pos2 - pos);
+            pos2++;
+            while (pos2[0] == ' ')
+                pos2++;
+            if (pos2[0] == ':')
+                pos2++;
+            *args = strdup (pos2);
+        }
     }
 }
 
@@ -735,7 +909,7 @@ void
 server_msgq_flush ()
 {
     t_irc_message *next;
-    char *entire_line, *ptr_data, *pos, *pos2;
+    char *ptr_data, *new_msg, *ptr_msg, *pos;
     char *host, *command, *args;
     
     while (recv_msgq)
@@ -745,84 +919,90 @@ server_msgq_flush ()
 #ifdef DEBUG
             gui_printf (gui_current_window->buffer, "[DEBUG] %s\n", recv_msgq->data);
 #endif
-            
             ptr_data = recv_msgq->data;
-            entire_line = strdup (ptr_data);
-            
             while (ptr_data[0] == ' ')
                 ptr_data++;
             
-            if (ptr_data && ptr_data[0])
+            if (ptr_data[0])
             {
-                gui_printf_raw_data (recv_msgq->server, 0, ptr_data);
+                gui_printf_raw_data (recv_msgq->server, 0, 0, ptr_data);
 #ifdef DEBUG
                 gui_printf (NULL, "[DEBUG] data received from server: %s\n", ptr_data);
 #endif
-                
-                host = NULL;
-                command = NULL;
-                args = ptr_data;
-                
-                if (ptr_data[0] == ':')
+#ifdef PLUGINS
+                new_msg = plugin_modifier_exec (PLUGIN_MODIFIER_IRC_IN,
+                                                recv_msgq->server->name,
+                                                ptr_data);
+#else
+                new_msg = NULL;
+#endif
+                /* no changes in new message */
+                if (new_msg && (strcmp (ptr_data, new_msg) == 0))
                 {
-                    pos = strchr (ptr_data, ' ');
-                    if (pos)
+                    free (new_msg);
+                    new_msg = NULL;
+                }
+                
+                /* message not dropped? */
+                if (!new_msg || new_msg[0])
+                {
+                    /* use new message (returned by plugin) */
+                    ptr_msg = (new_msg) ? new_msg : ptr_data;
+                    
+                    while (ptr_msg && ptr_msg[0])
                     {
-                        pos[0] = '\0';
-                        host = ptr_data + 1;
-                        pos++;
+                        pos = strchr (ptr_msg, '\n');
+                        if (pos)
+                            pos[0] = '\0';
+
+                        if (new_msg)
+                            gui_printf_raw_data (recv_msgq->server, 0, 1, ptr_msg);
+                        
+                        server_parse_message (ptr_msg, &host, &command, &args);
+                        
+                        switch (irc_recv_command (recv_msgq->server, ptr_msg, host, command, args))
+                        {
+                            case -1:
+                                irc_display_prefix (recv_msgq->server,
+                                                    recv_msgq->server->buffer, PREFIX_ERROR);
+                                gui_printf (recv_msgq->server->buffer,
+                                            _("%s Command \"%s\" failed!\n"), WEECHAT_ERROR, command);
+                                break;
+                            case -2:
+                                irc_display_prefix (recv_msgq->server,
+                                                    recv_msgq->server->buffer, PREFIX_ERROR);
+                                gui_printf (recv_msgq->server->buffer,
+                                            _("%s No command to execute!\n"), WEECHAT_ERROR);
+                                break;
+                            case -3:
+                                irc_display_prefix (recv_msgq->server,
+                                                    recv_msgq->server->buffer, PREFIX_ERROR);
+                                gui_printf (recv_msgq->server->buffer,
+                                            _("%s Unknown command: cmd=\"%s\", host=\"%s\", args=\"%s\"\n"),
+                                            WEECHAT_WARNING, command, host, args);
+                                break;
+                        }
+                        if (host)
+                            free (host);
+                        if (command)
+                            free (command);
+                        if (args)
+                            free (args);
+                        
+                        if (pos)
+                        {
+                            pos[0] = '\n';
+                            ptr_msg = pos + 1;
+                        }
+                        else
+                            ptr_msg = NULL;
                     }
-                    else
-                        pos = ptr_data;
                 }
                 else
-                    pos = ptr_data;
-                
-                if (pos && pos[0])
-                {
-                    while (pos[0] == ' ')
-                        pos++;
-                    pos2 = strchr (pos, ' ');
-                    if (pos2)
-                    {
-                        pos2[0] = '\0';
-                        command = strdup (pos);
-                        pos2++;
-                        while (pos2[0] == ' ')
-                            pos2++;
-                        args = (pos2[0] == ':') ? pos2 + 1 : pos2;
-                    }
-                }
-                
-                switch (irc_recv_command (recv_msgq->server, entire_line, host,
-                                          command, args))
-                {
-                    case -1:
-                        irc_display_prefix (recv_msgq->server,
-                                            recv_msgq->server->buffer, PREFIX_ERROR);
-                        gui_printf (recv_msgq->server->buffer,
-                                    _("%s Command \"%s\" failed!\n"), WEECHAT_ERROR, command);
-                        break;
-                    case -2:
-                        irc_display_prefix (recv_msgq->server,
-                                            recv_msgq->server->buffer, PREFIX_ERROR);
-                        gui_printf (recv_msgq->server->buffer,
-                                    _("%s No command to execute!\n"), WEECHAT_ERROR);
-                        break;
-                    case -3:
-                        irc_display_prefix (recv_msgq->server,
-                                            recv_msgq->server->buffer, PREFIX_ERROR);
-                        gui_printf (recv_msgq->server->buffer,
-                                    _("%s Unknown command: cmd=\"%s\", host=\"%s\", args=\"%s\"\n"),
-                                    WEECHAT_WARNING, command, host, args);
-                        break;
-                }
-                
-                if (command)
-                    free (command);
+                    gui_printf_raw_data (recv_msgq->server, 0, 1, _("(message dropped)"));
+                if (new_msg)
+                    free (new_msg);
             }
-            
-            free (entire_line);
             free (recv_msgq->data);
         }
         
@@ -928,6 +1108,7 @@ server_close_connection (t_irc_server *server)
         free (server->unterminated_message);
         server->unterminated_message = NULL;
     }
+    server_outqueue_free_all (server);
     
     /* server is now disconnected */
     server->is_connected = 0;
@@ -974,7 +1155,7 @@ server_child_read (t_irc_server *server)
                 if (server->ssl_connected)
                 {
                     gnutls_transport_set_ptr (server->gnutls_sess,
-                                              (gnutls_transport_ptr) server->sock);
+                                              (gnutls_transport_ptr) ((unsigned long) server->sock));
                     if (gnutls_handshake (server->gnutls_sess) < 0)
                     {
                         irc_display_prefix (server, server->buffer, PREFIX_ERROR);
@@ -1740,6 +1921,8 @@ server_disconnect (t_irc_server *server, int reconnect)
             nick_free_all (ptr_channel);
             irc_display_prefix (NULL, ptr_channel->buffer, PREFIX_INFO);
             gui_printf (ptr_channel->buffer, _("Disconnected from server!\n"));
+            gui_nicklist_draw (ptr_channel->buffer, 1, 1);
+            gui_status_draw (ptr_channel->buffer, 1);
         }
     }
     
@@ -2013,9 +2196,6 @@ server_print_log (t_irc_server *server)
     weechat_log_printf ("  autojoin. . . . . . : '%s'\n", server->autojoin);
     weechat_log_printf ("  autorejoin. . . . . : %d\n",   server->autorejoin);
     weechat_log_printf ("  notify_levels . . . : %s\n",   server->notify_levels);
-    weechat_log_printf ("  charset_decode_iso. : %s\n",   server->charset_decode_iso);
-    weechat_log_printf ("  charset_decode_utf. : %s\n",   server->charset_decode_utf);
-    weechat_log_printf ("  charset_encode. . . : %s\n",   server->charset_encode);
     weechat_log_printf ("  child_pid . . . . . : %d\n",   server->child_pid);
     weechat_log_printf ("  child_read  . . . . : %d\n",   server->child_read);
     weechat_log_printf ("  child_write . . . . : %d\n",   server->child_write);
@@ -2035,6 +2215,9 @@ server_print_log (t_irc_server *server)
                         server->lag_check_time.tv_sec,
                         server->lag_check_time.tv_usec);
     weechat_log_printf ("  lag_next_check. . . : %ld\n",  server->lag_next_check);
+    weechat_log_printf ("  last_user_message . : %ld\n",  server->last_user_message);
+    weechat_log_printf ("  outqueue. . . . . . : 0x%X\n", server->outqueue);
+    weechat_log_printf ("  last_outqueue . . . : 0x%X\n", server->last_outqueue);
     weechat_log_printf ("  buffer. . . . . . . : 0x%X\n", server->buffer);
     weechat_log_printf ("  channels. . . . . . : 0x%X\n", server->channels);
     weechat_log_printf ("  last_channel. . . . : 0x%X\n", server->last_channel);
