@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,10 @@
 
 #include "../gui/gui.h"
 
+#ifndef NI_MAXHOST
+#define NI_MAXHOST 256
+#endif
+
 /* prefixes for chat window */
 
 #define PREFIX_SERVER    "-@-"
@@ -46,6 +50,8 @@
 #define PREFIX_QUIT      "<--"
 #define PREFIX_ERROR     "=!="
 #define PREFIX_PLUGIN    "-P-"
+#define PREFIX_RECV_MOD  "==>"
+#define PREFIX_SEND_MOD  "<=="
 
 #define DEFAULT_IRC_PORT 6667
 
@@ -63,6 +69,14 @@
     else \
         nick->flags &= 0xFFFF - flag;
 
+#define server_sendf_queued(server, fmt, argz...) \
+    if (server) \
+    { \
+        server->queue_msg = 1; \
+        server_sendf(server, fmt, ##argz); \
+        server->queue_msg = 0; \
+    }
+
 typedef struct t_irc_nick t_irc_nick;
 
 struct t_irc_nick
@@ -111,6 +125,17 @@ struct t_irc_channel
 
 /* server types */
 
+typedef struct t_irc_outqueue t_irc_outqueue;
+
+struct t_irc_outqueue
+{
+    char *message_before_mod;       /* message before any modifier            */
+    char *message_after_mod;        /* message after modifier(s)              */
+    int modified;                   /* message was modified by modifier(s)    */
+    t_irc_outqueue *next_outqueue;  /* pointer to next message in queue       */
+    t_irc_outqueue *prev_outqueue;  /* pointer to previous message in queue   */
+};
+
 typedef struct t_irc_server t_irc_server;
 
 struct t_irc_server
@@ -137,9 +162,6 @@ struct t_irc_server
     char *autojoin;                 /* channels to automatically join         */
     int autorejoin;                 /* auto rejoin channels when kicked       */
     char *notify_levels;            /* channels notify levels                 */
-    char *charset_decode_iso;       /* channels charsets for decoding ISO     */
-    char *charset_decode_utf;       /* channels charsets for decoding UTF     */
-    char *charset_encode;           /* channels charsets for encoding msgs    */
     
     /* internal vars */
     pid_t child_pid;                /* pid of child process (connecting)      */
@@ -162,7 +184,11 @@ struct t_irc_server
     int lag;                        /* lag (in milliseconds)                  */
     struct timeval lag_check_time;  /* last time lag was checked (ping sent)  */
     time_t lag_next_check;          /* time for next check                    */
-    regex_t *cmd_list_regexp;       /* compiled Regular Expression for /list  */ 
+    regex_t *cmd_list_regexp;       /* compiled Regular Expression for /list  */
+    int queue_msg;                  /* set to 1 when queue (out) is required  */
+    time_t last_user_message;       /* time of last user message (anti flood) */
+    t_irc_outqueue *outqueue;       /* queue for outgoing user messages       */
+    t_irc_outqueue *last_outqueue;  /* last outgoing user message             */
     t_gui_buffer *buffer;           /* GUI buffer allocated for server        */
     t_gui_buffer *saved_buffer;     /* channel before jumping to next server  */
     t_irc_channel *channels;        /* opened channels on server              */
@@ -329,18 +355,20 @@ extern t_irc_ignore *irc_last_ignore;
 extern void server_init (t_irc_server *);
 extern int server_init_with_url (char *, t_irc_server *);
 extern t_irc_server *server_alloc ();
+extern void server_outqueue_free_all (t_irc_server *);
 extern void server_destroy (t_irc_server *);
 extern void server_free (t_irc_server *);
 extern void server_free_all ();
 extern t_irc_server *server_new (char *, int, int, int, int, char *, int, int, int,
                                  char *, char *, char *, char *, char *, char *,
-                                 char *, char *, int, char *, int, char *, char *,
-                                 char *, char *);
+                                 char *, char *, int, char *, int, char *);
 extern char *server_get_charset_decode_iso (t_irc_server *);
 extern char *server_get_charset_decode_utf (t_irc_server *);
 extern char *server_get_charset_encode (t_irc_server *);
 extern int server_send (t_irc_server *, char *, int);
+extern void server_outqueue_send (t_irc_server *);
 extern void server_sendf (t_irc_server *, char *, ...);
+extern void server_parse_message (char *, char **, char **, char **);
 extern void server_recv (t_irc_server *);
 extern void server_child_read (t_irc_server *);
 extern int server_connect (t_irc_server *);
@@ -381,8 +409,6 @@ extern int string_is_channel (char *);
 extern char *channel_get_charset_decode_iso (t_irc_server *, t_irc_channel *);
 extern char *channel_get_charset_decode_utf (t_irc_server *, t_irc_channel *);
 extern char *channel_get_charset_encode (t_irc_server *, t_irc_channel *);
-extern char *channel_iconv_decode (t_irc_server *, t_irc_channel *, char *);
-extern char *channel_iconv_encode (t_irc_server *, t_irc_channel *, char *);
 extern void channel_remove_away (t_irc_channel *);
 extern void channel_check_away (t_irc_server *, t_irc_channel *, int);
 extern void channel_set_away (t_irc_channel *, char *, int);
@@ -471,6 +497,7 @@ extern int irc_cmd_send_list (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_lusers (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_me (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_mode (t_irc_server *, t_irc_channel *, char *);
+extern void irc_send_mode_nicks (t_irc_server *, char *, char *, char *, int, char **);
 extern int irc_cmd_send_motd (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_msg (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_names (t_irc_server *, t_irc_channel *, char *);
