@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,18 +19,22 @@
 
 /* weechat-lua.c: Lua plugin support for WeeChat */
 
+#undef _
 
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
 #include <time.h>
+
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#undef _
+
 #include "../../weechat-plugin.h"
 #include "../weechat-script.h"
 
@@ -51,11 +55,14 @@ lua_State *lua_current_interpreter = NULL;
  * weechat_lua_exec: execute a Lua script
  */
 
-int
+void *
 weechat_lua_exec (t_weechat_plugin *plugin,
 		  t_plugin_script *script,
+		  int ret_type,
 		  char *function, char *arg1, char *arg2, char *arg3)
 {
+    void *ret_value;
+    int *ret_i;
 
     lua_current_interpreter = script->interpreter;
 
@@ -82,10 +89,27 @@ weechat_lua_exec (t_weechat_plugin *plugin,
 	plugin->print_server (plugin,
                               "Lua error: %s",
                               lua_tostring (lua_current_interpreter, -1));
-        return PLUGIN_RC_KO;
+        return NULL;
     }
     
-    return lua_tonumber (lua_current_interpreter, -1);
+    if (ret_type == SCRIPT_EXEC_STRING)
+	ret_value = strdup ((char *) lua_tostring (lua_current_interpreter, -1));
+    else if (ret_type == SCRIPT_EXEC_INT)
+    {
+	ret_i = (int *) malloc (sizeof(int));
+	if (ret_i)
+	    *ret_i = lua_tonumber (lua_current_interpreter, -1);
+	ret_value = ret_i;
+    }
+    else
+    {
+	lua_plugin->print_server (lua_plugin, 
+				  "Lua error: wrong parameters for function \"%s\"",
+				  function);
+	return NULL;
+    }
+    
+    return ret_value; 
 }
 
 /*
@@ -97,9 +121,23 @@ weechat_lua_cmd_msg_handler (t_weechat_plugin *plugin,
                              int argc, char **argv,
                              char *handler_args, void *handler_pointer)
 {
+    int *r;
+    int ret;
+    
     if (argc >= 3)
-        return weechat_lua_exec (plugin, (t_plugin_script *)handler_pointer,
-                                 handler_args, argv[0], argv[2], NULL);
+    {
+        r = (int *) weechat_lua_exec (plugin, (t_plugin_script *)handler_pointer,
+				      SCRIPT_EXEC_INT,
+				      handler_args, argv[0], argv[2], NULL);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
@@ -116,9 +154,20 @@ weechat_lua_timer_handler (t_weechat_plugin *plugin,
     /* make gcc happy */
     (void) argc;
     (void) argv;
+    int *r;
+    int ret;
     
-    return weechat_lua_exec (plugin, (t_plugin_script *)handler_pointer,
-                             handler_args, NULL, NULL, NULL);
+    r = (int *) weechat_lua_exec (plugin, (t_plugin_script *)handler_pointer,
+				  SCRIPT_EXEC_INT,
+				  handler_args, NULL, NULL, NULL);
+    if (r == NULL)
+	ret = PLUGIN_RC_KO;
+    else
+    {
+	ret = *r;
+	free (r);
+    }
+    return ret;
 }
 
 /*
@@ -130,13 +179,44 @@ weechat_lua_keyboard_handler (t_weechat_plugin *plugin,
                               int argc, char **argv,
                               char *handler_args, void *handler_pointer)
 {
-    if (argc >= 2)
-        return weechat_lua_exec (plugin, (t_plugin_script *)handler_pointer,
-                                 handler_args, argv[0], argv[1], argv[2]);
+    int *r;
+    int ret;
+    
+    if (argc >= 3)
+    {
+        r = (int *) weechat_lua_exec (plugin, (t_plugin_script *)handler_pointer,
+				      SCRIPT_EXEC_INT,
+				      handler_args, argv[0], argv[1], argv[2]);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
 
+/*
+ * weechat_lua_modifier: general modifier for Lua
+ */
+
+char *
+weechat_lua_modifier (t_weechat_plugin *plugin,
+                      int argc, char **argv,
+                      char *modifier_args, void *modifier_pointer)
+{
+    if (argc >= 2)
+        return (char *) weechat_lua_exec (plugin, (t_plugin_script *)modifier_pointer,
+					  SCRIPT_EXEC_STRING,
+					  modifier_args, argv[0], argv[1], NULL);
+    else
+        return NULL;
+}
+
 /*
  * weechat_lua_register: startup function for all WeeChat Lua scripts
  */
@@ -144,7 +224,7 @@ weechat_lua_keyboard_handler (t_weechat_plugin *plugin,
 static int
 weechat_lua_register (lua_State *L)
 {
-    const char *name, *version, *shutdown_func, *description;
+    const char *name, *version, *shutdown_func, *description, *charset;
     int n;
     
     /* make gcc happy */
@@ -156,10 +236,11 @@ weechat_lua_register (lua_State *L)
     version = NULL;
     shutdown_func = NULL;
     description = NULL;
+    charset = NULL;
     
     n = lua_gettop (lua_current_interpreter);
 
-    if (n != 4)
+    if ((n < 4) || (n > 5))
     {
 	lua_plugin->print_server (lua_plugin,
                                   "Lua error: wrong parameters for "
@@ -167,12 +248,24 @@ weechat_lua_register (lua_State *L)
         lua_pushnumber (lua_current_interpreter, 0);
 	return 1;
     }
-    
-    name = lua_tostring (lua_current_interpreter, -4);
-    version = lua_tostring (lua_current_interpreter, -3);
-    shutdown_func = lua_tostring (lua_current_interpreter, -2);
-    description = lua_tostring (lua_current_interpreter, -1);
 
+    switch (n)
+    {
+        case 4:
+            name = lua_tostring (lua_current_interpreter, -4);
+            version = lua_tostring (lua_current_interpreter, -3);
+            shutdown_func = lua_tostring (lua_current_interpreter, -2);
+            description = lua_tostring (lua_current_interpreter, -1);
+            break;
+        case 5:
+            name = lua_tostring (lua_current_interpreter, -5);
+            version = lua_tostring (lua_current_interpreter, -4);
+            shutdown_func = lua_tostring (lua_current_interpreter, -3);
+            description = lua_tostring (lua_current_interpreter, -2);
+            charset = lua_tostring (lua_current_interpreter, -1);
+            break;
+    }
+    
     if (weechat_script_search (lua_plugin, &lua_scripts, (char *) name))
     {
         /* error: another scripts already exists with this name! */
@@ -193,7 +286,8 @@ weechat_lua_register (lua_State *L)
 					     (char *) name, 
 					     (char *) version, 
 					     (char *) shutdown_func,
-					     (char *) description);
+					     (char *) description,
+                                             (char *) charset);
     if (lua_current_script)
     {
         lua_plugin->print_server (lua_plugin,
@@ -211,6 +305,51 @@ weechat_lua_register (lua_State *L)
     return 1;
 }
 
+/*
+ * weechat_lua_set_charset: set script charset
+ */
+
+static int
+weechat_lua_set_charset  (lua_State *L)
+{
+    const char *charset;
+    int n;
+    
+    /* make gcc happy */
+    (void) L;
+     
+    if (!lua_current_script)
+    {
+        lua_plugin->print_server (lua_plugin,
+                                  "Lua error: unable to set charset, "
+                                  "script not initialized");	
+	lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+    
+    charset = NULL;
+ 
+    n = lua_gettop (lua_current_interpreter);
+
+    if (n != 1)
+    {
+	lua_plugin->print_server (lua_plugin,
+                                  "Lua error: wrong parameters for "
+                                  "\"set_charset\" function");
+        lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+    
+    charset = lua_tostring (lua_current_interpreter, -1);
+    
+    weechat_script_set_charset (lua_plugin,
+                                lua_current_script,
+                                (char *) charset);
+    
+    lua_pushnumber (lua_current_interpreter, 1);
+    return 1;
+}
+
 /*
  * weechat_lua_print: print message into a buffer (current or specified one)
  */
@@ -871,6 +1010,109 @@ weechat_lua_remove_keyboard_handler (lua_State *L)
     return 1;
 }
 
+/*
+ * weechat_lua_add_modifier: add a modifier
+ */
+
+static int
+weechat_lua_add_modifier (lua_State *L)
+{
+    const char *type, *command, *function;
+    int n;
+    
+    /* make gcc happy */
+    (void) L;
+    
+    if (!lua_current_script)
+    {
+        lua_plugin->print_server (lua_plugin,
+                                  "Lua error: unable to add modifier, "
+                                  "script not initialized");
+	lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+
+    type = NULL;
+    command = NULL;
+    function = NULL;
+    
+    n = lua_gettop (lua_current_interpreter);
+
+    if (n != 3)
+    {
+	lua_plugin->print_server (lua_plugin,
+                                  "Lua error: wrong parameters for "
+                                  "\"add_modifier\" function");
+        lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+    
+    type = lua_tostring (lua_current_interpreter, -3);
+    command = lua_tostring (lua_current_interpreter, -2);
+    function = lua_tostring (lua_current_interpreter, -1);
+    
+    if (!lua_plugin->modifier_add (lua_plugin, (char *)type, (char *)command,
+                                   weechat_lua_modifier,
+                                   (char *)function,
+                                   (void *)lua_current_script))
+    {
+	lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+    
+    lua_pushnumber (lua_current_interpreter, 1);
+    return 1;
+}
+
+/*
+ * weechat_lua_remove_modifier: remove a modifier
+ */
+
+static int
+weechat_lua_remove_modifier (lua_State *L)
+{
+    const char *type, *command, *function;
+    int n;
+    
+    /* make gcc happy */
+    (void) L;
+     
+    if (!lua_current_script)
+    {
+        lua_plugin->print_server (lua_plugin,
+                                  "Lua error: unable to remove modifier, "
+                                  "script not initialized");
+        lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+    
+    type = NULL;
+    command = NULL;
+    function = NULL;
+ 
+    n = lua_gettop (lua_current_interpreter);
+    
+    if (n != 3)
+    {
+	lua_plugin->print_server (lua_plugin,
+                                  "Lua error: wrong parameters for "
+                                  "\"remove_modifier\" function");
+        lua_pushnumber (lua_current_interpreter, 0);
+	return 1;
+    }
+
+    type = lua_tostring (lua_current_interpreter, -3);
+    command = lua_tostring (lua_current_interpreter, -2);
+    function = lua_tostring (lua_current_interpreter, -1);
+    
+    weechat_script_remove_modifier (lua_plugin, lua_current_script,
+                                    (char *)type, (char *)command,
+                                    (char *)function);
+    
+    lua_pushnumber (lua_current_interpreter, 1);
+    return 1;
+}
+
 /*
  * weechat_lua_get_info: get various infos
  */
@@ -1348,18 +1590,6 @@ weechat_lua_get_server_info  (lua_State *L)
 	lua_pushstring (lua_current_interpreter, ptr_server->notify_levels);
 	lua_rawset (lua_current_interpreter, -3);
 	
-	lua_pushstring (lua_current_interpreter, "charset_decode_iso");
-	lua_pushstring (lua_current_interpreter, ptr_server->charset_decode_iso);
-	lua_rawset (lua_current_interpreter, -3);
-	
-	lua_pushstring (lua_current_interpreter, "charset_decode_utf");
-	lua_pushstring (lua_current_interpreter, ptr_server->charset_decode_utf);
-	lua_rawset (lua_current_interpreter, -3);
-	
-	lua_pushstring (lua_current_interpreter, "charset_encode");
-	lua_pushstring (lua_current_interpreter, ptr_server->charset_encode);
-	lua_rawset (lua_current_interpreter, -3);
-	
 	lua_pushstring (lua_current_interpreter, "is_connected");
 	lua_pushnumber (lua_current_interpreter, ptr_server->is_connected);
 	lua_rawset (lua_current_interpreter, -3);
@@ -1740,6 +1970,7 @@ weechat_lua_get_buffer_data  (lua_State *L)
 {
     t_plugin_buffer_line *buffer_data, *ptr_data;
     const char *server, *channel;
+    char timebuffer[64];
     int i, n;
     
     /* make gcc happy */
@@ -1792,6 +2023,13 @@ weechat_lua_get_buffer_data  (lua_State *L)
 	lua_pushnumber (lua_current_interpreter, i);
 	lua_newtable (lua_current_interpreter);
 
+	strftime(timebuffer, sizeof(timebuffer), "%F %T",
+		 localtime(&ptr_data->date));
+
+	lua_pushstring (lua_current_interpreter, "date");
+	lua_pushstring (lua_current_interpreter, timebuffer);
+	lua_rawset (lua_current_interpreter, -3);
+
 	lua_pushstring (lua_current_interpreter, "nick");
 	lua_pushstring (lua_current_interpreter,
 			ptr_data->nick == NULL ? "" : ptr_data->nick);
@@ -1870,40 +2108,43 @@ weechat_lua_constant_plugin_rc_ok_ignore_all  (lua_State *L)
 
 static
 const struct luaL_reg weechat_lua_funcs[] = {
-    { "register", weechat_lua_register},
-    { "print", weechat_lua_print},
-    { "print_server", weechat_lua_print_server},
-    { "print_infobar", weechat_lua_print_infobar},
-    { "remove_infobar", weechat_lua_remove_infobar},
-    { "log", weechat_lua_log},
-    { "command", weechat_lua_command},
-    { "add_message_handler", weechat_lua_add_message_handler},
-    { "add_command_handler", weechat_lua_add_command_handler},
-    { "add_timer_handler", weechat_lua_add_timer_handler},
-    { "add_keyboard_handler", weechat_lua_add_keyboard_handler},
-    { "remove_handler", weechat_lua_remove_handler},
-    { "remove_timer_handler", weechat_lua_remove_timer_handler},
-    { "remove_keyboard_handler", weechat_lua_remove_keyboard_handler},
-    { "get_info", weechat_lua_get_info},
-    { "get_dcc_info", weechat_lua_get_dcc_info},
-    { "get_config", weechat_lua_get_config},
-    { "set_config", weechat_lua_set_config},
-    { "get_plugin_config", weechat_lua_get_plugin_config},
-    { "set_plugin_config", weechat_lua_set_plugin_config},
-    { "get_server_info", weechat_lua_get_server_info},
-    { "get_channel_info", weechat_lua_get_channel_info},
-    { "get_nick_info", weechat_lua_get_nick_info},
-    { "get_irc_color", weechat_lua_get_irc_color},
-    { "get_window_info", weechat_lua_get_window_info},
-    { "get_buffer_info", weechat_lua_get_buffer_info},
-    { "get_buffer_data", weechat_lua_get_buffer_data},
+    { "register", weechat_lua_register },
+    { "set_charset", weechat_lua_set_charset },
+    { "print", weechat_lua_print },
+    { "print_server", weechat_lua_print_server },
+    { "print_infobar", weechat_lua_print_infobar },
+    { "remove_infobar", weechat_lua_remove_infobar },
+    { "log", weechat_lua_log },
+    { "command", weechat_lua_command },
+    { "add_message_handler", weechat_lua_add_message_handler },
+    { "add_command_handler", weechat_lua_add_command_handler },
+    { "add_timer_handler", weechat_lua_add_timer_handler },
+    { "add_keyboard_handler", weechat_lua_add_keyboard_handler },
+    { "remove_handler", weechat_lua_remove_handler },
+    { "remove_timer_handler", weechat_lua_remove_timer_handler },
+    { "remove_keyboard_handler", weechat_lua_remove_keyboard_handler },
+    { "add_modifier", weechat_lua_add_modifier },
+    { "remove_modifier", weechat_lua_remove_modifier },
+    { "get_info", weechat_lua_get_info },
+    { "get_dcc_info", weechat_lua_get_dcc_info },
+    { "get_config", weechat_lua_get_config },
+    { "set_config", weechat_lua_set_config },
+    { "get_plugin_config", weechat_lua_get_plugin_config },
+    { "set_plugin_config", weechat_lua_set_plugin_config },
+    { "get_server_info", weechat_lua_get_server_info },
+    { "get_channel_info", weechat_lua_get_channel_info },
+    { "get_nick_info", weechat_lua_get_nick_info },
+    { "get_irc_color", weechat_lua_get_irc_color },
+    { "get_window_info", weechat_lua_get_window_info },
+    { "get_buffer_info", weechat_lua_get_buffer_info },
+    { "get_buffer_data", weechat_lua_get_buffer_data },
     /* define constants as function which returns values */
-    { "PLUGIN_RC_OK", weechat_lua_constant_plugin_rc_ok},
-    { "PLUGIN_RC_KO", weechat_lua_constant_plugin_rc_ko},
-    { "PLUGIN_RC_OK_IGNORE_WEECHAT", weechat_lua_constant_plugin_rc_ok_ignore_weechat},
-    { "PLUGIN_RC_OK_IGNORE_PLUGINS", weechat_lua_constant_plugin_rc_ok_ignore_plugins},
-    { "PLUGIN_RC_OK_IGNORE_ALL", weechat_lua_constant_plugin_rc_ok_ignore_all},
-    { NULL, NULL}
+    { "PLUGIN_RC_OK", weechat_lua_constant_plugin_rc_ok },
+    { "PLUGIN_RC_KO", weechat_lua_constant_plugin_rc_ko },
+    { "PLUGIN_RC_OK_IGNORE_WEECHAT", weechat_lua_constant_plugin_rc_ok_ignore_weechat },
+    { "PLUGIN_RC_OK_IGNORE_PLUGINS", weechat_lua_constant_plugin_rc_ok_ignore_plugins },
+    { "PLUGIN_RC_OK_IGNORE_ALL", weechat_lua_constant_plugin_rc_ok_ignore_all },
+    { NULL, NULL }
 };
 
 int
@@ -1924,12 +2165,11 @@ weechat_lua_load (t_weechat_plugin *plugin, char *filename)
     
     if ((fp = fopen (filename, "r")) == NULL)
     {
-        plugin->print_server (plugin,
-                              "Lua error: unable to open file \"%s\"",
+        plugin->print_server (plugin, "Lua error: script \"%s\" not found",
                               filename);
         return 0;
     }
-
+    
     lua_current_script = NULL;
     
     lua_current_interpreter = lua_open ();
@@ -1942,16 +2182,24 @@ weechat_lua_load (t_weechat_plugin *plugin, char *filename)
         return 0;
     }
 
+#ifdef LUA_VERSION_NUM /* LUA_VERSION_NUM is defined only in lua >= 5.1.0 */
+    luaL_openlibs (lua_current_interpreter);
+#else
     luaopen_base (lua_current_interpreter);
-    luaopen_table (lua_current_interpreter);
-    luaopen_io (lua_current_interpreter);
     luaopen_string (lua_current_interpreter);
+    luaopen_table (lua_current_interpreter);
     luaopen_math (lua_current_interpreter);
+    luaopen_io (lua_current_interpreter);
     luaopen_debug (lua_current_interpreter);
+#endif
     
     luaL_openlib (lua_current_interpreter, "weechat", weechat_lua_funcs, 0);
-    
+
+#ifdef LUA_VERSION_NUM
+    if (luaL_dostring (lua_current_interpreter, weechat_lua_code) != 0)
+#else    
     if (lua_dostring (lua_current_interpreter, weechat_lua_code) != 0)
+#endif
         plugin->print_server (plugin,
                               "Lua warning: unable to redirect stdout and stderr");
     
@@ -2013,12 +2261,19 @@ weechat_lua_load (t_weechat_plugin *plugin, char *filename)
 void
 weechat_lua_unload (t_weechat_plugin *plugin, t_plugin_script *script)
 {
+    int *r;
+    
     plugin->print_server (plugin,
                           "Unloading Lua script \"%s\"",
                           script->name);
     
     if (script->shutdown_func[0])
-        weechat_lua_exec (plugin, script, script->shutdown_func, NULL, NULL, NULL);
+    {
+        r = weechat_lua_exec (plugin, script, SCRIPT_EXEC_INT,
+			      script->shutdown_func, NULL, NULL, NULL);
+	if (r)
+	    free (r);
+    }
     
     lua_close (script->interpreter);
     
@@ -2075,11 +2330,12 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
                  int cmd_argc, char **cmd_argv,
                  char *handler_args, void *handler_pointer)
 {
-    int argc, handler_found;
+    int argc, handler_found, modifier_found;
     char **argv, *path_script;
     t_plugin_script *ptr_script;
     t_plugin_handler *ptr_handler;
-    
+    t_plugin_modifier *ptr_modifier;
+
     /* make gcc happy */
     (void) handler_args;
     (void) handler_pointer;
@@ -2123,7 +2379,7 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_MESSAGE)
+                if ((ptr_handler->type == PLUGIN_HANDLER_MESSAGE)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2142,7 +2398,7 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_COMMAND)
+                if ((ptr_handler->type == PLUGIN_HANDLER_COMMAND)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2161,7 +2417,7 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_TIMER)
+                if ((ptr_handler->type == PLUGIN_HANDLER_TIMER)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2180,7 +2436,7 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_KEYBOARD)
+                if ((ptr_handler->type == PLUGIN_HANDLER_KEYBOARD)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2190,7 +2446,34 @@ weechat_lua_cmd (t_weechat_plugin *plugin,
             }
             if (!handler_found)
                 plugin->print_server (plugin, "  (none)");
-            break;
+            
+	    /* list Lua modifiers */
+	    plugin->print_server (plugin, "");
+            plugin->print_server (plugin, "Lua modifiers:");
+            modifier_found = 0;
+            for (ptr_modifier = plugin->modifiers;
+                 ptr_modifier; ptr_modifier = ptr_modifier->next_modifier)
+            {
+		modifier_found = 1;
+		if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_IN)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Lua(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_IN_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_USER)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Lua(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_USER_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_OUT)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Lua(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_OUT_STR,
+					  ptr_modifier->modifier_args);
+            }
+            if (!modifier_found)
+                plugin->print_server (plugin, "  (none)");
+	    break;
         case 1:
             if (plugin->ascii_strcasecmp (plugin, argv[0], "autoload") == 0)
                 weechat_script_auto_load (plugin, "lua", weechat_lua_load);
@@ -2252,7 +2535,7 @@ weechat_plugin_init (t_weechat_plugin *plugin)
                              "filename: Lua script (file) to load\n"
                              "script: script name to unload\n\n"
                              "Without argument, /lua command lists all loaded Lua scripts.",
-                             "load|autoload|reload|unload",
+                             "load|autoload|reload|unload %f",
                              weechat_lua_cmd, NULL, NULL);
 
     plugin->mkdir_home (plugin, "lua");
