@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,12 +19,24 @@
 
 /* weechat-perl.c: Perl plugin support for WeeChat */
 
+#undef _
 
-#include <stdlib.h>
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
-#undef _
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <stdarg.h>
+#include <time.h>
+
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "../../weechat-plugin.h"
 #include "../weechat-script.h"
 
@@ -100,17 +112,20 @@ char *perl_weechat_code =
  * weechat_perl_exec: execute a Perl script
  */
 
-int
+void *
 weechat_perl_exec (t_weechat_plugin *plugin,
                    t_plugin_script *script,
+		   int ret_type,
                    char *function, char *arg1, char *arg2, char *arg3)
 {
     char empty_arg[1] = { '\0' };
     char *func;
     char *argv[4];
     unsigned int count;
-    int return_code;
-    SV *sv;
+    void *ret_value;
+    int *ret_i;    
+    SV *ret_s;
+    int mem_err;
 
     /* this code is placed here to conform ISO C90 */
     dSP;
@@ -119,7 +134,7 @@ weechat_perl_exec (t_weechat_plugin *plugin,
     int size = strlen (script->interpreter) + strlen(function) + 3;
     func = (char *) malloc ( size * sizeof(char));
     if (!func)
-	return PLUGIN_RC_KO;
+	return NULL;
     snprintf (func, size, "%s::%s", (char *) script->interpreter, function);
 #else
     func = function;
@@ -129,6 +144,9 @@ weechat_perl_exec (t_weechat_plugin *plugin,
     ENTER;
     SAVETMPS;
     PUSHMARK(sp);
+    
+    perl_current_script = script;
+
     if (arg1)
     {
         argv[0] = (arg1) ? arg1 : empty_arg;
@@ -149,29 +167,50 @@ weechat_perl_exec (t_weechat_plugin *plugin,
     else
         argv[0] = NULL;
     
-    perl_current_script = script;
-    
     count = perl_call_argv (func, G_EVAL | G_SCALAR, argv);
-    
+    ret_value = NULL;
+    mem_err = 1;
+
     SPAGAIN;
     
-    sv = GvSV (gv_fetchpv ("@", TRUE, SVt_PV));
-    return_code = PLUGIN_RC_KO;
-    if (SvTRUE (sv))
+    if (SvTRUE (ERRSV))
     {
-        plugin->print_server (plugin, "Perl error: %s", SvPV (sv, count));
-        POPs;
+        plugin->print_server (plugin, "Perl error: %s", SvPV_nolen (ERRSV));
+        POPs; /* poping the 'undef' */	
+	mem_err = 0;
     }
     else
     {
         if (count != 1)
-        {
+	{
             plugin->print_server (plugin,
-                                  "Perl error: too much values from \"%s\" (%d). Expected: 1.",
+                                  "Perl error: function \"%s\" must return 1 valid value (%d)",
                                   function, count);
-        }
+	    mem_err = 0;
+	}
         else
-            return_code = POPi;
+	{
+	    if (ret_type == SCRIPT_EXEC_STRING)
+	    {
+		ret_s = newSVsv(POPs);
+		ret_value = strdup (SvPV_nolen (ret_s));
+		SvREFCNT_dec (ret_s);
+	    }
+	    else if (ret_type == SCRIPT_EXEC_INT)
+	    {
+		ret_i = (int *) malloc (sizeof(int));
+		if (ret_i)
+		    *ret_i = POPi;
+		ret_value = ret_i;
+	    }
+	    else
+	    {
+		plugin->print_server (plugin,
+				      "Perl error: function \"%s\" is internally misused.",
+				      function);
+		mem_err = 0;
+	    }
+	}
     }
     
     PUTBACK;
@@ -181,8 +220,16 @@ weechat_perl_exec (t_weechat_plugin *plugin,
 #ifndef MULTIPLICITY
     free (func);
 #endif
+
+    if (ret_value == NULL && mem_err == 1)
+    {
+	plugin->print_server (plugin,
+                              "Python error: unable to alloc memory in function \"%s\"",
+                              function);
+	return NULL;
+    }
     
-    return return_code;
+    return ret_value;
 }
 
 /*
@@ -194,9 +241,23 @@ weechat_perl_cmd_msg_handler (t_weechat_plugin *plugin,
                               int argc, char **argv,
                               char *handler_args, void *handler_pointer)
 {
+    int *r;
+    int ret;
+    
     if (argc >= 3)
-        return weechat_perl_exec (plugin, (t_plugin_script *)handler_pointer,
-                                  handler_args, argv[0], argv[2], NULL);
+    {
+        r = (int *) weechat_perl_exec (plugin, (t_plugin_script *)handler_pointer,
+				       SCRIPT_EXEC_INT,
+				       handler_args, argv[0], argv[2], NULL);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
@@ -210,12 +271,23 @@ weechat_perl_timer_handler (t_weechat_plugin *plugin,
                             int argc, char **argv,
                             char *handler_args, void *handler_pointer)
 {
+    int *r, ret;
+    
     /* make gcc happy */
     (void) argc;
     (void) argv;
     
-    return weechat_perl_exec (plugin, (t_plugin_script *)handler_pointer,
-                              handler_args, NULL, NULL, NULL);
+    r = (int *) weechat_perl_exec (plugin, (t_plugin_script *)handler_pointer,
+				   SCRIPT_EXEC_INT,
+				   handler_args, NULL, NULL, NULL);
+    if (r == NULL)
+	ret = PLUGIN_RC_KO;
+    else
+    {
+	ret = *r;
+	free (r);
+    }
+    return ret;
 }
 
 /*
@@ -227,20 +299,51 @@ weechat_perl_keyboard_handler (t_weechat_plugin *plugin,
                                int argc, char **argv,
                                char *handler_args, void *handler_pointer)
 {
+    int *r;
+    int ret;
+
     if (argc >= 3)
-        return weechat_perl_exec (plugin, (t_plugin_script *)handler_pointer,
-                                  handler_args, argv[0], argv[1], argv[2]);
+    {
+        r = (int *) weechat_perl_exec (plugin, (t_plugin_script *)handler_pointer,
+				       SCRIPT_EXEC_INT,
+				       handler_args, argv[0], argv[1], argv[2]);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
 
+/*
+ * weechat_perl_modifier: general modifier for Perl
+ */
+
+char *
+weechat_perl_modifier (t_weechat_plugin *plugin,
+                       int argc, char **argv,
+                       char *modifier_args, void *modifier_pointer)
+{
+    if (argc >= 2)
+        return (char *) weechat_perl_exec (plugin, (t_plugin_script *)modifier_pointer,
+					     SCRIPT_EXEC_STRING,
+					     modifier_args, argv[0], argv[1], NULL);
+    else
+        return NULL;
+}
+
 /*
  * weechat::register: startup function for all WeeChat Perl scripts
  */
 
 static XS (XS_weechat_register)
 {
-    char *name, *version, *shutdown_func, *description;
+    char *name, *version, *shutdown_func, *description, *charset;
     dXSARGS;
     
     /* make gcc happy */
@@ -249,7 +352,7 @@ static XS (XS_weechat_register)
 
     perl_current_script = NULL;
     
-    if (items != 4)
+    if ((items < 4) || (items > 5))
     {
         perl_plugin->print_server (perl_plugin,
                                    "Perl error: wrong parameters for "
@@ -261,6 +364,7 @@ static XS (XS_weechat_register)
     version = SvPV (ST (1), PL_na);
     shutdown_func = SvPV (ST (2), PL_na);
     description = SvPV (ST (3), PL_na);
+    charset = (items == 5) ? SvPV (ST (4), PL_na) : NULL;
     
     if (weechat_script_search (perl_plugin, &perl_scripts, name))
     {
@@ -279,7 +383,7 @@ static XS (XS_weechat_register)
                                               (perl_current_script_filename) ?
 					      perl_current_script_filename : "",
                                               name, version, shutdown_func,
-                                              description);
+                                              description, charset);
     if (perl_current_script)
     {
         perl_plugin->print_server (perl_plugin,
@@ -295,6 +399,40 @@ static XS (XS_weechat_register)
     XSRETURN_YES;
 }
 
+/*
+ * weechat::set_charset: set script charset
+ */
+
+static XS (XS_weechat_set_charset)
+{
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to set charset, "
+                                   "script not initialized");
+	XSRETURN_EMPTY;
+    }
+    
+    if (items < 1)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"set_charset\" function");
+        XSRETURN_EMPTY;
+    }
+    
+    weechat_script_set_charset (perl_plugin,
+                                perl_current_script,
+                                SvPV (ST (0), PL_na));
+    
+    XSRETURN_YES;
+}
+
 /*
  * weechat::print: print message into a buffer (current or specified one)
  */
@@ -335,9 +473,9 @@ static XS (XS_weechat_print)
             server_name = SvPV (ST (2), PL_na);
     }
     
-    perl_plugin->print (perl_plugin,
-                        server_name, channel_name,
-                        "%s", message);
+    weechat_script_print (perl_plugin, perl_current_script,
+                          server_name, channel_name,
+                          "%s", message);
     
     XSRETURN_YES;
 }
@@ -372,7 +510,8 @@ static XS (XS_weechat_print_server)
     
     message = SvPV (ST (0), PL_na);
     
-    perl_plugin->print_server (perl_plugin, "%s", message);
+    weechat_script_print_server (perl_plugin, perl_current_script,
+                                 "%s", message);
     
     XSRETURN_YES;
 }
@@ -404,10 +543,9 @@ static XS (XS_weechat_print_infobar)
         XSRETURN_NO;
     }
     
-    perl_plugin->print_infobar (perl_plugin,
-                                SvIV (ST (0)),
-                                "%s",
-                                SvPV (ST (1), PL_na));
+    weechat_script_print_infobar (perl_plugin, perl_current_script,
+                                  SvIV (ST (0)),
+                                  "%s", SvPV (ST (1), PL_na));
     
     XSRETURN_YES;
 }
@@ -477,9 +615,9 @@ static XS (XS_weechat_log)
             server_name = SvPV (ST (2), PL_na);
     }
     
-    perl_plugin->log (perl_plugin,
-		      server_name, channel_name,
-		      "%s", message);
+    weechat_script_log (perl_plugin, perl_current_script,
+                        server_name, channel_name,
+                        "%s", message);
     
     XSRETURN_YES;
 }
@@ -522,9 +660,9 @@ static XS (XS_weechat_command)
             server_name = SvPV (ST (2), PL_na);
     }
     
-    perl_plugin->exec_command (perl_plugin,
-                               server_name, channel_name,
-                               SvPV (ST (0), PL_na));
+    weechat_script_exec_command (perl_plugin, perl_current_script,
+                                 server_name, channel_name,
+                                 SvPV (ST (0), PL_na));
     
     XSRETURN_YES;
 }
@@ -806,6 +944,84 @@ static XS (XS_weechat_remove_keyboard_handler)
     XSRETURN_YES;
 }
 
+/*
+ * weechat::add_modifier: add a modifier
+ */
+
+static XS (XS_weechat_add_modifier)
+{
+    char *type, *command, *function;
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to add modifier, "
+                                   "script not initialized");
+	XSRETURN_NO;
+    }
+    
+    if (items < 3)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"add_modifier\" function");
+        XSRETURN_NO;
+    }
+    
+    type = SvPV (ST (0), PL_na);
+    command = SvPV (ST (1), PL_na);
+    function = SvPV (ST (2), PL_na);
+    
+    if (perl_plugin->modifier_add (perl_plugin, type, command,
+                                   weechat_perl_modifier, function,
+                                   (void *)perl_current_script))
+        XSRETURN_YES;
+    
+    XSRETURN_NO;
+}
+
+/*
+ * weechat::remove_modifier: remove a modifier
+ */
+
+static XS (XS_weechat_remove_modifier)
+{
+    char *type, *command, *function;
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to remove modifier, "
+                                   "script not initialized");
+	XSRETURN_NO;
+    }
+    
+    if (items < 2)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"remove_modifier\" function");
+        XSRETURN_NO;
+    }
+
+    type = SvPV (ST (0), PL_na);
+    command = SvPV (ST (1), PL_na);
+    function = SvPV (ST (2), PL_na);
+    
+    weechat_script_remove_modifier (perl_plugin, perl_current_script,
+                                    type, command, function);
+    
+    XSRETURN_YES;
+}
+
 /*
  * weechat::get_info: get various infos
  */
@@ -1157,9 +1373,6 @@ static XS (XS_weechat_get_server_info)
         hv_store (server_hash_member, "autojoin",              8, newSVpv (ptr_server->autojoin, 0), 0);
         hv_store (server_hash_member, "autorejoin",           10, newSViv (ptr_server->autorejoin), 0);
         hv_store (server_hash_member, "notify_levels",        13, newSVpv (ptr_server->notify_levels, 0), 0);
-        hv_store (server_hash_member, "charset_decode_iso",   18, newSVpv (ptr_server->charset_decode_iso, 0), 0);
-        hv_store (server_hash_member, "charset_decode_utf",   18, newSVpv (ptr_server->charset_decode_utf, 0), 0);
-        hv_store (server_hash_member, "charset_encode",       14, newSVpv (ptr_server->charset_encode, 0), 0);
         hv_store (server_hash_member, "is_connected",         12, newSViv (ptr_server->is_connected), 0);
         hv_store (server_hash_member, "ssl_connected",        13, newSViv (ptr_server->ssl_connected), 0);
         hv_store (server_hash_member, "nick",                  4, newSVpv (ptr_server->nick, 0), 0);
@@ -1500,6 +1713,7 @@ static XS (XS_weechat_get_buffer_data)
     t_plugin_buffer_line *buffer_data, *ptr_data;
     HV *data_list_member;
     char *server, *channel;
+    char timebuffer[64];
     int count;
     
     dXSARGS;
@@ -1542,7 +1756,11 @@ static XS (XS_weechat_get_buffer_data)
     for (ptr_data = buffer_data; ptr_data; ptr_data = ptr_data->next_line)
     {
 	data_list_member = (HV *) sv_2mortal((SV *) newHV());
+
+	strftime(timebuffer, sizeof(timebuffer), "%F %T",
+		 localtime(&ptr_data->date));
         
+	hv_store (data_list_member, "date", 4, newSVpv (timebuffer, 0), 0);
 	hv_store (data_list_member, "nick", 4, 
 		  newSVpv (ptr_data->nick == NULL ? "" : ptr_data->nick, 0), 0);
 	hv_store (data_list_member, "data", 4, 
@@ -1569,6 +1787,7 @@ weechat_perl_xs_init (pTHX)
     
     /* interface functions */
     newXS ("weechat::register", XS_weechat_register, "weechat");
+    newXS ("weechat::set_charset", XS_weechat_set_charset, "weechat");
     newXS ("weechat::print", XS_weechat_print, "weechat");
     newXS ("weechat::print_server", XS_weechat_print_server, "weechat");
     newXS ("weechat::print_infobar", XS_weechat_print_infobar, "weechat");
@@ -1582,6 +1801,8 @@ weechat_perl_xs_init (pTHX)
     newXS ("weechat::remove_handler", XS_weechat_remove_handler, "weechat");
     newXS ("weechat::remove_timer_handler", XS_weechat_remove_timer_handler, "weechat");
     newXS ("weechat::remove_keyboard_handler", XS_weechat_remove_keyboard_handler, "weechat");
+    newXS ("weechat::add_modifier", XS_weechat_add_modifier, "weechat");
+    newXS ("weechat::remove_modifier", XS_weechat_remove_modifier, "weechat");
     newXS ("weechat::get_info", XS_weechat_get_info, "weechat");
     newXS ("weechat::get_dcc_info", XS_weechat_get_dcc_info, "weechat");
     newXS ("weechat::get_config", XS_weechat_get_config, "weechat");
@@ -1615,8 +1836,9 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
 {
     STRLEN len;
     t_plugin_script tempscript;
-    int eval;
-
+    int *eval;
+    struct stat buf;
+    
 #ifndef MULTIPLICITY
     char pkgname[64];
 #else
@@ -1625,13 +1847,23 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
 #endif
     
     plugin->print_server (plugin, "Loading Perl script \"%s\"", filename);
-    perl_current_script = NULL;
+    
+    if (stat (filename, &buf) != 0)
+    {
+        plugin->print_server (plugin, "Perl error: script \"%s\" not found",
+                              filename);
+        return 0;
+    }
 
+    perl_current_script = NULL;
+    
 #ifndef MULTIPLICITY
     snprintf(pkgname, sizeof(pkgname), "%s%d", PKG_NAME_PREFIX, perl_num);
     perl_num++;
     tempscript.interpreter = "WeechatPerlScriptLoader";
-    eval = weechat_perl_exec (plugin, &tempscript, "weechat_perl_load_eval_file", filename, pkgname, "");
+    eval = weechat_perl_exec (plugin, &tempscript, 
+			      SCRIPT_EXEC_INT,
+			      "weechat_perl_load_eval_file", filename, pkgname, "");
 #else
     perl_current_interpreter = perl_alloc();
 
@@ -1650,15 +1882,24 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
     perl_parse (perl_current_interpreter, weechat_perl_xs_init, 3, perl_args, NULL);
     
     eval_pv (perl_weechat_code, TRUE);
-    eval = weechat_perl_exec (plugin, &tempscript, "weechat_perl_load_eval_file", filename, "", "");
+    eval = weechat_perl_exec (plugin, &tempscript,
+			      SCRIPT_EXEC_INT,
+			      "weechat_perl_load_eval_file", filename, "", "");
 
     free (perl_current_script_filename);
 
 #endif
-    
-    if ( eval != 0) 
+    if (eval == NULL)
     {
-	if (eval == 2) 
+	plugin->print_server (plugin,
+			      "Perl error: memory error while parsing file \"%s\"",
+			      filename);
+	return 0;
+    }
+    
+    if ( *eval != 0) 
+    {
+	if (*eval == 2) 
 	{
 	    plugin->print_server (plugin,
                                   "Perl error: unable to parse file \"%s\"",
@@ -1666,12 +1907,13 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
 	    plugin->print_server (plugin,
                                   "Perl error: %s",
 #ifndef MULTIPLICITY
-                                  SvPV(perl_get_sv("WeechatPerlScriptLoader::weechat_perl_load_eval_file_error", FALSE), len));
+                                  SvPV(perl_get_sv("WeechatPerlScriptLoader::weechat_perl_load_eval_file_error", FALSE), len)
 #else
-                                  SvPV(perl_get_sv("weechat_perl_load_eval_file_error", FALSE), len));
+                                  SvPV(perl_get_sv("weechat_perl_load_eval_file_error", FALSE), len)
 #endif
+		);
 	}
-	else if ( eval == 1)
+	else if (*eval == 1)
 	{
 	    plugin->print_server (plugin,
                                   "Perl error: unable to run file \"%s\"",
@@ -1688,10 +1930,13 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
 #endif
 	if ((perl_current_script != NULL) && (perl_current_script != &tempscript))
             weechat_script_remove (plugin, &perl_scripts, perl_current_script);
-
+	
+	free (eval);
 	return 0;
     }
     
+    free (eval);
+
     if (perl_current_script == NULL)
     {
 	plugin->print_server (plugin,
@@ -1721,6 +1966,8 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
 void
 weechat_perl_unload (t_weechat_plugin *plugin, t_plugin_script *script)
 {
+    int *r;
+
     plugin->print_server (plugin,
                           "Unloading Perl script \"%s\"",
                           script->name);
@@ -1732,8 +1979,13 @@ weechat_perl_unload (t_weechat_plugin *plugin, t_plugin_script *script)
 #endif        
 
     if (script->shutdown_func[0])
-        weechat_perl_exec (plugin, script, script->shutdown_func, NULL, NULL, NULL);
-
+    {
+        r = (int *) weechat_perl_exec (plugin, script, SCRIPT_EXEC_INT,
+				       script->shutdown_func, NULL, NULL, NULL);
+	if (r)
+	    free (r);
+    }
+    
 #ifndef MULTIPLICITY
     if (script->interpreter)
 	free (script->interpreter);
@@ -1795,10 +2047,11 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
                   int cmd_argc, char **cmd_argv,
                   char *handler_args, void *handler_pointer)
 {
-    int argc, handler_found;
+    int argc, handler_found, modifier_found;
     char **argv, *path_script;
     t_plugin_script *ptr_script;
     t_plugin_handler *ptr_handler;
+    t_plugin_modifier *ptr_modifier;
     
     if (cmd_argc < 3)
         return PLUGIN_RC_KO;
@@ -1843,7 +2096,7 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_MESSAGE)
+                if ((ptr_handler->type == PLUGIN_HANDLER_MESSAGE)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1862,7 +2115,7 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_COMMAND)
+                if ((ptr_handler->type == PLUGIN_HANDLER_COMMAND)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1881,7 +2134,7 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_TIMER)
+                if ((ptr_handler->type == PLUGIN_HANDLER_TIMER)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1900,7 +2153,7 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_KEYBOARD)
+                if ((ptr_handler->type == PLUGIN_HANDLER_KEYBOARD)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1910,7 +2163,35 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
             }
             if (!handler_found)
                 plugin->print_server (plugin, "  (none)");
-            break;
+            
+	    /* List Perl modifiers */
+	    plugin->print_server (plugin, "");
+            plugin->print_server (plugin, "Python modifiers:");
+            modifier_found = 0;
+            for (ptr_modifier = plugin->modifiers;
+                 ptr_modifier; ptr_modifier = ptr_modifier->next_modifier)
+            {
+		modifier_found = 1;
+		if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_IN)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Perl(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_IN_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_USER)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Perl(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_USER_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_OUT)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Perl(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_OUT_STR,
+					  ptr_modifier->modifier_args);
+            }
+            if (!modifier_found)
+                plugin->print_server (plugin, "  (none)");
+	    break;
+	    
         case 1:
             if (plugin->ascii_strcasecmp (plugin, argv[0], "autoload") == 0)
                 weechat_script_auto_load (plugin, "perl", weechat_perl_load);
@@ -1990,7 +2271,7 @@ weechat_plugin_init (t_weechat_plugin *plugin)
                              "filename: Perl script (file) to load\n"
                              "script: script name to unload\n\n"
                              "Without argument, /perl command lists all loaded Perl scripts.",
-                             "load|autoload|reload|unload",
+                             "load|autoload|reload|unload %f",
                              weechat_perl_cmd, NULL, NULL);
     
     plugin->mkdir_home (plugin, "perl");
