@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,14 +19,20 @@
 
 /* weechat-python.c: Python plugin support for WeeChat */
 
+#undef _
 
 #include <Python.h>
+
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
+#include <time.h>
+
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#undef _
+
 #include "../../weechat-plugin.h"
 #include "../weechat-script.h"
 
@@ -42,21 +48,24 @@ t_plugin_script *python_current_script = NULL;
 char *python_current_script_filename = NULL;
 PyThreadState *python_mainThreadState = NULL;
 
+char python_buffer_output[128];
 
 /*
  * weechat_python_exec: execute a Python script
  */
 
-int
+void *
 weechat_python_exec (t_weechat_plugin *plugin,
                      t_plugin_script *script,
+		     int ret_type,
                      char *function, char *arg1, char *arg2, char *arg3)
 {
     PyObject *evMain;
     PyObject *evDict;
     PyObject *evFunc;
     PyObject *rc;
-    int ret;
+    void *ret_value;
+    int *ret_i;
     
     /* PyEval_AcquireLock (); */
     PyThreadState_Swap (script->interpreter);
@@ -71,11 +80,9 @@ weechat_python_exec (t_weechat_plugin *plugin,
                               "Python error: unable to run function \"%s\"",
                               function);
 	/* PyEval_ReleaseThread (python_current_script->interpreter); */
-	return PLUGIN_RC_KO;
+	return NULL;
     }
 
-    ret = -1;
-    
     python_current_script = script;
     
     if (arg1)
@@ -91,30 +98,60 @@ weechat_python_exec (t_weechat_plugin *plugin,
             rc = PyObject_CallFunction (evFunc, "s", arg1);
     }
     else
-        rc = PyObject_CallFunction (evFunc, "");
+        rc = PyObject_CallFunction (evFunc, NULL);
 
+
+    ret_value = NULL;
     
-    if (rc == Py_None)
+    /* 
+       ugly hack : rc = NULL while 'return weechat.PLUGIN_RC_OK .... 
+       because of '#define PLUGIN_RC_OK 0'
+    */
+    if (rc ==  NULL)
+	rc = PyInt_FromLong (0);
+    
+    if (PyString_Check (rc) && (ret_type == SCRIPT_EXEC_STRING))
     {
-	python_plugin->print_server (python_plugin, "Python error: function \"%s\" must return a valid value", function);
-	/* PyEval_ReleaseThread (python_current_script->interpreter); */
-	return PLUGIN_RC_OK;
+	if (PyString_AsString (rc))
+	    ret_value = strdup (PyString_AsString(rc));
+	else
+	    ret_value = NULL;
+	
+	Py_XDECREF(rc);
     }
-
-    if (rc)
+    else if (PyInt_Check (rc) && (ret_type == SCRIPT_EXEC_INT))
     {
-        ret = (int) PyInt_AsLong(rc);
-        Py_XDECREF(rc);
+	
+	ret_i = (int *) malloc (sizeof(int));
+	if (ret_i)
+	    *ret_i = (int) PyInt_AsLong(rc);
+	ret_value = ret_i;
+	
+	Py_XDECREF(rc);
+    }
+    else
+    {
+	python_plugin->print_server (python_plugin, 
+				     "Python error: function \"%s\" must return a valid value",
+				     function);
+	/* PyEval_ReleaseThread (python_current_script->interpreter); */
+	return NULL;
+    }
+    
+    if (ret_value == NULL)
+    {
+	plugin->print_server (plugin,
+                              "Python error: unable to alloc memory in function \"%s\"",
+                              function);
+	/* PyEval_ReleaseThread (python_current_script->interpreter); */
+	return NULL;
     }
     
     if (PyErr_Occurred ()) PyErr_Print ();
     
-    if (ret < 0)
-	ret = PLUGIN_RC_OK;
-    
     /* PyEval_ReleaseThread (python_current_script->interpreter); */
     
-    return ret;
+    return ret_value;
 }
 
 /*
@@ -126,9 +163,23 @@ weechat_python_cmd_msg_handler (t_weechat_plugin *plugin,
                                 int argc, char **argv,
                                 char *handler_args, void *handler_pointer)
 {
+    int *r;
+    int ret;
+    
     if (argc >= 3)
-        return weechat_python_exec (plugin, (t_plugin_script *)handler_pointer,
-                                    handler_args, argv[0], argv[2], NULL);
+    {
+        r = (int *) weechat_python_exec (plugin, (t_plugin_script *)handler_pointer,
+					 SCRIPT_EXEC_INT,
+					 handler_args, argv[0], argv[2], NULL);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
@@ -145,9 +196,20 @@ weechat_python_timer_handler (t_weechat_plugin *plugin,
     /* make gcc happy */
     (void) argc;
     (void) argv;
+    int *r;
+    int ret;
     
-    return weechat_python_exec (plugin, (t_plugin_script *)handler_pointer,
-                                handler_args, NULL, NULL, NULL);
+    r = (int *) weechat_python_exec (plugin, (t_plugin_script *)handler_pointer,
+				     SCRIPT_EXEC_INT,
+				     handler_args, NULL, NULL, NULL);
+    if (r == NULL)
+	ret = PLUGIN_RC_KO;
+    else
+    {
+	ret = *r;
+	free (r);
+    }
+    return ret;
 }
 
 /*
@@ -159,13 +221,44 @@ weechat_python_keyboard_handler (t_weechat_plugin *plugin,
                                  int argc, char **argv,
                                  char *handler_args, void *handler_pointer)
 {
+    int *r;
+    int ret;
+    
     if (argc >= 3)
-        return weechat_python_exec (plugin, (t_plugin_script *)handler_pointer,
-                                    handler_args, argv[0], argv[1], argv[2]);
+    {
+        r = (int *) weechat_python_exec (plugin, (t_plugin_script *)handler_pointer,
+					 SCRIPT_EXEC_INT,
+					 handler_args, argv[0], argv[1], argv[2]);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
 
+/*
+ * weechat_python_modifier: general modifier for Python
+ */
+
+char *
+weechat_python_modifier (t_weechat_plugin *plugin,
+                         int argc, char **argv,
+                         char *modifier_args, void *modifier_pointer)
+{    
+    if (argc >= 2)
+        return (char *) weechat_python_exec (plugin, (t_plugin_script *)modifier_pointer,
+					     SCRIPT_EXEC_STRING,
+					     modifier_args, argv[0], argv[1], NULL);
+    else
+        return NULL;
+}
+
 /*
  * weechat_python_register: startup function for all WeeChat Python scripts
  */
@@ -173,7 +266,7 @@ weechat_python_keyboard_handler (t_weechat_plugin *plugin,
 static PyObject *
 weechat_python_register (PyObject *self, PyObject *args)
 {
-    char *name, *version, *shutdown_func, *description;
+    char *name, *version, *shutdown_func, *description, *charset;
     
     /* make gcc happy */
     (void) self;
@@ -184,8 +277,10 @@ weechat_python_register (PyObject *self, PyObject *args)
     version = NULL;
     shutdown_func = NULL;
     description = NULL;
+    charset = NULL;
     
-    if (!PyArg_ParseTuple (args, "ssss", &name, &version, &shutdown_func, &description))
+    if (!PyArg_ParseTuple (args, "ssss|s", &name, &version, &shutdown_func,
+                           &description, &charset))
     {
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
@@ -210,7 +305,7 @@ weechat_python_register (PyObject *self, PyObject *args)
                                                 (python_current_script_filename) ?
                                                 python_current_script_filename : "",
                                                 name, version, shutdown_func,
-                                                description);
+                                                description, charset);
     if (python_current_script)
     {
         python_plugin->print_server (python_plugin,
@@ -224,6 +319,46 @@ weechat_python_register (PyObject *self, PyObject *args)
     return Py_BuildValue ("i", 1);
 }
 
+/*
+ * weechat_python_set_charset: set script charset
+ */
+
+static PyObject *
+weechat_python_set_charset (PyObject *self, PyObject *args)
+{
+    char *charset;
+    
+    /* make gcc happy */
+    (void) self;
+    
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to set charset, "
+                                     "script not initialized");
+	Py_INCREF(Py_None);
+        return Py_None;
+    }
+    
+    charset = NULL;
+    
+    if (!PyArg_ParseTuple (args, "s", &charset))
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: wrong parameters for "
+                                     "\"set_charset\" function");
+	Py_INCREF(Py_None);
+        return Py_None;
+    }
+    
+    if (charset)
+        weechat_script_set_charset (python_plugin,
+                                    python_current_script,
+                                    charset);
+    
+    return Py_BuildValue ("i", 1);
+}
+
 /*
  * weechat_python_print: print message into a buffer (current or specified one)
  */
@@ -729,6 +864,85 @@ weechat_python_remove_keyboard_handler (PyObject *self, PyObject *args)
     return Py_BuildValue ("i", 1);
 }
 
+/*
+ * weechat_python_add_modifier: add a modifier
+ */
+
+static PyObject *
+weechat_python_add_modifier (PyObject *self, PyObject *args)
+{
+    char *type, *command, *function;
+    
+    /* make gcc happy */
+    (void) self;
+    
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to add modifier, "
+                                     "script not initialized");
+        return Py_BuildValue ("i", 0);
+    }
+    
+    type = NULL;
+    command = NULL;
+    function = NULL;
+    
+    if (!PyArg_ParseTuple (args, "sss", &type, &command, &function))
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: wrong parameters for "
+                                     "\"add_modifier\" function");
+        return Py_BuildValue ("i", 0);
+    }
+    
+    if (python_plugin->modifier_add (python_plugin, type, command,
+                                     weechat_python_modifier,
+                                     function,
+                                     (void *)python_current_script))	    
+        return Py_BuildValue ("i", 1);
+    
+    return Py_BuildValue ("i", 0);
+}
+
+/*
+ * weechat_python_remove_modifier: remove a modifier
+ */
+
+static PyObject *
+weechat_python_remove_modifier (PyObject *self, PyObject *args)
+{
+    char *type, *command, *function;
+    
+    /* make gcc happy */
+    (void) self;
+    
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to remove modifier, "
+                                     "script not initialized");
+        return Py_BuildValue ("i", 0);
+    }
+    
+    type = NULL;
+    command = NULL;
+    function = NULL;
+    
+    if (!PyArg_ParseTuple (args, "sss", &type, &command, &function))
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: wrong parameters for "
+                                     "\"remove_modifier\" function");
+        return Py_BuildValue ("i", 0);
+    }
+    
+    weechat_script_remove_modifier (python_plugin, python_current_script,
+                                    type, command, function);
+    
+    return Py_BuildValue ("i", 1);
+}
+
 /*
  * weechat_python_get_info: get various infos
  */
@@ -798,12 +1012,16 @@ weechat_python_get_dcc_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get DCC info, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
 
     dcc_list = PyList_New (0);    
     if (!dcc_list)
+    {
+	Py_INCREF(Py_None);
 	return Py_None;
+    }
     
     dcc_info = python_plugin->get_dcc_info (python_plugin);
     if (!dcc_info)
@@ -856,6 +1074,7 @@ weechat_python_get_dcc_info (PyObject *self, PyObject *args)
 			   Py_BuildValue("k", ptr_dcc->bytes_per_sec));	    
 	    
             PyList_Append(dcc_list, dcc_list_member);
+	    Py_DECREF (dcc_list_member);
         }
     }
     
@@ -882,6 +1101,7 @@ weechat_python_get_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get config option, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
@@ -892,6 +1112,7 @@ weechat_python_get_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_config\" function");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
@@ -968,6 +1189,7 @@ weechat_python_get_plugin_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get plugin config option, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
@@ -978,6 +1200,7 @@ weechat_python_get_plugin_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_plugin_config\" function");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
@@ -1060,12 +1283,16 @@ weechat_python_get_server_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get server infos, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
     server_hash = PyDict_New ();
     if (!server_hash)
+    {
+	Py_INCREF(Py_None);
 	return Py_None;
+    }
 
     server_info = python_plugin->get_server_info (python_plugin);
     if  (!server_info)
@@ -1118,12 +1345,6 @@ weechat_python_get_server_info (PyObject *self, PyObject *args)
 			   Py_BuildValue("i", ptr_server->autorejoin));
 	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "notify_levels"),
 			   Py_BuildValue("s", ptr_server->notify_levels));
-	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "charset_decode_iso"),
-			   Py_BuildValue("s", ptr_server->charset_decode_iso));
-	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "charset_decode_utf"),
-			   Py_BuildValue("s", ptr_server->charset_decode_utf));
-	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "charset_encode"),
-			   Py_BuildValue("s", ptr_server->charset_encode));
 	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "is_connected"),
 			   Py_BuildValue("i", ptr_server->is_connected));
 	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "ssl_connected"),
@@ -1138,6 +1359,7 @@ weechat_python_get_server_info (PyObject *self, PyObject *args)
 			   Py_BuildValue("i", ptr_server->lag));
 	    
 	    PyDict_SetItem(server_hash, Py_BuildValue("s", ptr_server->name), server_hash_member);
+	    Py_DECREF (server_hash_member);
 	}
     }    
 
@@ -1165,6 +1387,7 @@ weechat_python_get_channel_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get channel infos, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
@@ -1174,12 +1397,16 @@ weechat_python_get_channel_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_channel_info\" function");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
     channel_hash = PyDict_New ();
     if (!channel_hash)
+    {
+	Py_INCREF(Py_None);
 	return Py_None;
+    }
 
     channel_info = python_plugin->get_channel_info (python_plugin, server);
     if  (!channel_info)
@@ -1205,6 +1432,7 @@ weechat_python_get_channel_info (PyObject *self, PyObject *args)
 			   Py_BuildValue("i", ptr_channel->nicks_count));
 	    
 	    PyDict_SetItem(channel_hash, Py_BuildValue("s", ptr_channel->name), channel_hash_member);
+	    Py_DECREF (channel_hash_member);
 	}
     }    
 
@@ -1232,6 +1460,7 @@ weechat_python_get_nick_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get nick infos, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
@@ -1242,12 +1471,16 @@ weechat_python_get_nick_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_nick_info\" function");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
     nick_hash = PyDict_New ();
     if (!nick_hash)
+    {
+	Py_INCREF(Py_None);
 	return Py_None;
+    }
 
     nick_info = python_plugin->get_nick_info (python_plugin, server, channel);
     if  (!nick_info)
@@ -1265,6 +1498,7 @@ weechat_python_get_nick_info (PyObject *self, PyObject *args)
 			   Py_BuildValue("s", ptr_nick->host ? ptr_nick->host : ""));
 	    
 	    PyDict_SetItem(nick_hash, Py_BuildValue("s", ptr_nick->nick), nick_hash_member);
+	    Py_DECREF (nick_hash_member);
 	}
     }    
 
@@ -1329,12 +1563,15 @@ weechat_python_get_window_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get window info, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
         
     window_list = PyList_New (0);
-    if (!window_list)
+    if (!window_list) {
+	Py_INCREF(Py_None);
 	return Py_None;
+    }
 
     window_info = python_plugin->get_window_info (python_plugin);
     if  (!window_info)
@@ -1362,6 +1599,7 @@ weechat_python_get_window_info (PyObject *self, PyObject *args)
 			   Py_BuildValue("i", ptr_window->win_height_pct));
 	    
 	    PyList_Append(window_list, window_list_member);
+	    Py_DECREF (window_list_member);
 	}
     }
     
@@ -1389,12 +1627,16 @@ weechat_python_get_buffer_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get buffer info, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
     
     buffer_hash = PyDict_New ();
     if (!buffer_hash)
+    {
+	Py_INCREF(Py_None);
 	return Py_None;
+    }
 
     buffer_info = python_plugin->get_buffer_info (python_plugin);
     if  (!buffer_info)
@@ -1421,6 +1663,7 @@ weechat_python_get_buffer_info (PyObject *self, PyObject *args)
 	    
 	    PyDict_SetItem(buffer_hash, 
 			   Py_BuildValue("i", ptr_buffer->number), buffer_hash_member);
+	    Py_DECREF (buffer_hash_member);
 	}
     }    
     
@@ -1439,6 +1682,7 @@ weechat_python_get_buffer_data (PyObject *self, PyObject *args)
     t_plugin_buffer_line *buffer_data, *ptr_data;
     PyObject *data_list, *data_list_member;
     char *server, *channel;
+    char timebuffer[64];
     
     /* make gcc happy */
     (void) self;
@@ -1449,6 +1693,7 @@ weechat_python_get_buffer_data (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get buffer data, "
                                      "script not initialized");
+	Py_INCREF(Py_None);
         return Py_None;
     }
 
@@ -1457,12 +1702,16 @@ weechat_python_get_buffer_data (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_buffer_data\" function");
+	Py_INCREF(Py_None);
         return Py_None;
     }
 
     data_list = PyList_New (0);    
     if (!data_list)
+    {
+	Py_INCREF(Py_None);
 	return Py_None;
+    }
     
     buffer_data = python_plugin->get_buffer_data (python_plugin, server, channel);
     if (!buffer_data)
@@ -1474,12 +1723,18 @@ weechat_python_get_buffer_data (PyObject *self, PyObject *args)
 	
         if (data_list_member) 
         {
+	    strftime(timebuffer, sizeof(timebuffer), "%F %T",
+		     localtime(&ptr_data->date));
+	    
+	    PyDict_SetItem(data_list_member, Py_BuildValue("s", "date"),
+			   Py_BuildValue("s", timebuffer));
 	    PyDict_SetItem(data_list_member, Py_BuildValue("s", "nick"),
 			   Py_BuildValue("s", ptr_data->nick == NULL ? "" : ptr_data->nick));
 	    PyDict_SetItem(data_list_member, Py_BuildValue("s", "data"),
 			   Py_BuildValue("s", ptr_data->data == NULL ? "" : ptr_data->data));
 	    	    
             PyList_Append(data_list, data_list_member);
+	    Py_DECREF (data_list_member);
         }
     }
     
@@ -1495,6 +1750,7 @@ weechat_python_get_buffer_data (PyObject *self, PyObject *args)
 static
 PyMethodDef weechat_python_funcs[] = {
     { "register", weechat_python_register, METH_VARARGS, "" },
+    { "set_charset", weechat_python_set_charset, METH_VARARGS, "" },
     { "prnt", weechat_python_print, METH_VARARGS, "" },
     { "print_server", weechat_python_print_server, METH_VARARGS, "" },
     { "print_infobar", weechat_python_print_infobar, METH_VARARGS, "" },
@@ -1508,6 +1764,8 @@ PyMethodDef weechat_python_funcs[] = {
     { "remove_handler", weechat_python_remove_handler, METH_VARARGS, "" },
     { "remove_timer_handler", weechat_python_remove_timer_handler, METH_VARARGS, "" },
     { "remove_keyboard_handler", weechat_python_remove_keyboard_handler, METH_VARARGS, "" },
+    { "add_modifier", weechat_python_add_modifier, METH_VARARGS, "" },
+    { "remove_modifier", weechat_python_remove_modifier, METH_VARARGS, "" },
     { "get_info", weechat_python_get_info, METH_VARARGS, "" },
     { "get_dcc_info", weechat_python_get_dcc_info, METH_VARARGS, "" },
     { "get_config", weechat_python_get_config, METH_VARARGS, "" },
@@ -1531,7 +1789,7 @@ PyMethodDef weechat_python_funcs[] = {
 static PyObject *
 weechat_python_output (PyObject *self, PyObject *args)
 {
-    char *msg, *p;
+    char *msg, *m, *p;
     /* make gcc happy */
     (void) self;
     
@@ -1539,19 +1797,42 @@ weechat_python_output (PyObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple (args, "s", &msg))
     {
-        python_plugin->print_server (python_plugin,
-                                     "Python error: unable to get "
-                                     "stdout/stderr message(s)");
-        return NULL; 
+        if (strlen(python_buffer_output) > 0)
+	{
+	    python_plugin->print_server (python_plugin,
+					 "Python stdout/stderr : %s",
+					 python_buffer_output);
+	    python_buffer_output[0] = '\0';
+	}
+    }
+    else 
+    {
+	m = msg;
+	while ((p = strchr (m, '\n')) != NULL)
+	{
+	    *p = '\0';
+	    if (strlen (m) + strlen (python_buffer_output) > 0)
+		python_plugin->print_server (python_plugin,
+					     "Python stdout/stderr : %s%s", 
+					     python_buffer_output, m);
+	    *p = '\n';
+	    python_buffer_output[0] = '\0';
+	    m = ++p;
+	}
+	
+	if (strlen(m) + strlen(python_buffer_output) > sizeof(python_buffer_output))
+	{
+	    python_plugin->print_server (python_plugin,
+					 "Python stdout/stderr : %s%s",
+					 python_buffer_output, m);
+	    python_buffer_output[0] = '\0';
+	}
+	else
+	    strcat (python_buffer_output, m);
     }
     
-    while ((p = strrchr(msg, '\n')) != NULL)
-	*p = '\0';
-    
-    if (strlen(msg) > 0)
-	python_plugin->print_server (python_plugin,
-                                     "Python stdin/stdout: %s", msg);
-    return Py_BuildValue ("i", 1);
+    Py_INCREF(Py_None);
+    return Py_None;
 }
 
 /*
@@ -1575,19 +1856,21 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
     FILE *fp;
     PyThreadState *python_current_interpreter;
     PyObject *weechat_module, *weechat_outputs, *weechat_dict;
+    PyObject *python_path, *path;
+    char *w_home, *p_home;
+    int len;
     
     plugin->print_server (plugin, "Loading Python script \"%s\"", filename);
     
     if ((fp = fopen (filename, "r")) == NULL)
     {
-        plugin->print_server (plugin,
-                              "Python error: unable to open file \"%s\"",
+        plugin->print_server (plugin, "Python error: script \"%s\" not found",
                               filename);
         return 0;
     }
-
+    
     python_current_script = NULL;
-
+    
     /* PyEval_AcquireLock (); */
     python_current_interpreter = Py_NewInterpreter ();
     PySys_SetArgv(1, argv);
@@ -1617,6 +1900,27 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
 	return 0;
     }
 
+    /* adding $weechat_dir/python in $PYTHONPATH */    
+    python_path = PySys_GetObject ("path");
+    w_home = plugin->get_info (plugin, "weechat_dir", NULL);
+    if (w_home)
+    {
+	len = strlen (w_home) + 1 + strlen("python") + 1;
+	p_home = (char *) malloc (len * sizeof(char));
+	if (p_home)
+	{
+	    snprintf (p_home, len, "%s/python", w_home);
+	    path = PyString_FromString (p_home);
+	    if (path != NULL)
+	    {
+		PyList_Insert (python_path, 0, path);
+		Py_DECREF (path);
+	    }
+	    free (p_home);
+	}
+	free (w_home);
+    }
+    
     /* define some constants */
     weechat_dict = PyModule_GetDict(weechat_module);
     PyDict_SetItemString(weechat_dict, "PLUGIN_RC_OK", PyInt_FromLong((long) PLUGIN_RC_OK));
@@ -1693,12 +1997,19 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
 void
 weechat_python_unload (t_weechat_plugin *plugin, t_plugin_script *script)
 {
+    int *r;
+    
     plugin->print_server (plugin,
                           "Unloading Python script \"%s\"",
                           script->name);
     
     if (script->shutdown_func[0])
-        weechat_python_exec (plugin, script, script->shutdown_func, NULL, NULL, NULL);
+    {
+        r = (int *) weechat_python_exec (plugin, script, SCRIPT_EXEC_INT,
+					 script->shutdown_func, NULL, NULL, NULL);
+	if (r)
+	    free (r);
+    }
 
     PyThreadState_Swap (script->interpreter);
     Py_EndInterpreter (script->interpreter);
@@ -1756,10 +2067,11 @@ weechat_python_cmd (t_weechat_plugin *plugin,
                     int cmd_argc, char **cmd_argv,
                     char *handler_args, void *handler_pointer)
 {
-    int argc, handler_found;
+    int argc, handler_found, modifier_found;
     char **argv, *path_script;
     t_plugin_script *ptr_script;
     t_plugin_handler *ptr_handler;
+    t_plugin_modifier *ptr_modifier;
     
     /* make gcc happy */
     (void) handler_args;
@@ -1804,7 +2116,7 @@ weechat_python_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_MESSAGE)
+                if ((ptr_handler->type == PLUGIN_HANDLER_MESSAGE)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1823,7 +2135,7 @@ weechat_python_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_COMMAND)
+                if ((ptr_handler->type == PLUGIN_HANDLER_COMMAND)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1842,7 +2154,7 @@ weechat_python_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_TIMER)
+                if ((ptr_handler->type == PLUGIN_HANDLER_TIMER)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1861,7 +2173,7 @@ weechat_python_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_KEYBOARD)
+                if ((ptr_handler->type == PLUGIN_HANDLER_KEYBOARD)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -1871,7 +2183,35 @@ weechat_python_cmd (t_weechat_plugin *plugin,
             }
             if (!handler_found)
                 plugin->print_server (plugin, "  (none)");
+
+	    /* list Python modifiers */
+	    plugin->print_server (plugin, "");
+            plugin->print_server (plugin, "Python modifiers:");
+            modifier_found = 0;
+            for (ptr_modifier = plugin->modifiers;
+                 ptr_modifier; ptr_modifier = ptr_modifier->next_modifier)
+            {
+		modifier_found = 1;
+		if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_IN)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Python(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_IN_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_USER)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Python(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_USER_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_OUT)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Python(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_OUT_STR,
+					  ptr_modifier->modifier_args);
+            }
+            if (!modifier_found)
+                plugin->print_server (plugin, "  (none)");
             break;
+	    
         case 1:
             if (plugin->ascii_strcasecmp (plugin, argv[0], "autoload") == 0)
                 weechat_script_auto_load (plugin, "python", weechat_python_load);
@@ -1926,6 +2266,9 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     python_plugin = plugin;
     
     plugin->print_server (plugin, "Loading Python module \"weechat\"");
+
+    /* init stdout/stderr buffer */
+    python_buffer_output[0] = '\0';
     
     Py_Initialize ();
     if (Py_IsInitialized () == 0)
@@ -1953,7 +2296,7 @@ weechat_plugin_init (t_weechat_plugin *plugin)
                              "filename: Python script (file) to load\n"
                              "script: script name to unload\n\n"
                              "Without argument, /python command lists all loaded Python scripts.",
-                             "load|autoload|reload|unload",
+                             "load|autoload|reload|unload %f",
                              weechat_python_cmd, NULL, NULL);
 
     plugin->mkdir_home (plugin, "python");
