@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,18 +17,24 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/* wee-ruby.c: Ruby plugin support for WeeChat */
+/* weechat-ruby.c: Ruby plugin support for WeeChat */
 
+#undef _
 
 #include <ruby.h>
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <time.h>
+
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#undef _
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "../../weechat-plugin.h"
 #include "../weechat-script.h"
 
@@ -48,6 +54,8 @@ VALUE ruby_mWeechat, ruby_mWeechatOutputs;
 #define MOD_NAME_PREFIX "WeechatRubyModule"
 int ruby_num = 0;
 
+char ruby_buffer_output[128];
+
 typedef struct protect_call_arg {
     VALUE recv;
     ID mid;
@@ -73,7 +81,7 @@ protect_funcall0(VALUE arg)
  */
 
 VALUE
-rb_protect_funcall(VALUE recv, ID mid, int *state, int argc, ...)
+rb_protect_funcall (VALUE recv, ID mid, int *state, int argc, ...)
 {
     va_list ap;
     VALUE *argv;
@@ -101,13 +109,15 @@ rb_protect_funcall(VALUE recv, ID mid, int *state, int argc, ...)
  * weechat_ruby_exec: execute a Ruby script
  */
 
-int
+void *
 weechat_ruby_exec (t_weechat_plugin *plugin,
                    t_plugin_script *script,
+		   int ret_type,
                    char *function, char *arg1, char *arg2, char *arg3)
 {
-    VALUE ruby_retcode, err;
-    int ruby_error;
+    VALUE rc, err;
+    int ruby_error, *ret_i;
+    void *ret_value;
     /* make gcc happy */
     (void) plugin;
     
@@ -118,24 +128,24 @@ weechat_ruby_exec (t_weechat_plugin *plugin,
         if (arg2)
         {
             if (arg3)
-                ruby_retcode = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
+                rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
                                                    &ruby_error, 3,
                                                    rb_str_new2(arg1),
                                                    rb_str_new2(arg2),
                                                    rb_str_new2(arg3));
             else
-                ruby_retcode = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
+                rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
                                                    &ruby_error, 2,
                                                    rb_str_new2(arg1),
                                                    rb_str_new2(arg2));
         }
         else
-            ruby_retcode = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
+            rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
                                                &ruby_error, 1,
                                                rb_str_new2(arg1));
     }
     else
-        ruby_retcode = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
+        rc = rb_protect_funcall ((VALUE) script->interpreter, rb_intern(function),
                                            &ruby_error, 0);
     
     if (ruby_error)
@@ -146,21 +156,41 @@ weechat_ruby_exec (t_weechat_plugin *plugin,
 	
 	err = rb_inspect(rb_gv_get("$!"));
 	ruby_plugin->print_server (ruby_plugin, "Ruby error: \"%s\"", STR2CSTR(err));
-	/*
-	err = rb_inspect(rb_gv_get("$@"));
-	ruby_plugin->print_server (ruby_plugin, "Ruby error: \"%s\"", STR2CSTR(err));
-	*/
-	
-        return PLUGIN_RC_KO;
+        
+	return NULL;
     }
 
-    if (TYPE(ruby_retcode) != T_FIXNUM)
+    if ((TYPE(rc) == T_STRING) && ret_type == SCRIPT_EXEC_STRING)
     {
-	ruby_plugin->print_server (ruby_plugin, "Ruby error: function \"%s\" must return a valid value", function);
+	if (STR2CSTR (rc))
+	    ret_value = strdup (STR2CSTR (rc));
+	else
+	    ret_value = NULL;
+    }
+    else if ((TYPE(rc) == T_FIXNUM) && ret_type == SCRIPT_EXEC_INT)
+    {
+	ret_i = (int *) malloc (sizeof(int));
+	if (ret_i)
+	    *ret_i = NUM2INT(rc);
+	ret_value = ret_i;
+    }
+    else
+    {
+	ruby_plugin->print_server (ruby_plugin,
+				   "Ruby error: function \"%s\" must return a valid value",
+				   function);
 	return PLUGIN_RC_OK;
     }
     
-    return NUM2INT(ruby_retcode);
+    if (ret_value == NULL)
+    {
+	plugin->print_server (plugin,
+                              "Ruby error: unable to alloc memory in function \"%s\"",
+                              function);
+	return NULL;
+    }
+    
+    return ret_value;
 }
 
 /*
@@ -172,9 +202,23 @@ weechat_ruby_cmd_msg_handler (t_weechat_plugin *plugin,
                               int argc, char **argv,
                               char *handler_args, void *handler_pointer)
 {
+    int *r;
+    int ret;
+    
     if (argc >= 3)
-        return weechat_ruby_exec (plugin, (t_plugin_script *)handler_pointer,
-                                  handler_args, argv[0], argv[2], NULL);
+    {
+        r = (int *) weechat_ruby_exec (plugin, (t_plugin_script *)handler_pointer,
+				       SCRIPT_EXEC_INT,
+				       handler_args, argv[0], argv[2], NULL);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
@@ -191,9 +235,20 @@ weechat_ruby_timer_handler (t_weechat_plugin *plugin,
     /* make gcc happy */
     (void) argc;
     (void) argv;
+    int *r;
+    int ret;
     
-    return weechat_ruby_exec (plugin, (t_plugin_script *)handler_pointer,
-                              handler_args, NULL, NULL, NULL);
+    r = (int *) weechat_ruby_exec (plugin, (t_plugin_script *)handler_pointer,
+				   SCRIPT_EXEC_INT,
+				   handler_args, NULL, NULL, NULL);
+    if (r == NULL)
+	ret = PLUGIN_RC_KO;
+    else
+    {
+	ret = *r;
+	free (r);
+    }
+    return ret;    
 }
 
 /*
@@ -205,27 +260,71 @@ weechat_ruby_keyboard_handler (t_weechat_plugin *plugin,
                               int argc, char **argv,
                               char *handler_args, void *handler_pointer)
 {
-    if (argc >= 2)
-        return weechat_ruby_exec (plugin, (t_plugin_script *)handler_pointer,
-                                  handler_args, argv[0], argv[1], argv[2]);
+    int *r;
+    int ret;
+    
+    if (argc >= 3)
+    {
+        r = (int *) weechat_ruby_exec (plugin, (t_plugin_script *)handler_pointer,
+				       SCRIPT_EXEC_INT,
+				       handler_args, argv[0], argv[1], argv[2]);
+	if (r == NULL)
+	    ret = PLUGIN_RC_KO;
+	else
+	{
+	    ret = *r;
+	    free (r);
+	}
+	return ret;
+    }
     else
         return PLUGIN_RC_KO;
 }
 
+/*
+ * weechat_ruby_modifier: general modifier for Ruby
+ */
+
+char *
+weechat_ruby_modifier (t_weechat_plugin *plugin,
+                       int argc, char **argv,
+                       char *modifier_args, void *modifier_pointer)
+{
+    if (argc >= 2)
+        return (char *) weechat_ruby_exec (plugin, (t_plugin_script *)modifier_pointer,
+					   SCRIPT_EXEC_STRING,
+					   modifier_args, argv[0], argv[1], NULL);
+    else
+        return NULL;
+}
+
 /*
  * weechat_ruby_register: startup function for all WeeChat Ruby scripts
  */
 
 static VALUE
-weechat_ruby_register (VALUE class, VALUE name, VALUE version,
-                       VALUE shutdown_func, VALUE description)
+weechat_ruby_register (int argc, VALUE *argv, VALUE class)
 {
-    char *c_name, *c_version, *c_shutdown_func, *c_description;
+    VALUE name, version, shutdown_func, description, charset;
+    char *c_name, *c_version, *c_shutdown_func, *c_description, *c_charset;
     
     /* make gcc happy */
     (void) class;
 
     ruby_current_script = NULL;
+
+    name = Qnil;
+    version = Qnil;
+    shutdown_func = Qnil;
+    description = Qnil;
+    charset = Qnil;
+    c_name = NULL;
+    c_version = NULL;
+    c_shutdown_func = NULL;
+    c_description = NULL;
+    c_charset = NULL;
+
+    rb_scan_args (argc, argv, "41", &name, &version, &shutdown_func, &description, &charset);
     
     if (NIL_P (name) || NIL_P (version) || NIL_P (shutdown_func) || NIL_P (description))
     {
@@ -245,6 +344,12 @@ weechat_ruby_register (VALUE class, VALUE name, VALUE version,
     c_shutdown_func = STR2CSTR (shutdown_func);
     c_description = STR2CSTR (description);
     
+    if (!NIL_P (charset))
+    {
+        Check_Type (charset, T_STRING);
+        c_charset = STR2CSTR (charset);
+    }
+    
     if (weechat_script_search (ruby_plugin, &ruby_scripts, c_name))
     {
         /* error: another scripts already exists with this name! */
@@ -262,7 +367,7 @@ weechat_ruby_register (VALUE class, VALUE name, VALUE version,
                                               (ruby_current_script_filename) ?
                                               ruby_current_script_filename : "",
                                               c_name, c_version, c_shutdown_func,
-                                              c_description);
+                                              c_description, c_charset);
     if (ruby_current_script)
     {
         ruby_plugin->print_server (ruby_plugin,
@@ -276,6 +381,47 @@ weechat_ruby_register (VALUE class, VALUE name, VALUE version,
     return INT2FIX (1);
 }
 
+/*
+ * weechat_ruby_set_charset: set script charset
+ */
+
+static VALUE
+weechat_ruby_set_charset (VALUE class, VALUE charset)
+{
+    char *c_charset;
+    
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to set charset, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+    
+    c_charset = NULL;
+    
+    if (NIL_P (c_charset))
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"set_charset\" function");
+        return INT2FIX (0);
+    }
+    
+    Check_Type (charset, T_STRING);
+    c_charset = STR2CSTR (charset);
+    
+    if (c_charset)
+        weechat_script_set_charset (ruby_plugin,
+                                    ruby_current_script,
+                                    c_charset);
+    
+    return INT2FIX (1);    
+}
+
 /*
  * weechat_ruby_print: print message into a buffer (current or specified one)
  */
@@ -924,6 +1070,101 @@ weechat_ruby_remove_keyboard_handler (VALUE class, VALUE function)
     return INT2FIX (1);
 }
 
+/*
+ * weechat_ruby_add_modifier: add a modifier
+ */
+
+static VALUE
+weechat_ruby_add_modifier (VALUE class, VALUE type, VALUE message, VALUE function)
+{
+    char *c_type, *c_message, *c_function;
+    
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to add modifier, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+
+    c_type = NULL;
+    c_message = NULL;
+    c_function = NULL;
+    
+    if (NIL_P (type) || NIL_P (message) || NIL_P (function))
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"add_modifier\" function");
+        return INT2FIX (0);
+    }
+
+    Check_Type (type, T_STRING);
+    Check_Type (message, T_STRING);
+    Check_Type (function, T_STRING);
+
+    c_type = STR2CSTR (type);
+    c_message = STR2CSTR (message);
+    c_function = STR2CSTR (function);
+    
+    if (ruby_plugin->modifier_add (ruby_plugin, c_type, c_message,
+                                   weechat_ruby_modifier,
+                                   c_function,
+                                   (void *)ruby_current_script))
+        return INT2FIX (1);
+    
+    return INT2FIX (0);
+}
+
+/*
+ * weechat_ruby_remove_modifier: remove a modifier
+ */
+
+static VALUE
+weechat_ruby_remove_modifier (VALUE class, VALUE type, VALUE command, VALUE function)
+{
+    char *c_type, *c_command, *c_function;
+    
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to remove modifier, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+
+    c_type = NULL;
+    c_command = NULL;
+    c_function = NULL;
+    
+    if (NIL_P (type) || NIL_P (command) || NIL_P (function))
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"remove_modifier\" function");
+        return INT2FIX (0);
+    }
+
+    Check_Type (type, T_STRING);
+    Check_Type (command, T_STRING);
+    Check_Type (function, T_STRING);
+
+    c_type = STR2CSTR (type);
+    c_command = STR2CSTR (command);
+    c_function = STR2CSTR (function);
+    
+    weechat_script_remove_modifier (ruby_plugin, ruby_current_script,
+                                    c_type, c_command, c_function);
+    
+    return INT2FIX (1);
+}
+
 /*
  * weechat_ruby_get_info: get various infos
  */
@@ -1342,12 +1583,6 @@ weechat_ruby_get_server_info (VALUE class)
 			  INT2FIX(ptr_server->autorejoin));
 	    rb_hash_aset (server_hash_member, rb_str_new2("notify_levels"),
 			  rb_str_new2(ptr_server->notify_levels));
-	    rb_hash_aset (server_hash_member, rb_str_new2("charset_decode_iso"),
-			  rb_str_new2(ptr_server->charset_decode_iso));
-	    rb_hash_aset (server_hash_member, rb_str_new2("charset_decode_utf"),
-			  rb_str_new2(ptr_server->charset_decode_utf));
-	    rb_hash_aset (server_hash_member, rb_str_new2("charset_encode"),
-			  rb_str_new2(ptr_server->charset_encode));
 	    rb_hash_aset (server_hash_member, rb_str_new2("is_connected"),
 			  INT2FIX(ptr_server->is_connected));
 	    rb_hash_aset (server_hash_member, rb_str_new2("ssl_connected"),
@@ -1678,6 +1913,7 @@ weechat_ruby_get_buffer_data (int argc, VALUE *argv, VALUE class)
     VALUE data_list, data_list_member;
     VALUE server, channel;
     char *c_server, *c_channel;
+    char timebuffer[64];
 
     /* make gcc happy */
     (void) class;
@@ -1723,6 +1959,11 @@ weechat_ruby_get_buffer_data (int argc, VALUE *argv, VALUE class)
 
 	if (!NIL_P (data_list_member))
 	{
+	    strftime(timebuffer, sizeof(timebuffer), "%F %T",
+		     localtime(&ptr_data->date));
+	    
+	    rb_hash_aset (data_list_member, rb_str_new2("date"), 
+			  rb_str_new2(timebuffer));
 	    rb_hash_aset (data_list_member, rb_str_new2("nick"), 
 			  rb_str_new2(ptr_data->nick == NULL ? "" : ptr_data->nick));
 	    rb_hash_aset (data_list_member, rb_str_new2("data"), 
@@ -1744,18 +1985,35 @@ weechat_ruby_get_buffer_data (int argc, VALUE *argv, VALUE class)
 static VALUE 
 weechat_ruby_output(VALUE self, VALUE str)
 {
-    char *msg, *p;
+    char *msg, *p, *m;
     /* make gcc happy */
     (void) self;
     
     msg = strdup(STR2CSTR(str));
     
-    while ((p = strrchr(msg, '\n')) != NULL)
+    m = msg;
+    while ((p = strchr (m, '\n')) != NULL)
+    {
 	*p = '\0';
+	if (strlen (m) + strlen (ruby_buffer_output) > 0)
+	    ruby_plugin->print_server (ruby_plugin,
+				       "Ruby stdout/stderr : %s%s", 
+				       ruby_buffer_output, m);
+	*p = '\n';
+	ruby_buffer_output[0] = '\0';
+	m = ++p;
+    }
     
-    if (strlen(msg) > 0)
+    if (strlen(m) + strlen(ruby_buffer_output) > sizeof(ruby_buffer_output))
+    {
 	ruby_plugin->print_server (ruby_plugin,
-                                   "Ruby stdout/stderr: %s", msg);
+				   "Ruby stdout/stderr : %s%s",
+				   ruby_buffer_output, m);
+	ruby_buffer_output[0] = '\0';
+    }
+    else
+	strcat (ruby_buffer_output, m);
+
     if (msg)
 	free (msg);
     
@@ -1786,10 +2044,19 @@ weechat_ruby_load (t_weechat_plugin *plugin, char *filename)
     char modname[64];
     VALUE curModule, ruby_retcode, err;
     int ruby_error;
+    struct stat buf;
     
     plugin->print_server (plugin, "Loading Ruby script \"%s\"", filename);
+    
+    if (stat (filename, &buf) != 0)
+    {
+        plugin->print_server (plugin, "Ruby error: script \"%s\" not found",
+                              filename);
+        return 0;
+    }
+    
     ruby_current_script = NULL;
-
+    
     snprintf(modname, sizeof(modname), "%s%d", MOD_NAME_PREFIX, ruby_num);
     ruby_num++;
 
@@ -1805,10 +2072,6 @@ weechat_ruby_load (t_weechat_plugin *plugin, char *filename)
     if (ruby_retcode == Qnil) {
 	err = rb_inspect(rb_gv_get("$!"));
 	ruby_plugin->print_server (ruby_plugin, "Ruby error: \"%s\"", STR2CSTR(err));
-	/*
-	err = rb_inspect(rb_gv_get("$@"));
-	ruby_plugin->print_server (ruby_plugin, "Ruby error: \"%s\"", STR2CSTR(err));
-	*/
 	return 0;
     }
     
@@ -1858,11 +2121,7 @@ weechat_ruby_load (t_weechat_plugin *plugin, char *filename)
 	
 	err = rb_inspect(rb_gv_get("$!"));
 	ruby_plugin->print_server (ruby_plugin, "Ruby error: \"%s\"", STR2CSTR(err));
-	/*
-	err = rb_inspect(rb_gv_get("$!"));
-	ruby_plugin->print_server (ruby_plugin, "Ruby error: \"%s\"", STR2CSTR(err));
-	*/
-
+	
 	if (ruby_current_script != NULL)
 	    weechat_script_remove (plugin, &ruby_scripts, ruby_current_script);
 	return 0;
@@ -1889,12 +2148,19 @@ weechat_ruby_load (t_weechat_plugin *plugin, char *filename)
 void
 weechat_ruby_unload (t_weechat_plugin *plugin, t_plugin_script *script)
 {
+    int *r;
+
     plugin->print_server (plugin,
                           "Unloading Ruby script \"%s\"",
                           script->name);
     
     if (script->shutdown_func[0])
-        weechat_ruby_exec (plugin, script, script->shutdown_func, NULL, NULL, NULL);
+    {
+        r = (int *) weechat_ruby_exec (plugin, script, SCRIPT_EXEC_INT,
+				       script->shutdown_func, NULL, NULL, NULL);
+	if (r)
+	    free (r);
+    }
     
     if (script->interpreter)
 	rb_gc_unregister_address (script->interpreter);
@@ -1953,10 +2219,11 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
                   int cmd_argc, char **cmd_argv,
                   char *handler_args, void *handler_pointer)
 {
-    int argc, handler_found;
+    int argc, handler_found, modifier_found;
     char **argv, *path_script;
     t_plugin_script *ptr_script;
     t_plugin_handler *ptr_handler;
+    t_plugin_modifier *ptr_modifier;
     
     /* make gcc happy */
     (void) handler_args;
@@ -2001,7 +2268,7 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_MESSAGE)
+                if ((ptr_handler->type == PLUGIN_HANDLER_MESSAGE)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2020,7 +2287,7 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_COMMAND)
+                if ((ptr_handler->type == PLUGIN_HANDLER_COMMAND)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2039,7 +2306,7 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_TIMER)
+                if ((ptr_handler->type == PLUGIN_HANDLER_TIMER)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2058,7 +2325,7 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
             for (ptr_handler = plugin->handlers;
                  ptr_handler; ptr_handler = ptr_handler->next_handler)
             {
-                if ((ptr_handler->type == HANDLER_KEYBOARD)
+                if ((ptr_handler->type == PLUGIN_HANDLER_KEYBOARD)
                     && (ptr_handler->handler_args))
                 {
                     handler_found = 1;
@@ -2068,7 +2335,34 @@ weechat_ruby_cmd (t_weechat_plugin *plugin,
             }
             if (!handler_found)
                 plugin->print_server (plugin, "  (none)");
-            break;
+            
+	    /* list Ruby modifiers */
+	    plugin->print_server (plugin, "");
+            plugin->print_server (plugin, "Ruby modifiers:");
+            modifier_found = 0;
+            for (ptr_modifier = plugin->modifiers;
+                 ptr_modifier; ptr_modifier = ptr_modifier->next_modifier)
+            {
+		modifier_found = 1;
+		if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_IN)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Ruby(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_IN_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_USER)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Ruby(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_USER_STR,
+					  ptr_modifier->modifier_args);
+		else if (ptr_modifier->type == PLUGIN_MODIFIER_IRC_OUT)
+		    plugin->print_server (plugin, "  IRC(%s, %s) => Ruby(%s)",
+					  ptr_modifier->command,
+					  PLUGIN_MODIFIER_IRC_OUT_STR,
+					  ptr_modifier->modifier_args);
+            }
+            if (!modifier_found)
+                plugin->print_server (plugin, "  (none)");
+	    break;
         case 1:
             if (plugin->ascii_strcasecmp (plugin, argv[0], "autoload") == 0)
                 weechat_script_auto_load (plugin, "ruby", weechat_ruby_load);
@@ -2166,6 +2460,9 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     ruby_error = 0;
 
     plugin->print_server (plugin, "Loading Ruby module \"weechat\"");
+
+    /* init stdout/stderr buffer */
+    ruby_buffer_output[0] = '\0';
     
     ruby_init ();
     ruby_init_loadpath ();
@@ -2177,7 +2474,8 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_WEECHAT", INT2NUM(PLUGIN_RC_OK_IGNORE_WEECHAT));
     rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_PLUGINS", INT2NUM(PLUGIN_RC_OK_IGNORE_PLUGINS));
     rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_ALL", INT2NUM(PLUGIN_RC_OK_IGNORE_ALL));    
-    rb_define_module_function (ruby_mWeechat, "register", weechat_ruby_register, 4);
+    rb_define_module_function (ruby_mWeechat, "register", weechat_ruby_register, -1);
+    rb_define_module_function (ruby_mWeechat, "set_charset", weechat_ruby_set_charset, 1);
     rb_define_module_function (ruby_mWeechat, "print", weechat_ruby_print, -1);
     rb_define_module_function (ruby_mWeechat, "print_server", weechat_ruby_print_server, 1);
     rb_define_module_function (ruby_mWeechat, "print_infobar", weechat_ruby_print_infobar, 2);
@@ -2191,6 +2489,8 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     rb_define_module_function (ruby_mWeechat, "remove_handler", weechat_ruby_remove_handler, 2);
     rb_define_module_function (ruby_mWeechat, "remove_timer_handler", weechat_ruby_remove_timer_handler, 1);
     rb_define_module_function (ruby_mWeechat, "remove_keyboard_handler", weechat_ruby_remove_keyboard_handler, 1);
+    rb_define_module_function (ruby_mWeechat, "add_modifier", weechat_ruby_add_modifier, 3);
+    rb_define_module_function (ruby_mWeechat, "remove_modifier", weechat_ruby_remove_modifier, 3);
     rb_define_module_function (ruby_mWeechat, "get_info", weechat_ruby_get_info, -1);
     rb_define_module_function (ruby_mWeechat, "get_dcc_info", weechat_ruby_get_dcc_info, 0);
     rb_define_module_function (ruby_mWeechat, "get_config", weechat_ruby_get_config, 1);
@@ -2219,7 +2519,7 @@ weechat_plugin_init (t_weechat_plugin *plugin)
                              "filename: Ruby script (file) to load\n"
                              "script: script name to unload\n\n"
                              "Without argument, /ruby command lists all loaded Ruby scripts.",
-                             "load|autoload|reload|unload",
+                             "load|autoload|reload|unload %f",
                              weechat_ruby_cmd, NULL, NULL);
     
     plugin->mkdir_home (plugin, "ruby");
