@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 
 #include <stdlib.h>
 #include <unistd.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -110,14 +111,7 @@ weechat_script_search_full_name (t_weechat_plugin *plugin,
         }
         return NULL;
     }
-    
-#ifdef _WIN32
-    if (strstr(filename, "\\"))
-#else
-    if (strstr(filename, "/"))
-#endif
-        return strdup(filename);
-    
+
     /* try WeeChat user's autoload dir */
     dir_home = plugin->get_info (plugin, "weechat_dir", NULL);
     if (dir_home)
@@ -137,7 +131,7 @@ weechat_script_search_full_name (t_weechat_plugin *plugin,
         free (dir_home);
     }
     
-    /* try WeeChat user's dir */
+    /* try WeeChat language user's dir */
     dir_home = plugin->get_info (plugin, "weechat_dir", NULL);
     if (dir_home)
     {
@@ -156,6 +150,25 @@ weechat_script_search_full_name (t_weechat_plugin *plugin,
         free (dir_home);
     }
     
+    /* try WeeChat user's dir */
+    dir_home = plugin->get_info (plugin, "weechat_dir", NULL);
+    if (dir_home)
+    {
+        length = strlen (dir_home) + strlen (filename) + 16;
+        final_name = (char *) malloc (length);
+        if (final_name)
+        {
+            snprintf (final_name, length, "%s/%s", dir_home, filename);
+            if ((stat (final_name, &st) == 0) && (st.st_size > 0))
+            {
+                free (dir_home);
+                return final_name;
+            }
+            free (final_name);
+        }
+        free (dir_home);
+    }
+
     /* try WeeChat system dir */
     dir_system = plugin->get_info (plugin, "weechat_sharedir", NULL);
     if (dir_system)
@@ -174,8 +187,8 @@ weechat_script_search_full_name (t_weechat_plugin *plugin,
         }
         free (dir_system);
     }
-    
-    return NULL;
+
+    return strdup(filename);
 }
 
 /*
@@ -187,7 +200,8 @@ weechat_script_add (t_weechat_plugin *plugin,
                     t_plugin_script **script_list,
                     char *filename,
                     char *name, char *version,
-                    char *shutdown_func, char *description)
+                    char *shutdown_func, char *description,
+                    char *charset)
 {
     t_plugin_script *new_script;
     
@@ -209,6 +223,7 @@ weechat_script_add (t_weechat_plugin *plugin,
         new_script->version = strdup (version);
         new_script->shutdown_func = strdup (shutdown_func);
         new_script->description = strdup (description);
+        new_script->charset = (charset) ? strdup (charset) : NULL;
         
         /* add new script to list */
         if ((*script_list))
@@ -235,6 +250,7 @@ weechat_script_remove (t_weechat_plugin *plugin,
                        t_plugin_script **script_list, t_plugin_script *script)
 {
     t_plugin_handler *ptr_handler, *next_handler;
+    t_plugin_modifier *ptr_modifier, *next_modifier;
     
     /* remove all handlers pointing to script */
     ptr_handler = plugin->handlers;
@@ -250,6 +266,20 @@ weechat_script_remove (t_weechat_plugin *plugin,
             ptr_handler = ptr_handler->next_handler;
     }
     
+    /* remove all modifiers pointing to script */
+    ptr_modifier = plugin->modifiers;
+    while (ptr_modifier)
+    {
+        if ((t_plugin_script *)ptr_modifier->modifier_pointer == script)
+        {
+            next_modifier = ptr_modifier->next_modifier;
+            plugin->modifier_remove (plugin, ptr_modifier);
+            ptr_modifier = next_modifier;
+        }
+        else
+            ptr_modifier = ptr_modifier->next_modifier;
+    }
+    
     /* free data */
     if (script->filename)
         free (script->filename);
@@ -261,6 +291,8 @@ weechat_script_remove (t_weechat_plugin *plugin,
         free (script->version);
     if (script->shutdown_func)
         free (script->shutdown_func);
+    if (script->charset)
+        free (script->charset);
     
     /* remove script from list */
     if (script->prev_script)
@@ -274,6 +306,122 @@ weechat_script_remove (t_weechat_plugin *plugin,
     free (script);
 }
 
+/*
+ * weechat_script_print: print a message on a server or channel buffer
+ */
+
+void
+weechat_script_print (t_weechat_plugin *plugin,
+                      t_plugin_script *script,
+                      char *server, char *channel,
+                      char *message, ...)
+{
+    va_list argptr;
+    static char buf[8192];
+    char *buf2;
+    
+    va_start (argptr, message);
+    vsnprintf (buf, sizeof (buf) - 1, message, argptr);
+    va_end (argptr);
+    
+    buf2 = (script->charset && script->charset[0]) ?
+        plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
+    plugin->print (plugin, server, channel, (buf2) ? buf2 : buf);
+    if (buf2)
+        free (buf2);
+}
+
+/*
+ * weechat_script_print_server: print a message on server buffer
+ */
+
+void
+weechat_script_print_server (t_weechat_plugin *plugin,
+                             t_plugin_script *script,
+                             char *message, ...)
+{
+    va_list argptr;
+    static char buf[8192];
+    char *buf2;
+    
+    va_start (argptr, message);
+    vsnprintf (buf, sizeof (buf) - 1, message, argptr);
+    va_end (argptr);
+    
+    buf2 = (script->charset && script->charset[0]) ?
+        plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
+    plugin->print_server (plugin, (buf2) ? buf2 : buf);
+    if (buf2)
+        free (buf2);
+}
+
+/*
+ * weechat_script_print_infobar: print a message in infobar
+ */
+
+void
+weechat_script_print_infobar (t_weechat_plugin *plugin,
+                              t_plugin_script *script,
+                              int time_displayed, char *message, ...)
+{
+    va_list argptr;
+    static char buf[1024];
+    char *buf2;
+    
+    va_start (argptr, message);
+    vsnprintf (buf, sizeof (buf) - 1, message, argptr);
+    va_end (argptr);
+    
+    buf2 = (script->charset && script->charset[0]) ?
+        plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
+    plugin->print_infobar (plugin, time_displayed, (buf2) ? buf2 : buf);
+    if (buf2)
+        free (buf2);
+}
+
+/*
+ * weechat_script_log: add a message in buffer log file
+ */
+
+void
+weechat_script_log (t_weechat_plugin *plugin,
+                    t_plugin_script *script,
+                    char *server, char *channel, char *message, ...)
+{
+    va_list argptr;
+    static char buf[1024];
+    char *buf2;
+    
+    va_start (argptr, message);
+    vsnprintf (buf, sizeof (buf) - 1, message, argptr);
+    va_end (argptr);
+    
+    buf2 = (script->charset && script->charset[0]) ?
+        plugin->iconv_to_internal (plugin, script->charset, buf) : NULL;
+    plugin->log (plugin, server, channel, (buf2) ? buf2 : buf);
+    if (buf2)
+        free (buf2);
+}
+
+/*
+ * weechat_script_exec_command: execute a command (simulate user entry)
+ */
+
+void
+weechat_script_exec_command (t_weechat_plugin *plugin,
+                             t_plugin_script *script,
+                             char *server, char *channel, char *command)
+{
+    char *command2;
+    
+    command2 = (script->charset && script->charset[0]) ?
+        plugin->iconv_to_internal (plugin, script->charset, command) : NULL;
+    plugin->exec_command (plugin, server, channel,
+                          (command2) ? command2 : command);
+    if (command2)
+        free (command2);
+}
+
 /*
  * weechat_script_remove_handler: remove a handler for a script
  *                                for a msg handler, arg1=irc command, arg2=function
@@ -293,9 +441,9 @@ weechat_script_remove_handler (t_weechat_plugin *plugin,
     while (ptr_handler)
     {
         ptr_arg1 = NULL;
-        if (ptr_handler->type == HANDLER_MESSAGE)
+        if (ptr_handler->type == PLUGIN_HANDLER_MESSAGE)
             ptr_arg1 = ptr_handler->irc_command;
-        else if (ptr_handler->type == HANDLER_COMMAND)
+        else if (ptr_handler->type == PLUGIN_HANDLER_COMMAND)
             ptr_arg1 = ptr_handler->command;
         
         if ((ptr_arg1)
@@ -327,7 +475,7 @@ weechat_script_remove_timer_handler (t_weechat_plugin *plugin,
     ptr_handler = plugin->handlers;
     while (ptr_handler)
     {
-        if ((ptr_handler->type == HANDLER_TIMER)
+        if ((ptr_handler->type == PLUGIN_HANDLER_TIMER)
             && ((t_plugin_script *)ptr_handler->handler_pointer == script)
             && (plugin->ascii_strcasecmp (plugin, ptr_handler->handler_args, function) == 0))
         {
@@ -355,7 +503,7 @@ weechat_script_remove_keyboard_handler (t_weechat_plugin *plugin,
     ptr_handler = plugin->handlers;
     while (ptr_handler)
     {
-        if ((ptr_handler->type == HANDLER_KEYBOARD)
+        if ((ptr_handler->type == PLUGIN_HANDLER_KEYBOARD)
             && ((t_plugin_script *)ptr_handler->handler_pointer == script)
             && (plugin->ascii_strcasecmp (plugin, ptr_handler->handler_args, function) == 0))
         {
@@ -368,6 +516,51 @@ weechat_script_remove_keyboard_handler (t_weechat_plugin *plugin,
     }
 }
 
+/*
+ * weechat_script_remove_modifier: remove a modifier
+ *                                 arg1=type, arg2=command, arg3=function
+ */
+
+void
+weechat_script_remove_modifier (t_weechat_plugin *plugin,
+                                t_plugin_script *script,
+                                char *arg1, char *arg2, char *arg3)
+{
+    t_plugin_modifier *ptr_modifier, *next_modifier;
+    t_plugin_modifier_type type;
+    char *ptr_arg2;
+
+    if (strcasecmp (arg1, PLUGIN_MODIFIER_IRC_IN_STR) == 0)
+        type = PLUGIN_MODIFIER_IRC_IN;
+    else if (strcasecmp (arg1, PLUGIN_MODIFIER_IRC_USER_STR) == 0)
+        type = PLUGIN_MODIFIER_IRC_USER;
+    else if (strcasecmp (arg1, PLUGIN_MODIFIER_IRC_OUT_STR) == 0)
+        type = PLUGIN_MODIFIER_IRC_OUT;
+    else
+        return;
+    
+    /* search and remove modifiers */
+    ptr_modifier = plugin->modifiers;
+    while (ptr_modifier)
+    {
+        ptr_arg2 = NULL;
+        if (ptr_modifier->type == type)
+            ptr_arg2 = ptr_modifier->command;
+        
+        if ((ptr_arg2)
+            && ((t_plugin_script *)ptr_modifier->modifier_pointer == script)
+            && (plugin->ascii_strcasecmp (plugin, ptr_arg2, arg2) == 0)
+            && (plugin->ascii_strcasecmp (plugin, ptr_modifier->modifier_args, arg3) == 0))
+        {
+            next_modifier = ptr_modifier->next_modifier;
+            plugin->modifier_remove (plugin, ptr_modifier);
+            ptr_modifier = next_modifier;
+        }
+        else
+            ptr_modifier = ptr_modifier->next_modifier;
+    }
+}
+
 /*
  * weechat_script_get_plugin_config: get a value of a script option
  *                                   format in file is: plugin.script.option=value
@@ -422,3 +615,21 @@ weechat_script_set_plugin_config (t_weechat_plugin *plugin,
     
     return return_code;
 }
+
+/*
+ * weechat_script_set_charset: set charset for script
+ */
+
+void
+weechat_script_set_charset (t_weechat_plugin *plugin,
+                            t_plugin_script *script,
+                            char *charset)
+{
+    /* make gcc happy */
+    (void) plugin;
+    
+    if (script->charset)
+        free (script->charset);
+    
+    script->charset = (charset) ? strdup (charset) : NULL;
+}
