@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2007 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,10 @@
 #ifndef __WEECHAT_WEECHAT_SCRIPT_H
 #define __WEECHAT_WEECHAT_SCRIPT_H 1
 
+/* constants which defines return types for weechat_<lang>_exec functions */
+#define SCRIPT_EXEC_INT    1
+#define SCRIPT_EXEC_STRING 2
+
 typedef struct t_plugin_script t_plugin_script;
 
 struct t_plugin_script
@@ -34,6 +38,7 @@ struct t_plugin_script
     char *description;                  /* plugin description                  */
     char *version;                      /* plugin version                      */
     char *shutdown_func;                /* function when script is unloaded    */
+    char *charset;                      /* script charset                      */
     
     t_plugin_script *prev_script;       /* link to previous script             */
     t_plugin_script *next_script;       /* link to next script                 */
@@ -47,9 +52,24 @@ extern char *weechat_script_search_full_name (t_weechat_plugin *,
                                               char *, char *);
 extern t_plugin_script *weechat_script_add (t_weechat_plugin *,
                                             t_plugin_script **, char *, char *,
-                                            char *, char *, char *);
+                                            char *, char *, char *, char *);
 extern void weechat_script_remove (t_weechat_plugin *,
                                    t_plugin_script **, t_plugin_script *);
+extern void weechat_script_print (t_weechat_plugin *,
+                                  t_plugin_script *,
+                                  char *, char *, char *, ...);
+extern void weechat_script_print_server (t_weechat_plugin *,
+                                         t_plugin_script *,
+                                         char *, ...);
+extern void weechat_script_print_infobar (t_weechat_plugin *,
+                                          t_plugin_script *,
+                                          int, char *, ...);
+extern void weechat_script_log (t_weechat_plugin *,
+                                t_plugin_script *,
+                                char *, char *, char *, ...);
+extern void weechat_script_exec_command (t_weechat_plugin *,
+                                         t_plugin_script *,
+                                         char *, char *, char *);
 extern void weechat_script_remove_handler (t_weechat_plugin *,
                                            t_plugin_script *,
                                            char *, char *);
@@ -59,11 +79,17 @@ extern void weechat_script_remove_timer_handler (t_weechat_plugin *,
 extern void weechat_script_remove_keyboard_handler (t_weechat_plugin *,
                                                     t_plugin_script *,
                                                     char *);
+extern void weechat_script_remove_modifier (t_weechat_plugin *,
+                                            t_plugin_script *,
+                                            char *, char *, char *);
 extern char *weechat_script_get_plugin_config (t_weechat_plugin *,
                                                t_plugin_script *,
                                                char *);
 extern int weechat_script_set_plugin_config (t_weechat_plugin *,
                                              t_plugin_script *,
                                              char *, char *);
+extern void weechat_script_set_charset (t_weechat_plugin *,
+                                        t_plugin_script *,
+                                        char *);
 
 #endif /* weechat-script.h */
